//  File:        SamInfo.java
//  Copyright:   Copyright (C) 2010 AL ENGINEERING AG. All rights reserved.
//  Website:     http://www.al-ag.de
//  Description:
//  Comments:
//  Last edit:   18.06.2013
//  Author:      AL ENGINEERING AG
//  E-mail:      info@al-ag.de
//
//   This file is part of d(((eti-API.
//
//   d(((eti-API is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   d(((eti-API is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; If not, see <http://www.gnu.org/licenses/> or
//   write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//   Boston, MA 02111-1307 USA.
package de.alag.eticket.ticketinfo;

import de.alag.vdv.ka.cv.CVCertPKCS;
import de.alag.vdv.ka.cv.CVCertISO;
import de.alag.vdv.ka.cv.CAR;
import de.alag.vdv.ka.cv.OID;
import de.alag.vdv.ka.cv.CHA;
import de.alag.vdv.ka.cv.CHR;
import de.alag.util.Converters;
import java.util.Map.Entry;
import java.util.*;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import de.alag.vdv.ka.SymSchluessel;
import de.alag.vdv.ka.nfc.ISAM;

/**
 *
 * @author Alexander Schuetz
 */
public class SamInfo extends javax.swing.JDialog {

    
    /**
     * Creates new form SamInfo
     */
    public SamInfo(java.awt.Frame parent, boolean modal, ISAM sam, String readerName) {
        super(parent, modal);
        initComponents();
        if (sam == null) {
            DefaultMutableTreeNode root = new DefaultMutableTreeNode(readerName);
            DefaultTreeModel model = new DefaultTreeModel(root);
            jTree1.setModel(model);
        } else {
            reload(sam, readerName);
        }
    }

    private DefaultMutableTreeNode getEFSamID(ISAM sam) {
        DefaultMutableTreeNode efSamId = new DefaultMutableTreeNode("SAM ID");
        
        DefaultMutableTreeNode nr = new DefaultMutableTreeNode("Nr: " + Integer.toString(sam.getSamNr()));
        DefaultMutableTreeNode nrHex = new DefaultMutableTreeNode("Hex: " + Integer.toHexString(sam.getSamNr()));
        nr.add(nrHex);
        efSamId.add(nr);
        
        DefaultMutableTreeNode orgid = new DefaultMutableTreeNode("OrgID: " + Integer.toString(sam.getOrgId()));
        DefaultMutableTreeNode orgidHex = new DefaultMutableTreeNode("Hex: " + Integer.toHexString(sam.getOrgId()));
        orgid.add(orgidHex);
        efSamId.add(orgid);
        
        DefaultMutableTreeNode von = new DefaultMutableTreeNode("Gültigkeitsbeginn: " + sam.getSamId().getGueltigkeitsbeginn().toString());
        DefaultMutableTreeNode vonHex = new DefaultMutableTreeNode("Hex: " + Converters.bytesToHexString(sam.getSamId().getGueltigkeitsbeginn().toRaw()));
        von.add(vonHex);
        efSamId.add(von);
        
        DefaultMutableTreeNode bis = new DefaultMutableTreeNode("Verfallsdatum: " + sam.getSamId().getVerfalldatum().toString());
        DefaultMutableTreeNode bisHex = new DefaultMutableTreeNode("Hex: " + Converters.bytesToHexString(sam.getSamId().getVerfalldatum().toRaw()));
        bis.add(bisHex);
        efSamId.add(bis);
        
        return efSamId;
    }
    
    private DefaultMutableTreeNode getKeys(DefaultMutableTreeNode subRoot, Set<SymSchluessel> keys) {
        HashMap<Integer, List<SymSchluessel>> keyMap = new HashMap<Integer, List<SymSchluessel>>();
        for (SymSchluessel key : keys) {
            List<SymSchluessel> tmpList = keyMap.get(key.GetOrgId());
            if (tmpList == null) {
                tmpList = new LinkedList<SymSchluessel>();
                keyMap.put(key.GetOrgId(), tmpList);
            }
            tmpList.add(key);
        }
        
        for (Entry<Integer, List<SymSchluessel>> e : keyMap.entrySet()) {
            DefaultMutableTreeNode orgId = new DefaultMutableTreeNode(Integer.toString(e.getKey()));
            for (SymSchluessel k : e.getValue()) {
                DefaultMutableTreeNode keyVer = new DefaultMutableTreeNode("Version: " + Converters.byteToHexString(k.GetVersion()));
                DefaultMutableTreeNode keyCnt = new DefaultMutableTreeNode("Zähler: " + Long.toString(k.getCounter()));
                DefaultMutableTreeNode keyLmt = new DefaultMutableTreeNode("Limit: " + Long.toString(k.getLimit()));
                DefaultMutableTreeNode keyHex = new DefaultMutableTreeNode("Hex: " + Converters.bytesToHexString(k.toRaw()));
                keyVer.add(keyCnt);
                keyVer.add(keyLmt);
                keyVer.add(keyHex);
                orgId.add(keyVer);
            }
            subRoot.add(orgId);
        }
        return subRoot;
    }
    
    private DefaultMutableTreeNode getCert(DefaultMutableTreeNode subRoot, CVCertISO cert) {
        DefaultMutableTreeNode tag1 = new DefaultMutableTreeNode("ISO-Data");
        DefaultMutableTreeNode tag1Hex = new DefaultMutableTreeNode("Hex: " + Converters.bytesToHexString(cert.get5F375F38()));
        DefaultMutableTreeNode tag2 = new DefaultMutableTreeNode("Remainder");
        DefaultMutableTreeNode tag2Hex = new DefaultMutableTreeNode("Hex: " + Converters.bytesToHexString(cert.getRemainder()));
        DefaultMutableTreeNode tag3 = new DefaultMutableTreeNode("Signature");
        DefaultMutableTreeNode tag3Hex = new DefaultMutableTreeNode("Hex: " + Converters.bytesToHexString(cert.getSignature()));
        tag1.add(tag1Hex);
        tag2.add(tag2Hex);
        tag3.add(tag3Hex);
        subRoot.add(tag1);
        subRoot.add(tag2);
        subRoot.add(tag3);
        return subRoot;
    }
    
    private DefaultMutableTreeNode getCAR(CAR car) {
        DefaultMutableTreeNode carTag = new DefaultMutableTreeNode("CAR");
        DefaultMutableTreeNode carAuth = new DefaultMutableTreeNode("Authority: " + car.getAuthority());
        DefaultMutableTreeNode carCountry = new DefaultMutableTreeNode("Countrycode: " + car.getCountryCode());
        DefaultMutableTreeNode carCA = new DefaultMutableTreeNode("CA: " + Integer.toString(car.getCA()));
        DefaultMutableTreeNode carYear = new DefaultMutableTreeNode("Key creation year: " + Integer.toString(car.getKeyCreationYear()));
        DefaultMutableTreeNode carDis = new DefaultMutableTreeNode("Discretionary data: " + car.getDiscretionaryData().name());
        DefaultMutableTreeNode carSi = new DefaultMutableTreeNode("Service indicator: " + car.getServiceIndicator().name());
        DefaultMutableTreeNode carHex = new DefaultMutableTreeNode("Hex: " + Converters.bytesToHexString(car.toRaw()));
        carTag.add(carAuth);
        carTag.add(carCountry);
        carTag.add(carCA);
        carTag.add(carYear);
        carTag.add(carDis);
        carTag.add(carSi);
        carTag.add(carHex);
        return carTag;
    }
    
    private DefaultMutableTreeNode getCHR(CHR chr) {
        DefaultMutableTreeNode chrTag = new DefaultMutableTreeNode("CHR");
        DefaultMutableTreeNode chrTyp = new DefaultMutableTreeNode("Type: " + chr.getType().name());
        DefaultMutableTreeNode chrcar = getCAR(chr.getCACAR());
        DefaultMutableTreeNode chrActGen = new DefaultMutableTreeNode("Act-Key generation: " + Integer.toString(chr.getActKeyGeneration()));
        DefaultMutableTreeNode chrSigGen = new DefaultMutableTreeNode("Sig-Key generation: " + Integer.toString(chr.getSigKeyGeneration()));
        DefaultMutableTreeNode chrOrg = new DefaultMutableTreeNode("OrgID: " + Integer.toString(chr.getOrgId()));
        DefaultMutableTreeNode chrOrtt = new DefaultMutableTreeNode("Orttyp: " + Integer.toString(chr.getTagOrttypCode()));
        DefaultMutableTreeNode chrOrtn = new DefaultMutableTreeNode("Ortnr: " + Integer.toString(chr.getTagOrtNr()));
        DefaultMutableTreeNode chrTagVon = new DefaultMutableTreeNode("Tag-Cert. gültig ab: " + chr.getTagCertValidityStart().toString());
        DefaultMutableTreeNode chrNmVon = new DefaultMutableTreeNode("NM-Cert. gültig ab: " + chr.getNmCertValidityStart().toString());
        DefaultMutableTreeNode chrApp = new DefaultMutableTreeNode("NM-Appinstanznr: " + Long.toString(chr.getNmAppInstanzNr()));
        DefaultMutableTreeNode chrHex = new DefaultMutableTreeNode("Hex: " + Converters.bytesToHexString(chr.toRaw()));
        chrTag.add(chrTyp);
        chrTag.add(chrcar);
        chrTag.add(chrActGen);
        chrTag.add(chrSigGen);
        chrTag.add(chrOrg);
        chrTag.add(chrOrtt);
        chrTag.add(chrOrtn);
        chrTag.add(chrTagVon);
        chrTag.add(chrNmVon);
        chrTag.add(chrApp);
        chrTag.add(chrHex);
        return chrTag;
    }
    
    private DefaultMutableTreeNode getCHA(CHA cha) {
        DefaultMutableTreeNode chaTag = new DefaultMutableTreeNode("CHA");
        DefaultMutableTreeNode chaHld = new DefaultMutableTreeNode("Name: " + cha.getHolder());
        DefaultMutableTreeNode chaX = new DefaultMutableTreeNode("X: " + cha.getX().name());
        DefaultMutableTreeNode chaY = new DefaultMutableTreeNode("Y: " + cha.getY().name());
        DefaultMutableTreeNode chaHex = new DefaultMutableTreeNode("Hex: " + Converters.bytesToHexString(cha.toRaw()));
        chaTag.add(chaHld);
        chaTag.add(chaX);
        chaTag.add(chaY);
        chaTag.add(chaHex);
        return chaTag;
    }
    
    private DefaultMutableTreeNode getOID(OID oid) {
        DefaultMutableTreeNode oidTag = new DefaultMutableTreeNode("OID");
        DefaultMutableTreeNode oidS = new DefaultMutableTreeNode("Size: " + Integer.toString(oid.getSize()));
        DefaultMutableTreeNode oidHex = new DefaultMutableTreeNode("Hex: " + Converters.bytesToHexString(oid.toRaw()));
        oidTag.add(oidS);
        oidTag.add(oidHex);
        return oidTag;
    }
    
    private DefaultMutableTreeNode getCert(DefaultMutableTreeNode subRoot, CVCertPKCS cert) {
        DefaultMutableTreeNode data = new DefaultMutableTreeNode("CV-Data");
        data.add(getCAR(cert.getCVData().getCar()));
        data.add(getCHR(cert.getCVData().getChr()));
        data.add(getCHA(cert.getCVData().getCha()));
        data.add(new DefaultMutableTreeNode("CPI: " + cert.getCVData().getCpi().name()));
        data.add(getOID(cert.getCVData().getOid()));
        data.add(new DefaultMutableTreeNode("End of validity: " + cert.getCVData().getEov().toString()));
        data.add(new DefaultMutableTreeNode("Modulus: " + Converters.bytesToHexString(cert.getCVData().getModulus())));
        data.add(new DefaultMutableTreeNode("Pub-Exponent: " + Converters.bytesToHexString(cert.getCVData().getkPubExp())));
        subRoot.add(data);
        return subRoot;
    }
    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        jTree1 = new javax.swing.JTree();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        javax.swing.tree.DefaultMutableTreeNode treeNode1 = new javax.swing.tree.DefaultMutableTreeNode("root");
        jTree1.setModel(new javax.swing.tree.DefaultTreeModel(treeNode1));
        jScrollPane1.setViewportView(jTree1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 380, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 278, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTree jTree1;
    // End of variables declaration//GEN-END:variables

    public void reload(ISAM sam, String samReaderName) {
        DefaultTreeModel model = (DefaultTreeModel) jTree1.getModel();
        DefaultMutableTreeNode root = (DefaultMutableTreeNode) model.getRoot();
        DefaultMutableTreeNode samRoot = null;
        Enumeration childEnum = root.children();
        while (childEnum.hasMoreElements()) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)childEnum.nextElement();
            if (samReaderName.equals(node.getUserObject())) {
                node.removeAllChildren();
                samRoot = node;
                break;
            }
        }
        if (samRoot == null) {
            samRoot = new DefaultMutableTreeNode(samReaderName);
            model.insertNodeInto(samRoot, root, 0);
        }

        DefaultMutableTreeNode certAuthIso = new DefaultMutableTreeNode("Cert SAM-AUTH-ISO");
        DefaultMutableTreeNode certAuthPkcs = new DefaultMutableTreeNode("Cert SAM-AUTH-PKCS");
        DefaultMutableTreeNode certCAIso = new DefaultMutableTreeNode("Cert SUB-CA-ISO");
        DefaultMutableTreeNode certCAPkcs = new DefaultMutableTreeNode("Cert SUB-CA-PKCS");
        DefaultMutableTreeNode erfnm = new DefaultMutableTreeNode("MK erf-nm-mac");
        DefaultMutableTreeNode kvpnm = new DefaultMutableTreeNode("MK kvp-nm-mac");
        DefaultMutableTreeNode pvnm = new DefaultMutableTreeNode("MK pv-nm-mac");
        DefaultMutableTreeNode dlerf1 = new DefaultMutableTreeNode("MK dl-erf-mac");
        DefaultMutableTreeNode dlerf2 = new DefaultMutableTreeNode("K dl-erf-mac");
        DefaultMutableTreeNode slenc = new DefaultMutableTreeNode("K sperrlisten-enc-mac");

        model.insertNodeInto(getCert(certCAIso, sam.getSubCAISO()), samRoot, 0);
        model.insertNodeInto(getCert(certCAPkcs, sam.getSubCAPKCS()), samRoot, 0);
        model.insertNodeInto(getCert(certAuthIso, sam.getSamAuthISO() ), samRoot, 0);
        model.insertNodeInto(getCert(certAuthPkcs, sam.getSamAuthPKCS() ), samRoot, 0);
        model.insertNodeInto(getKeys(slenc, sam.getKsperrlistenEnc()), samRoot, 0);
        model.insertNodeInto(getKeys(dlerf2, sam.getKdlerfmac()), samRoot, 0);
        model.insertNodeInto(getKeys(dlerf1, sam.getMKdlerfmac()), samRoot, 0);
        model.insertNodeInto(getKeys(pvnm, sam.getMKpvnmmac()), samRoot, 0);
        model.insertNodeInto(getKeys(kvpnm, sam.getMKkvpnmmac()), samRoot, 0);
        model.insertNodeInto(getKeys(erfnm, sam.getMKerfnmmac()), samRoot, 0);
        model.insertNodeInto(new DefaultMutableTreeNode("Loadkey-Zähler: " + Long.toString(sam.getLoadkeyCounter(false))), samRoot, 0);
        model.insertNodeInto(new DefaultMutableTreeNode("Transaktion-Zähler: " + Long.toString(sam.getTransactionCounter(false))), samRoot, 0);
        model.insertNodeInto(new DefaultMutableTreeNode("Aktivierungsstatus: " + (sam.isActivated(false) ? "ok" : "inaktiv")), samRoot, 0);
        model.insertNodeInto(getEFSamID(sam), samRoot, 0);
        
        model.reload();
    }
}
