//  File:        Ticketinfo.java
//  Copyright:   Copyright (C) 2010 AL ENGINEERING AG. All rights reserved.
//  Website:     http://www.al-ag.de
//  Description:
//  Comments:
//  Last edit:   18.06.2013
//  Author:      AL ENGINEERING AG
//  E-mail:      info@al-ag.de
//
//   This file is part of d(((eti-API.
//
//   d(((eti-API is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   d(((eti-API is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; If not, see <http://www.gnu.org/licenses/> or
//   write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//   Boston, MA 02111-1307 USA.
package de.alag.eticket.ticketinfo;

import de.alag.eticket.ticketinfo.helpers.MultiSelectTable;
import de.alag.eticket.ticketinfo.helpers.UmbruchTableCellRenderer;
import de.alag.eticket.ticketinfo.helpers.SaveLoadFile;
import de.alag.eticket.ticketinfo.helpers.NormalTableCellRenderer;
import de.alag.eticket.ticketinfo.helpers.SimpleFileFilter;
import de.alag.eticket.ticketinfo.helpers.ConfigFileLoader;
import de.alag.util.FileLogManager;
import de.alag.util.Pair;
import de.alag.util.nfc.ISmartcard;
import de.alag.util.nfc.PCSC;
import de.alag.util.nfc.SCConnection;
import de.alag.eticket.data.efm.EfmPvDatei;
import de.alag.eticket.data.efm.IEfmXmlParser;
//import de.alag.eticket.data.efm.SaxEfmXmlParser;
import de.alag.eticket.service.CharGeschlechtInterpreter;
import de.alag.eticket.service.LocalCodeInterpreter;
import de.alag.eticket.service.LocalOrganisationInterpreter;
import de.alag.eticket.service.XmlInterpreter;
import de.alag.eticket.ticketinfo.helpers.ConfigWebLoader;
import de.alag.eticket.ticketinfo.helpers.EfmXmlParser;
import de.alag.eticket.ticketinfo.professional.NMBerechtigung;
import de.alag.eticket.ticketinfo.professional.Nutzermedium;
import de.alag.eticket.ticketinfo.professional.NutzermediumListe;
import de.alag.eticket.ticketinfo.professional.OptionDialog;
import de.alag.eticket.ticketinfo.professional.WaitForCardThread;
import de.alag.util.Converters;
import de.alag.util.nfc.IIso7816_Response;
import java.awt.*;
import java.awt.event.*;
import java.awt.print.PrinterException;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.List;
import java.util.Map.Entry;
import java.util.Queue;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.smartcardio.CardTerminal;
import javax.swing.Timer;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import de.alag.vdv.ka.KAVersion;
import de.alag.vdv.ka.Tarifgebiet;
import de.alag.vdv.ka.Wabe;
import de.alag.vdv.ka.nfc.ISAM;
import de.alag.vdv.ka.nfc.KundenMedium;
import de.alag.vdv.ka.nfc.SAM;
import de.alag.vdv.ka.service.ICodeInterpreter;
import de.alag.vdv.ka.service.IOrganisationInterpreter;
import de.alag.vdv.ka.service.IOrtInterpreter;
import de.alag.vdv.ka.service.IServicekennungInterpreter;
import de.alag.vdv.ka.tlv.TLVParser;
import de.alag.vdv.ka.tlv.Template;
import de.alag.vdv.ka.tlv.TemplateProvider;
import de.alag.vdv.ka.tlv.bom.DateTimeCompact;
import de.alag.vdv.ka.tlv.directory.Applikationsverzeichnis;
import de.alag.vdv.ka.tlv.entitlement.Berechtigung;
import de.alag.vdv.ka.tlv.entitlement.SeparateDatenBerechtigung;
import de.alag.vdv.ka.tlv.global.EfmProduktID;
import de.alag.vdv.ka.tlv.global.NmApplikationInstanzID;
import de.alag.vdv.ka.tlv.log.SeparateDatenApplikationslogbuch;
import de.alag.vdv.ka.tlv.global.OrtID;
import de.alag.vdv.ka.tlv.templates.efs.DatenPSTeilEfs;
import de.alag.vdv.ka.tlv.templates.nrw.DatenPSTeilNrw;
import de.alag.vdv.ka.tlv.transaction.Transaktion;
import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.net.URI;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.xml.bind.JAXB;

/**
 * Viewer MainWindow. Und Startklasse.
 * @author Frank Hamm, Alexander Schuetz
 */
public class Ticketinfo extends javax.swing.JFrame {

    private static final Logger LOGGER = Logger.getLogger(Ticketinfo.class.getName());
    private ISmartcard sci;
    private KundenMedium kum;
    private Applikationsverzeichnis app;
    private boolean appOK = false;
    private boolean keystoreOK = false;
    private boolean custOK = false;
    private boolean logOK = false;
    private IEfmXmlParser saxParser;
    private NmApplikationInstanzID authCardId = null;
    private boolean pinOK = false;
    private FileFilter nmFileFilter = new SimpleFileFilter("Ticketinfo-Datei (*.tid)", "tid");
    private FileFilter utfFileFilter = new SimpleFileFilter("Text UTF-8 (*.txt)", "txt");
//    private FileFilter excelFileFilter = new SimpleFileFilter("Excel (*.xls)", "xls");
    
    private NutzermediumListe nmListe = new NutzermediumListe();
    
    private int datIndex = 0;
    private int nmIndex = 0;
    private int nmAnzeigeIndex = 0;
    
    private Nutzermedium musterNM;
    private int musterNMeinlesen = 0;
    
    private long lastticket;
    private long lastread;
    
    private WaitForCardThread wfc;

    public enum CONFIG {
        LogFile,
        LogLevel,
        UpdateUrl,
        LoggerState,
        KumReader,
        SamReader,
        EfmPath,
        Templates,
        einzelneDatei,
        einzelneNM,
        einzelneBer,
        PraefixDatei,
        PraefixNM,
        PraefixBer,
        StellenNM,
        StellenBer,
        Trennzeichen1,
        Trennzeichen2,
        Pfad,
        Ton,
        FehlerVDVKA,
        FehlerTarif,
        FehlerVergleich
    };

    public static Properties getDefaultConfig() {
        Properties defaultCfg = new Properties();
        defaultCfg.setProperty(CONFIG.LogFile.name(), System.getProperty("user.home") + File.separator + "Ticketinfo.log");
        defaultCfg.setProperty(CONFIG.LogLevel.name(), Level.CONFIG.getName());
        defaultCfg.setProperty(CONFIG.UpdateUrl.name(), "http://www.ticket-info.net/");
        defaultCfg.setProperty(CONFIG.LoggerState.name(), Boolean.toString(false));
        defaultCfg.setProperty(CONFIG.KumReader.name(), "");
        defaultCfg.setProperty(CONFIG.SamReader.name(), "");
        defaultCfg.setProperty(CONFIG.EfmPath.name(), ".");
        //TemplateProvider.getInstance().addProduct("NRW", new EfmProduktID(0, 42));
        //TemplateProvider.getInstance().addProduct("NRW", new EfmProduktID(0, 70));
        //TemplateProvider.getInstance().addProduct("NRW", new EfmProduktID(0, 72));
        //TemplateProvider.getInstance().addProduct("NRW", new EfmProduktID(0, 77));
        //TemplateProvider.getInstance().addProduct("NRW", new EfmProduktID(0, 102));
        //TemplateProvider.getInstance().addProduct("NRW", new EfmProduktID(0, 104));
        //TemplateProvider.getInstance().addProduct("NRW", new EfmProduktID(0, 151));
        defaultCfg.setProperty(CONFIG.Templates.name(), "NRW;0000002A;00000046;00000048;00000066;00000068;00000097\n"
                + "TLVEFS;0000004D\n"
                + "AFB;232A004D;232B004D");
        
        defaultCfg.setProperty(CONFIG.einzelneDatei.name(), Boolean.toString(true));
        defaultCfg.setProperty(CONFIG.einzelneNM.name(), Boolean.toString(true));
        defaultCfg.setProperty(CONFIG.einzelneBer.name(), Boolean.toString(true));
        defaultCfg.setProperty(CONFIG.PraefixDatei.name(), "NMListe");
        defaultCfg.setProperty(CONFIG.PraefixNM.name(), "Karte");
        defaultCfg.setProperty(CONFIG.PraefixBer.name(), "Berechtigung");
        defaultCfg.setProperty(CONFIG.StellenNM.name(), "3");
        defaultCfg.setProperty(CONFIG.StellenBer.name(), "2");
        defaultCfg.setProperty(CONFIG.Trennzeichen1.name(), "");
        defaultCfg.setProperty(CONFIG.Trennzeichen2.name(), "-");
        defaultCfg.setProperty(CONFIG.Pfad.name(), "/tmp/");
        defaultCfg.setProperty(CONFIG.Ton.name(), Boolean.toString(true));
        defaultCfg.setProperty(CONFIG.FehlerVDVKA.name(), Boolean.toString(true));
        defaultCfg.setProperty(CONFIG.FehlerTarif.name(), Boolean.toString(true));
        defaultCfg.setProperty(CONFIG.FehlerVergleich.name(), Boolean.toString(true));
        return defaultCfg;
    }

    private String getConfig(CONFIG cfg) {
        return mProperties.getProperty(cfg.name());
    }
    
    private boolean getConfigBoolean(CONFIG cfg) {
        String tmp = mProperties.getProperty(cfg.name());
        if (tmp != null) {
            return Boolean.parseBoolean(tmp);
        }
        return false;
    }
    
    private int getConfigInt(CONFIG cfg) {
        String tmp = mProperties.getProperty(cfg.name());
        if (tmp != null) {
            return Integer.parseInt(tmp);
        }
        return 0;
    }
    
    public static String getConfig(Properties p, CONFIG cfg) {
        return p.getProperty(cfg.name());
    }
    
    public static boolean getConfigBoolean(Properties p, CONFIG cfg) {
        String tmp = p.getProperty(cfg.name());
        if (tmp != null) {
            return Boolean.parseBoolean(tmp);
        }
        return false;
    }
    
    public static int getConfigInt(Properties p, CONFIG cfg) {
        String tmp = p.getProperty(cfg.name());
        if (tmp != null) {
            return Integer.parseInt(tmp);
        }
        return 0;
    }

    public static void setConfig(Properties p, CONFIG cfg, String value) {
        p.put(cfg.name(), value);
    }
    
    public static void setConfig(Properties p, CONFIG cfg, boolean b) {
        p.put(cfg.name(), Boolean.toString(b));
    }
    
    public static void setConfig(Properties p, CONFIG cfg, int i) {
        p.put(cfg.name(), Integer.toString(i));
    }
    
    private void setConfig(CONFIG cfg, String value) {
        mProperties.put(cfg.name(), value);
    }

    private String getConfig(CONFIG cfg, String def) {
        String res = mProperties.getProperty(cfg.name());
        if (res != null) {
            return res;
        }
        return null;
    }

    /**
     * Creates new form NewJFrame.
     */
    public Ticketinfo(IConfigurator configurator) {
        this.mConfigurator = configurator;
        
        Applikationsverzeichnis.setInterpreter(this.mConfigurator.getCodeInterpreter());
        Applikationsverzeichnis.setInterpreter(this.mConfigurator.getOrganisationInterpreter());
        Applikationsverzeichnis.setInterpreter(this.mConfigurator.getOrtInterpreter());
        Applikationsverzeichnis.setInterpreter(this.mConfigurator.getServicekennungInterpreter());
        //ClassPathXmlApplicationContext ctx = new ClassPathXmlApplicationContext("de/alag/eticket/ticketinfo/appContext.xml");
        //Applikationsverzeichnis.setInterpreter(ctx.getBean(ICodeInterpreter.class));
        //Applikationsverzeichnis.setInterpreter(ctx.getBean(IOrganisationInterpreter.class));
        //Applikationsverzeichnis.setInterpreter(ctx.getBean(IOrtInterpreter.class));
        //Applikationsverzeichnis.setInterpreter(ctx.getBean(IServicekennungInterpreter.class));
        
        Logger.getLogger("").setLevel(Level.ALL);
        FileLogManager.getInstance().registerLogger(LOGGER);
        String lookAndFeel = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";
        try {
            UIManager.setLookAndFeel(lookAndFeel);
        } catch (Exception ex) {
            LOGGER.log(Level.WARNING, "{0} konnte nicht geladen werden. Das default LookAndFeel wird verwendet.", lookAndFeel);
        }
        initComponents();
        loadPreferences();
        // Dateien Index suchen
        String datPrae = mProperties.getProperty("PraefixDatei") + mProperties.getProperty("Trennzeichen1");
        String tckPrae = mProperties.getProperty("PraefixNM") + mProperties.getProperty("Trennzeichen1");
        
        int datST = 3;
        int tckST = Integer.parseInt(mProperties.getProperty("StellenNM"));
        
        try {
            File dir = new File(mProperties.getProperty("Pfad"));
            File[] fileList = dir.listFiles();

            for(File f: fileList) {
                String fn = f.getName();
                if(fn.startsWith(datPrae)) {
                    int iBegin = datPrae.length();
                    int iEnd = datPrae.length() + datST;
                    String s_datLfdnr = fn.substring(iBegin, iEnd);
                    try { 
                        int i_datLfdnr = Integer.parseInt(s_datLfdnr);
                        if (i_datLfdnr > datIndex)
                            datIndex = i_datLfdnr;
                    } catch (Exception ex) {}
                }
                if(fn.startsWith(tckPrae)) {
                    int iBegin = tckPrae.length();
                    int iEnd = tckPrae.length() + tckST;
                    String s_tckLfdnr = fn.substring(iBegin, iEnd);
                    try {
                        int i_tckLfdnr = Integer.parseInt(s_tckLfdnr);
                        if (i_tckLfdnr > nmIndex)
                            nmIndex = i_tckLfdnr;
                    } catch (Exception ex) {}    
                }
            }
        } catch(Exception ex) {}
        
        datIndex++;
        nmIndex++;
        this.grundeinstellung();
        //saxParser = new SaxEfmXmlParser("[a-zA-Z]{3}-[eE][fF][mM]-[dD]aten-[0-9]{4}-[0-9]{2}-[0-9]{2}.[xX][mM][lL]", null);
        saxParser = this.mConfigurator.getParser();
        saxParser.FindEfmXmlFiles(efmFilePfad);
        app = new Applikationsverzeichnis();

        sci = new SCConnection(null);
        kum = new KundenMedium(sci);
        sam = null;

        readerRefreshButtonActionPerformed(null);
        buildTemplateDropDown();
        ((MultiSelectTable)expertTable).setSelectEnabled(true);
        checkGUI();
        wfc = new WaitForCardThread(sci);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        saveFileChooser = new javax.swing.JFileChooser();
        loadFileChooser = new javax.swing.JFileChooser();
        folderChooser = new javax.swing.JFileChooser();
        professionalPanel = new javax.swing.JPanel();
        profBottomPanel = new javax.swing.JPanel();
        labelLabel = new javax.swing.JLabel();
        jScrollPane2 = new javax.swing.JScrollPane();
        textAreaLabel = new javax.swing.JTextArea();
        buttonAktualisieren = new javax.swing.JButton();
        profSplittPane = new javax.swing.JSplitPane();
        upperScrollPane = new javax.swing.JScrollPane();
        lowerScrollPane = new javax.swing.JScrollPane();
        errorTextArea = new javax.swing.JTextArea();
        expertTable = new MultiSelectTable();
        mainPanel = new javax.swing.JScrollPane();
        normalTable = new MultiSelectTable();
        statusPanel = new javax.swing.JPanel();
        sucheProgressBar = new javax.swing.JProgressBar();
        currentItemLabel = new javax.swing.JLabel();
        jButtonBackward = new javax.swing.JButton();
        jButtonForward = new javax.swing.JButton();
        readParadigmLabel = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        jToolBar1 = new javax.swing.JToolBar();
        readButton = new javax.swing.JButton();
        readSamButton = new javax.swing.JButton();
        loadKeyButton = new javax.swing.JButton();
        saveButton = new javax.swing.JButton();
        ladenButton = new javax.swing.JButton();
        upButton = new javax.swing.JButton();
        downButton = new javax.swing.JButton();
        druckenButton = new javax.swing.JButton();
        hilfeButton = new javax.swing.JButton();
        readerRefreshButton = new javax.swing.JButton();
        exitButton = new javax.swing.JButton();
        logoALLabel = new javax.swing.JLabel();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        readMenuItem = new javax.swing.JMenuItem();
        readSamMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JPopupMenu.Separator();
        druckMenuItem = new javax.swing.JMenuItem();
        speicherMenuItem = new javax.swing.JMenuItem();
        ladenMenuItem = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JPopupMenu.Separator();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        ansichtMenu = new javax.swing.JMenu();
        expertRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        jCheckBoxMenuItem_showCustomer = new javax.swing.JCheckBoxMenuItem();
        schriftMenu = new javax.swing.JMenu();
        schriftGroesserMenuItem = new javax.swing.JMenuItem();
        schriftKleinerMenuItem = new javax.swing.JMenuItem();
        optionMenu = new javax.swing.JMenu();
        readerMenu = new javax.swing.JMenu();
        efmFolderMenuItem = new javax.swing.JMenuItem();
        TemplatesMenu = new javax.swing.JMenu();
        jMenuItem1 = new javax.swing.JMenuItem();
        optionSeparator = new javax.swing.JPopupMenu.Separator();
        professionalEnableMenuItem = new javax.swing.JCheckBoxMenuItem();
        checkBoxLog = new javax.swing.JCheckBoxMenuItem();
        professionalMenu = new javax.swing.JMenu();
        readParadigmMenuItem = new javax.swing.JMenuItem();
        stopNMCompareMenuItem = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JPopupMenu.Separator();
        loadParadigmMenuItem = new javax.swing.JMenuItem();
        saveParadigmMenuItem = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JPopupMenu.Separator();
        samReaderMenu = new javax.swing.JMenu();
        profPropertiesMenuItem = new javax.swing.JMenuItem();
        help_aboutMenu = new javax.swing.JMenu();
        helpMenuItem = new javax.swing.JMenuItem();
        updateMenuItem = new javax.swing.JMenuItem();
        aboutMenuItem = new javax.swing.JMenuItem();

        folderChooser.setFileFilter(null);
        folderChooser.setFileSelectionMode(javax.swing.JFileChooser.DIRECTORIES_ONLY);
        folderChooser.setName("EFM-Verzeichnis"); // NOI18N

        professionalPanel.setLayout(new java.awt.BorderLayout());

        profBottomPanel.setPreferredSize(new java.awt.Dimension(843, 130));
        profBottomPanel.setLayout(new java.awt.BorderLayout());

        labelLabel.setText("Etikettierungstext:");
        profBottomPanel.add(labelLabel, java.awt.BorderLayout.PAGE_START);

        textAreaLabel.setColumns(20);
        textAreaLabel.setRows(5);
        jScrollPane2.setViewportView(textAreaLabel);

        profBottomPanel.add(jScrollPane2, java.awt.BorderLayout.CENTER);

        buttonAktualisieren.setText("Aktualisieren");
        buttonAktualisieren.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonAktualisierenActionPerformed(evt);
            }
        });
        profBottomPanel.add(buttonAktualisieren, java.awt.BorderLayout.EAST);

        professionalPanel.add(profBottomPanel, java.awt.BorderLayout.SOUTH);

        profSplittPane.setDividerLocation(210);
        profSplittPane.setDividerSize(2);
        profSplittPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        profSplittPane.setMaximumSize(null);
        profSplittPane.setPreferredSize(new java.awt.Dimension(820, 313));

        upperScrollPane.setPreferredSize(null);
        profSplittPane.setLeftComponent(upperScrollPane);

        errorTextArea.setColumns(20);
        errorTextArea.setForeground(new java.awt.Color(255, 0, 0));
        errorTextArea.setRows(5);
        errorTextArea.setMaximumSize(null);
        errorTextArea.setMinimumSize(null);
        lowerScrollPane.setViewportView(errorTextArea);

        profSplittPane.setRightComponent(lowerScrollPane);

        professionalPanel.add(profSplittPane, java.awt.BorderLayout.CENTER);

        expertTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {

            }
        ));
        expertTable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_NEXT_COLUMN);
        expertTable.setFillsViewportHeight(true);
        expertTable.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        expertTable.setShowHorizontalLines(false);
        expertTable.setShowVerticalLines(false);
        expertTable.getTableHeader().setReorderingAllowed(false);
        expertTable.setUpdateSelectionOnSort(false);
        expertTable.setVerifyInputWhenFocusTarget(false);

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("eTicketinfo");
        setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));

        mainPanel.setBackground(new java.awt.Color(255, 255, 255));

        normalTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {

            }
        ));
        normalTable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_NEXT_COLUMN);
        normalTable.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        normalTable.setFillsViewportHeight(true);
        normalTable.setRowSelectionAllowed(false);
        normalTable.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        normalTable.setShowHorizontalLines(false);
        normalTable.setShowVerticalLines(false);
        normalTable.getTableHeader().setReorderingAllowed(false);
        normalTable.setVerifyInputWhenFocusTarget(false);
        mainPanel.setViewportView(normalTable);

        getContentPane().add(mainPanel, java.awt.BorderLayout.CENTER);

        statusPanel.setMaximumSize(new java.awt.Dimension(32767, 23));
        statusPanel.setMinimumSize(new java.awt.Dimension(200, 23));
        statusPanel.setPreferredSize(new java.awt.Dimension(843, 23));

        sucheProgressBar.setMaximumSize(new java.awt.Dimension(146, 24));
        sucheProgressBar.setMinimumSize(new java.awt.Dimension(146, 24));
        sucheProgressBar.setPreferredSize(new java.awt.Dimension(146, 24));
        sucheProgressBar.setString("Suche Karte");

        currentItemLabel.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
        currentItemLabel.setText("0/0");

        jButtonBackward.setText("zurück");
        jButtonBackward.setAlignmentX(0.5F);
        jButtonBackward.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonBackwardActionPerformed(evt);
            }
        });

        jButtonForward.setText("vorwärts");
        jButtonForward.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonForwardActionPerformed(evt);
            }
        });

        readParadigmLabel.setForeground(new java.awt.Color(255, 0, 0));
        readParadigmLabel.setText("Mustermedium auslesen");

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addComponent(sucheProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, 363, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 67, Short.MAX_VALUE)
                .addComponent(readParadigmLabel)
                .addGap(121, 121, 121)
                .addComponent(currentItemLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButtonBackward, javax.swing.GroupLayout.PREFERRED_SIZE, 77, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButtonForward))
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addGap(0, 0, 0)
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jButtonForward)
                        .addComponent(jButtonBackward)
                        .addComponent(currentItemLabel)
                        .addComponent(readParadigmLabel))
                    .addComponent(sucheProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );

        getContentPane().add(statusPanel, java.awt.BorderLayout.PAGE_END);

        jPanel1.setPreferredSize(new java.awt.Dimension(820, 61));

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);

        readButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/read.png"))); // NOI18N
        readButton.setText("Lesen");
        readButton.setFocusable(false);
        readButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        readButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        readButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                readButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(readButton);

        readSamButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/reads.png"))); // NOI18N
        readSamButton.setText("SAM-Info");
        readSamButton.setFocusable(false);
        readSamButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        readSamButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        readSamButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                readSamButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(readSamButton);
        readSamButton.getAccessibleContext().setAccessibleName("SAMInfo");

        loadKeyButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/loadKey.png"))); // NOI18N
        loadKeyButton.setText("Load-Key");
        loadKeyButton.setFocusable(false);
        loadKeyButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        loadKeyButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        loadKeyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadKeyButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(loadKeyButton);

        saveButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/document-save-as.png"))); // NOI18N
        saveButton.setText("Speichern");
        saveButton.setFocusable(false);
        saveButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        saveButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        saveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(saveButton);

        ladenButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/document-open.png"))); // NOI18N
        ladenButton.setText("Laden");
        ladenButton.setFocusable(false);
        ladenButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        ladenButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        ladenButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ladenButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(ladenButton);

        upButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/up.png"))); // NOI18N
        upButton.setText("aufwärts");
        upButton.setFocusable(false);
        upButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        upButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        upButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                upButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(upButton);
        upButton.getAccessibleContext().setAccessibleName("Nach Oben");

        downButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/down.png"))); // NOI18N
        downButton.setText("abwärts");
        downButton.setFocusable(false);
        downButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        downButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        downButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                downButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(downButton);

        druckenButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/document-print.png"))); // NOI18N
        druckenButton.setText("Drucken");
        druckenButton.setFocusable(false);
        druckenButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        druckenButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        druckenButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                druckenButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(druckenButton);

        hilfeButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/help-contents.png"))); // NOI18N
        hilfeButton.setText("Hilfe");
        hilfeButton.setFocusable(false);
        hilfeButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        hilfeButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        hilfeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                hilfeButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(hilfeButton);

        readerRefreshButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/gtk-refresh.png"))); // NOI18N
        readerRefreshButton.setText("Reader\n"); // NOI18N
        readerRefreshButton.setBorderPainted(false);
        readerRefreshButton.setDefaultCapable(false);
        readerRefreshButton.setFocusPainted(false);
        readerRefreshButton.setFocusable(false);
        readerRefreshButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        readerRefreshButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        readerRefreshButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                readerRefreshButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(readerRefreshButton);
        readerRefreshButton.getAccessibleContext().setAccessibleParent(jToolBar1);

        exitButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/application-exit.png"))); // NOI18N
        exitButton.setText("Schließen");
        exitButton.setFocusable(false);
        exitButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        exitButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        exitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(exitButton);

        logoALLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        logoALLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/KCEFM-Logo.png"))); // NOI18N
        logoALLabel.setAlignmentX(0.5F);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 770, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(logoALLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 69, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(logoALLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(0, 2, Short.MAX_VALUE))
        );

        getContentPane().add(jPanel1, java.awt.BorderLayout.PAGE_START);

        fileMenu.setText("Datei");

        readMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_L, java.awt.event.InputEvent.CTRL_MASK));
        readMenuItem.setText("Karte Lesen");
        readMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                readMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(readMenuItem);

        readSamMenuItem.setText("SAM lesen");
        readSamMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                readSamButtonActionPerformed(evt);
            }
        });
        fileMenu.add(readSamMenuItem);
        fileMenu.add(jSeparator1);

        druckMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.event.InputEvent.CTRL_MASK));
        druckMenuItem.setText("Drucken");
        druckMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                druckMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(druckMenuItem);

        speicherMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        speicherMenuItem.setText("Speichern");
        speicherMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                speicherMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(speicherMenuItem);

        ladenMenuItem.setText("Laden");
        ladenMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ladenMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(ladenMenuItem);
        fileMenu.add(jSeparator3);

        exitMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_ESCAPE, 0));
        exitMenuItem.setText("Schließen");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        ansichtMenu.setText("Ansicht");

        expertRadioButtonMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_E, java.awt.event.InputEvent.CTRL_MASK));
        expertRadioButtonMenuItem.setText("Expertenansicht");
        expertRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                expertRadioButtonMenuItemActionPerformed(evt);
            }
        });
        ansichtMenu.add(expertRadioButtonMenuItem);

        jCheckBoxMenuItem_showCustomer.setText("Kundendaten anzeigen");
        jCheckBoxMenuItem_showCustomer.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jCheckBoxMenuItem_showCustomerStateChanged(evt);
            }
        });
        ansichtMenu.add(jCheckBoxMenuItem_showCustomer);

        schriftMenu.setText("Schriftgröße");

        schriftGroesserMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_PLUS, java.awt.event.InputEvent.CTRL_MASK));
        schriftGroesserMenuItem.setText("Größer");
        schriftGroesserMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                schriftGroesserMenuItemActionPerformed(evt);
            }
        });
        schriftMenu.add(schriftGroesserMenuItem);

        schriftKleinerMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_MINUS, java.awt.event.InputEvent.CTRL_MASK));
        schriftKleinerMenuItem.setText("Kleiner");
        schriftKleinerMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                schriftKleinerMenuItemActionPerformed(evt);
            }
        });
        schriftMenu.add(schriftKleinerMenuItem);

        ansichtMenu.add(schriftMenu);

        menuBar.add(ansichtMenu);

        optionMenu.setText("Optionen");

        readerMenu.setText("RFID-Geräte");
        optionMenu.add(readerMenu);

        efmFolderMenuItem.setText("EFM-Daten");
        efmFolderMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                efmFolderMenuItemActionPerformed(evt);
            }
        });
        optionMenu.add(efmFolderMenuItem);

        TemplatesMenu.setText("Templates");

        jMenuItem1.setText("jMenuItem1");
        jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem1ActionPerformed(evt);
            }
        });
        TemplatesMenu.add(jMenuItem1);

        optionMenu.add(TemplatesMenu);
        optionMenu.add(optionSeparator);

        professionalEnableMenuItem.setText("Professional");
        professionalEnableMenuItem.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                professionalEnableMenuItemStateChanged(evt);
            }
        });
        optionMenu.add(professionalEnableMenuItem);

        checkBoxLog.setText("Log erstellen");
        checkBoxLog.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBoxLogActionPerformed(evt);
            }
        });
        optionMenu.add(checkBoxLog);

        menuBar.add(optionMenu);

        professionalMenu.setText("Professional");

        readParadigmMenuItem.setText("Muster-NM einlesen");
        readParadigmMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                readParadigmMenuItemActionPerformed(evt);
            }
        });
        professionalMenu.add(readParadigmMenuItem);

        stopNMCompareMenuItem.setText("NM-Vergleich stoppen");
        stopNMCompareMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopNMCompareMenuItemActionPerformed(evt);
            }
        });
        professionalMenu.add(stopNMCompareMenuItem);
        professionalMenu.add(jSeparator4);

        loadParadigmMenuItem.setText("Muster-NM laden");
        loadParadigmMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadParadigmMenuItemActionPerformed(evt);
            }
        });
        professionalMenu.add(loadParadigmMenuItem);

        saveParadigmMenuItem.setText("Muster-NM speichern");
        saveParadigmMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveParadigmMenuItemActionPerformed(evt);
            }
        });
        professionalMenu.add(saveParadigmMenuItem);
        professionalMenu.add(jSeparator2);

        samReaderMenu.setText("SAM-Reader");
        professionalMenu.add(samReaderMenu);

        profPropertiesMenuItem.setText("Einstellungen");
        profPropertiesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                profPropertiesMenuItemActionPerformed(evt);
            }
        });
        professionalMenu.add(profPropertiesMenuItem);

        menuBar.add(professionalMenu);

        help_aboutMenu.setText("?");

        helpMenuItem.setText("Hilfe");
        helpMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                helpMenuItemActionPerformed(evt);
            }
        });
        help_aboutMenu.add(helpMenuItem);

        updateMenuItem.setText("Aktualisieren");
        updateMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateMenuItemActionPerformed(evt);
            }
        });
        help_aboutMenu.add(updateMenuItem);

        aboutMenuItem.setText("Info");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });
        help_aboutMenu.add(aboutMenuItem);

        menuBar.add(help_aboutMenu);

        setJMenuBar(menuBar);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * ReadButton Action welche beim Drücken des Buttons ausgeführt wird.
     * @param evt Event.
     */
    private void readMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_readMenuItemActionPerformed
        readButtonActionPerformed(evt);
    }//GEN-LAST:event_readMenuItemActionPerformed

    /**
     * Exit Menue Aktion.
     * @param evt Event.
     */
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        beenden();
    }//GEN-LAST:event_exitMenuItemActionPerformed

    /**
     * Exit Button aktion.
     * @param evt Event.
     */
    private void exitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitButtonActionPerformed
        exitMenuItemActionPerformed(evt);
    }//GEN-LAST:event_exitButtonActionPerformed

    /**
     * Expert View Button Aktion.
     * @param evt Event.
     */
    private void expertRadioButtonMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_expertRadioButtonMenuItemActionPerformed
        if (expertRadioButtonMenuItem.isSelected()) {
            mainPanel.setViewportView(expertTable);
        } else {
            mainPanel.setViewportView(normalTable);
        }
    }//GEN-LAST:event_expertRadioButtonMenuItemActionPerformed

    /**
     * Hilfe Button Aktion.
     * @param evt Event.
     */
    private void hilfeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_hilfeButtonActionPerformed
        TicketinfoHelp vh = new TicketinfoHelp();
        vh.initHelpView();
    }//GEN-LAST:event_hilfeButtonActionPerformed

    /**
     * Schriftgröße vergrößern Aktion.
     * @param evt Event.
     */
    private void schriftGroesserMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_schriftGroesserMenuItemActionPerformed
        UmbruchTableCellRenderer.getInstance().increaseFont();
        NormalTableCellRenderer.getInstance().increaseFont();
        normalTable.updateUI();
        expertTable.updateUI();
    }//GEN-LAST:event_schriftGroesserMenuItemActionPerformed

    /**
     * Schriftgröße verkleinern Aktion.
     * @param evt Event.
     */
    private void schriftKleinerMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_schriftKleinerMenuItemActionPerformed
        UmbruchTableCellRenderer.getInstance().decreaseFont();
        NormalTableCellRenderer.getInstance().decreaseFont();
        normalTable.updateUI();
        expertTable.updateUI();
    }//GEN-LAST:event_schriftKleinerMenuItemActionPerformed

    /**
     * Drucken Button Aktion.
     * @param evt Event.
     */
    private void druckenButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_druckenButtonActionPerformed
        if (!expertRadioButtonMenuItem.isSelected()) {
            try {
                normalTable.print();
            } catch (PrinterException ex) {
                LOGGER.severe("Fehler beim Drucken der Normalansicht");
                LOGGER.log(Level.FINEST, null, ex);
            }
        } else {
            try {
                expertTable.print();
            } catch (PrinterException ex) {
                LOGGER.severe("Fehler beim Drucken der Expertenansicht");
                LOGGER.log(Level.FINEST, null, ex);
            }
        }
    }//GEN-LAST:event_druckenButtonActionPerformed

    /**
     * Drucken Menue Aktion.
     * @param evt Event.
     */
    private void druckMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_druckMenuItemActionPerformed
        this.druckenButtonActionPerformed(evt);
    }//GEN-LAST:event_druckMenuItemActionPerformed

    /**
     * Save Button Aktion.
     * @param evt Event.
     */
    private void saveButtonActionPerformed(ActionEvent evt) {
        ladeSaveFileFilter();
        if (!expertRadioButtonMenuItem.isSelected()) {

            int returnValue = saveFileChooser.showSaveDialog(this);

            if (returnValue == JFileChooser.APPROVE_OPTION) {
                final FileFilter fileFileFilter = saveFileChooser.getFileFilter();
                File saveFile = saveFileChooser.getSelectedFile();
                File validateSaveFile = SaveLoadFile.getInstance().validateFileName(saveFile, fileFileFilter);
                boolean success = false;
                if (fileFileFilter.toString().equalsIgnoreCase(nmFileFilter.toString())) {
                    success = SaveLoadFile.getInstance().save(validateSaveFile, app);
                } else {
                    success = SaveLoadFile.getInstance().save(validateSaveFile, normalTable);
                }
                if (!success) {
                    LOGGER.log(Level.SEVERE, "Speichern in die Datei {0} fehlgeschlagen!", validateSaveFile.getAbsolutePath());
                }
            }
        } else {
            int returnValue = saveFileChooser.showSaveDialog(this);
            if (returnValue == JFileChooser.APPROVE_OPTION) {
                final FileFilter fileFileFilter = saveFileChooser.getFileFilter();
                File saveFile = saveFileChooser.getSelectedFile();
                File validateSaveFile = SaveLoadFile.getInstance().validateFileName(saveFile, fileFileFilter);
                boolean success = false;
                if (fileFileFilter.toString().equalsIgnoreCase(nmFileFilter.toString())) {
                    success = SaveLoadFile.getInstance().save(validateSaveFile, app);
                } else {
                    success = SaveLoadFile.getInstance().save(validateSaveFile, expertTable);
                }
                if (!success) {
                    LOGGER.log(Level.SEVERE, "Speichern in die Datei {0} fehlgeschlagen!", validateSaveFile.getAbsolutePath());
                }
            }
        }
    }

    /**
     * Speichern Menu Aktion
     * @param evt Event
     */
    private void speicherMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_speicherMenuItemActionPerformed
        saveButtonActionPerformed(evt);
    }//GEN-LAST:event_speicherMenuItemActionPerformed

    /**
     * Laden Menu Aktion.
     * @param evt Event.
     */
    private void ladenMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ladenMenuItemActionPerformed
        ladeLoadFileFilter();
        int returnValue = loadFileChooser.showOpenDialog(this);
        if (returnValue == JFileChooser.APPROVE_OPTION) {
//            if (!mProfessional) {
//                this.viewerState = viewerState.VIEWONLY;
//            }
            final FileFilter loadFileFilter = loadFileChooser.getFileFilter();
            File openFile = loadFileChooser.getSelectedFile();
            File validateOpenFile;
            validateOpenFile = SaveLoadFile.getInstance().validateFileName(openFile, loadFileFilter);
            Object[] obs = null;
            if (loadFileFilter.toString().equalsIgnoreCase(nmFileFilter.toString())) {
                app = SaveLoadFile.getInstance().load(validateOpenFile);
                if (app == null) {
                    app = new Applikationsverzeichnis();
                }
                // Jetzt geht es an die Ausgabe
                appOK = true;
                logOK = true;
                custOK = true;
                keystoreOK = true;
                this.normalTableInhalt = ErstelleNormaleTabelle(app);
                this.expertTableInhalt = ErstelleExpertenTabelle(app);
                expertRadioButtonMenuItem.setSelected(false);
                this.buildTable();
            } else {
                DefaultTableModel tableModel = null;
                try {
                    tableModel = SaveLoadFile.getInstance().open(validateOpenFile);
                } catch (IOException ex) {
                    LOGGER.log(Level.FINEST, null, ex);
                }

                if (tableModel.getColumnCount() == 2) {
                    normalTable.setModel(tableModel);
                    sizeColumnOptimal(0, 0, this.normalTable.getRowCount(), normalTable, 20);
                    this.normalTable.getColumnModel().getColumn(1).setCellRenderer(UmbruchTableCellRenderer.getInstance());
                    this.normalTable.getColumnModel().getColumn(0).setCellRenderer(NormalTableCellRenderer.getInstance());
                    normalTable.getColumnModel().getColumn(0).setMaxWidth(this.mainPanel.getWidth() / 2);
                    expertTable.setModel(new DefaultTableModel(expertColHeaders, 0));
                    normalTable.updateUI();
                    if (expertRadioButtonMenuItem.isSelected()) {
                        expertRadioButtonMenuItem.setSelected(false);
                    }
                } else if (tableModel.getColumnCount() >= 3) {
                    expertTable.setModel(tableModel);
                    normalTable.setModel(new DefaultTableModel(normalColHeaders, 0));
                    expertTable.updateUI();
                    if (!expertRadioButtonMenuItem.isSelected()) {
                        expertRadioButtonMenuItem.setSelected(true);
                    }
                }
            }
            checkGUI();
        }
    }//GEN-LAST:event_ladenMenuItemActionPerformed

    /**
     * Aktualisieren der Liste vorhandener und einsatzbereiter PCSC-Geräte.
     * Falls mindestens ein Gerät gefunden wird, wird die entsprechende Funktionaliät eingeschalten.
     *
     * @param evt Der auslösende Event.
     */
    private void readerRefreshButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_readerRefreshButtonActionPerformed
        if (PCSC.updateAvailableTerminals() > 0) {
            viewerState = ViewerState.READERMODE_K;
        } else {
            viewerState = ViewerState.VIEWONLY;
        }
        buildReaderDropDown(true);
    }//GEN-LAST:event_readerRefreshButtonActionPerformed

    /**
     * Abhängig vom Zustand von checkBoxLog
     * <ul>
     *   <li><b>true</b> Initialisiert einen FileHandler (fileHandler) der dem Logger (LOGGER) hinzugefügt wird. Der FileHandler wird bei Bedarf erzeugt.</li>
     *   <li><b>false</b> Schließt den fileHandler und entfernt diesen vom LOGGER.
     *
     * @param evt Das auslösende Ereignis.
     */
    private void checkBoxLogActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checkBoxLogActionPerformed
        if (checkBoxLog.getState()) {
            FileLogManager.getInstance().enableFileLogging(null, null, null);
        } else {
            FileLogManager.getInstance().disableFileLogging();
        }
    }//GEN-LAST:event_checkBoxLogActionPerformed

    private void checkGUI() {
        if (mUseSam) {
            samReaderMenu.setVisible(true);
            loadKeyButton.setVisible(true);
            readSamButton.setVisible(true);
            readSamMenuItem.setVisible(true);
        } else {
            samReaderMenu.setVisible(false);
            loadKeyButton.setVisible(false);
            readSamButton.setVisible(false);
            readSamMenuItem.setVisible(false);
        }
        if (mProfessional) {
            professionalMenu.setVisible(true);
            jButtonBackward.setVisible(true);
            jButtonForward.setVisible(true);
            currentItemLabel.setVisible(true);
            expertRadioButtonMenuItem.setEnabled(false);
            mainPanel.setViewportView(null);
            upperScrollPane.setViewportView(expertTable);
            remove(mainPanel);
            add(professionalPanel, BorderLayout.CENTER);
        } else {
            readParadigmLabel.setVisible(false);
            professionalMenu.setVisible(false);
            expertRadioButtonMenuItem.setEnabled(true);
            jButtonBackward.setVisible(false);
            jButtonForward.setVisible(false);
            currentItemLabel.setVisible(false);
            upperScrollPane.setViewportView(null);
            remove(professionalPanel);
            add(mainPanel, BorderLayout.CENTER);
            if (expertRadioButtonMenuItem.isSelected()) {
                mainPanel.setViewportView(expertTable);
            } else {
                mainPanel.setViewportView(normalTable);
            }
        }
        if (pinOK && !mProfessional) {
            upButton.setVisible(true);
            downButton.setVisible(true);
            normalTable.setRowSelectionAllowed(true);
        } else {
            upButton.setVisible(false);
            downButton.setVisible(false);
            normalTable.setRowSelectionAllowed(false);
        }
        int idx = ((MultiSelectTable)normalTable).getGroupIdx();
        int berCount = 0;
        if (app != null) {
            berCount = app.getBerechtigungen().size();
        }
        if (normalTable.getRowCount() > 0
             || expertTable.getRowCount() > 0) {
            this.schriftMenu.setEnabled(true);
        } else {
            this.schriftMenu.setEnabled(true);
        }
        switch (viewerState) {
            case READERMODE_K:
                if (mProfessional) {
                    sucheProgressBar.setValue(0);
                    sucheProgressBar.setVisible(true);
                    readButton.setText("Scannen");
                    readMenuItem.setText("Scannen");
                    professionalEnableMenuItem.setEnabled(true);
                } else {
                    sucheProgressBar.setVisible(false);
                    readButton.setText("Lesen");
                    readMenuItem.setText("Karte auslesen");
                }
                if (readerName == null || readerName.isEmpty()) {
                    readButton.setEnabled(false);
                    readMenuItem.setEnabled(false);
                } else {
                    readButton.setEnabled(true);
                    readMenuItem.setEnabled(true);
                }
                samReaderMenu.setEnabled(mUseSam);
                if (!mUseSam || samReaderName == null || samReaderName.isEmpty()) {
                    readSamButton.setEnabled(false);
                    readSamMenuItem.setEnabled(false);
                    loadKeyButton.setEnabled(false);
                } else {
                    readSamButton.setEnabled(true);
                    readSamMenuItem.setEnabled(true);
                    loadKeyButton.setEnabled(true);
                }
                if (pinOK) {
                    downButton.setEnabled(idx > 0 && idx < berCount);
                    upButton.setEnabled(idx >= 0 && idx < (berCount-1));
                } else {
                    downButton.setEnabled(false);
                    upButton.setEnabled(false);
                }
                ladenButton.setEnabled(true);
                ladenMenuItem.setEnabled(true);
                if (normalTable.getRowCount() > 0
                        || expertTable.getRowCount() > 0) {
                    speicherMenuItem.setEnabled(true);
                    saveButton.setEnabled(true);
                    druckMenuItem.setEnabled(true);
                    druckenButton.setEnabled(true);
                } else {
                    speicherMenuItem.setEnabled(false);
                    saveButton.setEnabled(false);
                    druckMenuItem.setEnabled(false);
                    druckenButton.setEnabled(false);
                }
                readerRefreshButton.setEnabled(true);
                checkBoxLog.setVisible(true);
                readerMenu.setVisible(true);
                readerMenu.setEnabled(true);
                samReaderMenu.setEnabled(true);
                break;
            case VIEWONLY:
                sucheProgressBar.setVisible(false);
                readButton.setEnabled(false);
                readMenuItem.setEnabled(false);
                readSamButton.setEnabled(false);
                readSamMenuItem.setEnabled(false);
                loadKeyButton.setEnabled(false);
                downButton.setEnabled(false);
                upButton.setEnabled(false);
                ladenButton.setEnabled(true);
                ladenMenuItem.setEnabled(true);
                if (normalTable.getRowCount() > 0
                        || expertTable.getRowCount() > 0) {
                    speicherMenuItem.setEnabled(true);
                    saveButton.setEnabled(true);
                    druckMenuItem.setEnabled(true);
                    druckenButton.setEnabled(true);
                } else {
                    speicherMenuItem.setEnabled(false);
                    saveButton.setEnabled(false);
                    druckMenuItem.setEnabled(false);
                    druckenButton.setEnabled(false);
                }
                readerRefreshButton.setEnabled(true);
                checkBoxLog.setVisible(false);
                samReaderMenu.setVisible(false);
                readerMenu.setVisible(false);
                break;
            case READING:
                sucheProgressBar.setVisible(true);
                readButton.setEnabled(false);
                readMenuItem.setEnabled(false);
                readSamButton.setEnabled(false);
                readSamMenuItem.setEnabled(false);
                loadKeyButton.setEnabled(false);
                downButton.setEnabled(false);
                upButton.setEnabled(false);
                ladenButton.setEnabled(false);
                ladenMenuItem.setEnabled(false);
                speicherMenuItem.setEnabled(false);
                saveButton.setEnabled(false);
                druckMenuItem.setEnabled(false);
                druckenButton.setEnabled(false);
                readerRefreshButton.setEnabled(true);
                checkBoxLog.setVisible(true);
                readerMenu.setEnabled(false);
                samReaderMenu.setEnabled(false);
                readerMenu.setVisible(true);
                break;
            case SCANNING:
                sucheProgressBar.setVisible(true);
                readButton.setText("Stop");
                readMenuItem.setText("Scan beenden");
                professionalEnableMenuItem.setEnabled(false);
                readSamButton.setEnabled(false);
                readSamMenuItem.setEnabled(false);
                loadKeyButton.setEnabled(false);
                downButton.setEnabled(false);
                upButton.setEnabled(false);
                ladenButton.setEnabled(false);
                ladenMenuItem.setEnabled(false);
                speicherMenuItem.setEnabled(false);
                saveButton.setEnabled(false);
                druckMenuItem.setEnabled(false);
                druckenButton.setEnabled(false);
                readerRefreshButton.setEnabled(true);
                checkBoxLog.setVisible(true);
                readerMenu.setVisible(true);
                readerMenu.setEnabled(false);
                samReaderMenu.setEnabled(false);
                break;
        }
    }
    
    private void readButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_readButtonActionPerformed
        
        if (mProfessional && viewerState == ViewerState.SCANNING) {
            timer.stop();
            viewerState = ViewerState.READERMODE_K;
            checkGUI();
            return;
        }
        if (this.efmFilePfad.isEmpty() || EfmPvDatei.HasNoFiles()) {
            if (mConfigurator.isLocalConfig()) {
                this.openEFMFolder();
            }
        }

        if (mProfessional) {
            viewerState = ViewerState.SCANNING;
        } else {
            viewerState = ViewerState.READING;
        }
        normalTable.setModel(new DefaultTableModel());
        expertTable.setModel(new DefaultTableModel());
        normalTable.updateUI();
        expertTable.updateUI();

        checkGUI();
        readSam = false;
        app = new Applikationsverzeichnis();
        appOK = false;
        logOK = false;
        custOK = false;
        keystoreOK = false;
        progressBar();
    }//GEN-LAST:event_readButtonActionPerformed

    private SamInfo samInfo = null;
    private void ReadSAM() {
        if (sam.selectFile()) {
            LOGGER.info("SelectFile() erfolgreich ausgeführt.");
        } else {
            LOGGER.fine("SelectFile() fehlgeschlagen.");
            JOptionPane.showMessageDialog(null,
                        "Die Karte im Lesegerät \"" + readerName + "\" ist kein"
                      + " SAM der VDV-Kernapplikation oder kann nicht gelesen werden!", "Info",
                        JOptionPane.INFORMATION_MESSAGE);
            return;
        }

        final Frame tmpThis = this;
        final JDialog waitDialog = new JDialog(this);
        waitDialog.setTitle("Bitte warten");
        waitDialog.add(new JLabel("Das SAM wird ausgelesen..."));
        waitDialog.pack();
        waitDialog.setLocationRelativeTo(tmpThis);
        waitDialog.setVisible(true);
        if (samInfo == null) {
            samInfo = new SamInfo(this, true, null, "SAMs");
        }
        SwingWorker worker = new SwingWorker() {

            @Override
            protected Object doInBackground() throws Exception {
                sam.init();
                samInfo.reload(sam, samReaderName);
                return sam;
            }
            
            @Override
            protected void done() {
                waitDialog.setVisible(false);
                if (loadKey) {
                    LoadkeyDialog dlg = new LoadkeyDialog(tmpThis, true, sam);
                    dlg.setLocationRelativeTo(tmpThis);
                    dlg.setVisible(true);
                    loadKey = false;
                } else {
                    samInfo.setLocationRelativeTo(tmpThis);
                    samInfo.setVisible(true);
                }
                viewerState = ViewerState.READERMODE_K;
                checkGUI();
            }
        };
        worker.execute();
    }
    
    private void ReadKundenMedium() {
        if (kum.selectFile(app)) {
            LOGGER.info("SelectFile(" + app.getVersion().name() + ") erfolgreich ausgeführt.");
            if (mProfessional) {
                long lastdiff = System.currentTimeMillis() - lastread;                 
                if (app.getVerzAppDat().getVerzStatTeil().getAppInstanzID().getNmAppInstanznummer() == lastticket
                            && lastdiff <= 5000) {
                    progressBar();
                    return;
                }
                lastticket = app.getVerzAppDat().getVerzStatTeil().getAppInstanzID().getNmAppInstanznummer();
                lastread = System.currentTimeMillis();
            }
        } else {
            LOGGER.fine("SelectFile() fehlgeschlagen.");
            if (mProfessional) {
                if (kum.getLastError() == IIso7816_Response.SW1SW2_WRONG_LENGTH) {
                    if (JOptionPane.YES_OPTION != JOptionPane.showConfirmDialog(null,
                            "Die Karte im Lesegerät \"" + readerName + "\" ist kein"
                            + " E-Ticket der VDV-Kernapplikation oder kann nicht gelesen werden!\n\n"
                            + "Soll das Scannen fortgesetzt werden?", "Info", JOptionPane.YES_NO_OPTION,
                            JOptionPane.QUESTION_MESSAGE)) {
                        readButtonActionPerformed(null);
                        return;
                    }
                }
                progressBar();
                return;
            } else {
                JOptionPane.showMessageDialog(null,
                        "Die Karte im Lesegerät \"" + readerName + "\" ist kein"
                      + " E-Ticket der VDV-Kernapplikation oder kann nicht gelesen werden!", "Info",
                        JOptionPane.INFORMATION_MESSAGE);
                viewerState = viewerState.READERMODE_K;
                checkGUI();
            }
            return;
        }
        final Component tmpThis = this;
        final JDialog waitDialog = new JDialog(this);
        waitDialog.setTitle("Bitte warten");
        waitDialog.add(new JLabel("Die Karte wird ausgelesen..."));
        waitDialog.pack();
        waitDialog.setLocationRelativeTo(tmpThis);
        waitDialog.setVisible(true);
        SwingWorker worker = new SwingWorker() {

            private boolean result = true;
            @Override
            protected Object doInBackground() throws Exception {
                result = ReadKundenMediumImpl();
                if (mProfessional && result) {
                    handleProfessional();
                }
                return result;
            }
            
            @Override
            protected void done() {
                waitDialog.setVisible(false);
                normalTableInhalt = ErstelleNormaleTabelle(app);
                expertTableInhalt = ErstelleExpertenTabelle(app);
                buildTable();
                if (viewerState == ViewerState.SCANNING) {
                    progressBar();
                } else {
                    viewerState = viewerState.READERMODE_K;
                    checkGUI();
                }
            }
        };
        worker.execute();
    }
    
    private void handleProfessional() {
        Nutzermedium nm = new Nutzermedium();
        nm.Init(app);

        if(musterNMeinlesen==1) {
            musterNM = nm;

            setIgnoreList();                        
            musterNMeinlesen = 0;
            readParadigmLabel.setText("Muster-Medium bereit");
            stopNMCompareMenuItem.setVisible(true);

            JFileChooser fc = new JFileChooser();
            fc.setCurrentDirectory(new File(getConfig(CONFIG.Pfad)));
            fc.accept(null);
            fc.addChoosableFileFilter(nmFileFilter);
            fc.setFileFilter(nmFileFilter);
            if (fc.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
                String musterFilename = fc.getSelectedFile().getAbsolutePath();
                if (!musterFilename.endsWith("."+nmFileFilter.toString())) {
                    musterFilename += "."+nmFileFilter.toString();
                }
                boolean success = SaveLoadFile.getInstance().save(new File(musterFilename), app);
                if(!success) {
                    JOptionPane.showMessageDialog(null, "Speichern fehlgeschlagen", "Fehler", JOptionPane.ERROR);
                }
            }
        } else {
            nm.setFreitext("AppInstanzID: " + app.getVerzAppDat().getVerzStatTeil().getAppInstanzID().getNmAppInstanznummer()+"\r\n");
            if (app.getBerechtigungen().size() > 0) {
                DatenPSTeilNrw nrw = app.getBerechtigung(0).getSepDataTeil().getProdSpezTeil().get(DatenPSTeilNrw.class);
                if(nrw != null) {
                    String nn = nrw.getTicketteil().getKundeName();
                    String vn = nrw.getTicketteil().getKundeVorname();
                    nm.setFreitext(nm.getFreitext() + "Name: " + vn + " " + nn); 
                }
            }
            textAreaLabel.setText(nm.getFreitext());
            nm.validateBom();
            nm.validateEFM();
            errorTextArea.setText(nm.getErrortext());                    

//            app = new Applikationsverzeichnis();
            nmListe.Add(nm);                    

            nmAnzeigeIndex = nmListe.getAnzahl();
            currentItemLabel.setText(nmAnzeigeIndex + " / " + nmListe.getAnzahl());                    

            if (musterNM != null) {
                nm.diff(musterNM);
                errorTextArea.setText(nm.getErrortext());
            }

            try {
                if (getConfigBoolean(CONFIG.Ton)) {
                    try {
                        AudioInputStream audioInputStream = null;
                        if(nm.getErrortext().equals("\r\n\r\n")) {
                            audioInputStream = AudioSystem.getAudioInputStream(getClass().getResource("/Ticketinfo/resources/go.wav"));
                        } else {
                            audioInputStream = AudioSystem.getAudioInputStream(getClass().getResource("/Ticketinfo/resources/stop.wav"));
                        }
                        AudioFormat af = audioInputStream.getFormat();
                        int size = (int)(af.getFrameSize() * audioInputStream.getFrameLength());
                        byte[] audio = new byte[size];
                        DataLine.Info info = new DataLine.Info(Clip.class, af, size);
                        audioInputStream.read(audio,0,size);
                        Clip clip = (Clip) AudioSystem.getLine(info);
                        clip.open(af,audio,0,size);
                        clip.start();
                    } catch (LineUnavailableException ex) {
                        LOGGER.info(ex.getMessage());
                    } catch (UnsupportedAudioFileException ex) {
                        LOGGER.info(ex.getMessage());
                    } catch (IOException ex) {
                        LOGGER.info(ex.getMessage());
                    }
                }
            } catch (Exception except) {
                LOGGER.info(except.getMessage());
            }
            DatenSpeichern(true);
        }
    }

    private boolean ReadKundenMediumImpl() {
        if (LangesAppVerzeichnisNoetig()) {
            if (!kum.getData(true, app, (byte) 0x00)) {
                if (kum.getLastError() == KundenMedium.NM_PARSER_ERROR) {
                    JOptionPane.showMessageDialog(null,
                            "Es wurde ein E-Ticket mit mehr als 5 Berechtigungen gefunden.\n"
                          + "Die gelesenen Daten konnten jedoch nicht verarbeitet werden, da das Format\n"
                          + "dieser Karte offensichtlich nicht unterstützt wird oder fehlerhaft ist.", "Warnung",
                            JOptionPane.WARNING_MESSAGE);
                    LOGGER.severe("Langes ApplikationsVerzeichnis konnte nicht verarbeitet werden.");
                    return false;
                } else {
                    JOptionPane.showMessageDialog(null,
                            "Es wurde ein E-Ticket mit mehr als 5 Berechtigungen gefunden.\n"
                          + "Es konnten jedoch nicht alle Daten gelesen werden.\n"
                          + "Bitte halten Sie die Karte näher an das Lesegerät und entfernen Geräte\n"
                          + "und Gegenstände (z.B. metallische Gegenstände), die die Übertragung stören\n"
                          + "könnten und wiederholen Sie den Lesevorgang.", "Warnung",
                            JOptionPane.WARNING_MESSAGE);
                    LOGGER.severe("GetData (ApplikationsVerzeichnis) schlug fehl.");
                    LOGGER.finest(kum.getLastErrorString());
                    return false;
                }
            } else {
                if (kum.getLastError() == KundenMedium.NM_PARSER_ERROR) {
                    JOptionPane.showMessageDialog(null,
                            "Es wurde ein E-Ticket mit mehr als 5 Berechtigungen gefunden.\n"
                          + "Es konnten jedoch nicht alle Daten korrekt verarbeitet werden.\n"
                          + "Das Format dieser Karte wird offensichtlich teilweise nicht unterstützt oder\n"
                          + "ist teilweise fehlerhaft.\n"
                          + "Die angezeigten Daten können deshalb von den auf der Karte\n"
                          + "gespeicherten Daten abweichen.", "Warnung",
                            JOptionPane.WARNING_MESSAGE);
                    LOGGER.severe("Langes ApplikationsVerzeichnis konnte nicht korrekt verarbeitet werden.");
                }
                LOGGER.info("GetData (ApplikationsVerzeichnis) erfolgreich ausgeführt.");
            }
        }
        for (int idx = 0; idx < app.getBerechtigungen().size(); ++idx) {
            Berechtigung b = app.getBerechtigung(idx);
            if (!kum.getData(false, b, (byte) b.getVerzTeil().getSysSpezVerzTeil().getZeiger().getValue())) {
                if (kum.getLastError() == KundenMedium.NM_PARSER_ERROR) {
                    JOptionPane.showMessageDialog(null,
                            "Es wurde ein E-Ticket mit Berechtigungen gefunden.\n"
                          + "Die gelesenen Daten der " + Integer.toString(idx + 1) + ". Berechtigung konnten jedoch\n"
                          + "leider nicht verarbeitet werden.\n"
                          + "Das Format dieser Karte wird offensichtlich nicht unterstützt oder ist fehlerhaft.", "Warnung",
                            JOptionPane.WARNING_MESSAGE);
                    app = new Applikationsverzeichnis();
                    LOGGER.severe("Berechtigung konnte nicht verarbeitet werden.");
                } else {
                    JOptionPane.showMessageDialog(null,
                            "Es wurde ein E-Ticket mit Berechtigungen gefunden.\n"
                          + "Die Details der " + Integer.toString(idx + 1) + ". Berechtigung konnten jedoch leider nicht\n"
                          + "gelesen werden!\n"
                          + "Bitte halten Sie die Karte näher an das Lesegerät und entfernen Geräte\n"
                          + "und Gegenstände (z.B. metallische Gegenstände), die die Übertragung stören\n"
                          + "könnten und wiederholen Sie den Lesevorgang.", "Warnung",
                            JOptionPane.WARNING_MESSAGE);
                    LOGGER.severe("GetData (SeparateDaten_Berechtigung) schlug fehl.");
                    LOGGER.finest(kum.getLastErrorString());
                }
                continue;
            } else {
                if (kum.getLastError() == KundenMedium.NM_PARSER_ERROR) {
                    JOptionPane.showMessageDialog(null,
                            "Es wurde ein E-Ticket mit Berechtigungen gefunden.\n"
                          + "Die gelesenen Daten der " + Integer.toString(idx + 1) + ". Berechtigung konnten jedoch\n"
                          + "teilweise nicht korrekt verarbeitet werden.\n"
                          + "Das Format dieser Karte wird offensichtlich teilweise nicht unterstützt oder ist teilweise fehlerhaft.\n"
                          + "Die angezeigten Daten dieser Berechtigung können deshalb von den auf der Karte gespeicherten Daten abweichen.", "Warnung",
                            JOptionPane.WARNING_MESSAGE);
                    LOGGER.severe("Berechtigung konnte nicht korrekt verarbeitet werden.");
                }
                LOGGER.info("GetData (SeparateDaten_Berechtigung) erfolgreich ausgeführt.");
            }
        }

        //if (!kum.getData(true, app.getApplikation(), (byte) app.getApplikation().getVerzTeil().getSysSpezVerzTeil().getZeiger().getValue())) {
        if (!kum.getData(false, app.getApplikation(), (byte) app.getApplikation().getVerzTeil().getSysSpezVerzTeil().getZeiger().getValue())) {
            appOK = false;
            if (kum.getLastError() == KundenMedium.NM_PARSER_ERROR) {
                LOGGER.severe("Applikationsdaten konnten nicht verarbeitet werden.");
            } else {
                LOGGER.severe("GetData (Applikationsdaten) schlug fehl.");
                LOGGER.finest(kum.getLastErrorString());
            }
        } else {
            appOK = true;
            LOGGER.info("GetData (Applikationslogbuch) erfolgreich ausgeführt.");
        }
        
        if (!kum.getData(true, app.getLogbuch(), (byte) app.getLogbuch().getVerzTeil().getSysSpezVerzTeil().getZeiger().getValue())) {
//            logOK = false;
            if (kum.getLastError() == KundenMedium.NM_PARSER_ERROR) {
                LOGGER.severe("Applikationslogbuch konnte nicht verarbeitet werden.");
                
                for (int logI=1; logI<=20; ++logI) {
                    byte[] logBuffer = new byte[255];
                    int len = kum.getData(false, SeparateDatenApplikationslogbuch.TAG, logI, logBuffer);
                    if (len > 0) {
                        Transaktion tr = new Transaktion(app, app.getVersion());
                        tr.unmarshal(logBuffer, 0);
                        app.getLogbuch().getSepDataTeil().addTransaction(tr);
                    } else {
                        break;
                    }
                }
                logOK = true;
            } else {
                LOGGER.severe("GetData (Applikationslogbuch) schlug fehl.");
                LOGGER.finest(kum.getLastErrorString());
            }
        } else {
            logOK = true;
            LOGGER.info("GetData (Applikationslogbuch) erfolgreich ausgeführt.");
        }

        if (mShowCustomer) {
            if (authCardId == null || !authCardId.equals(app.getVerzAppDat().getVerzStatTeil().getAppInstanzID())) {
                PinDialog pinDialog = new PinDialog(this, kum);
                pinDialog.setVisible(true);
                //this.pinFrame.pack();
                //this.pinFrame.setVisible(true);
                if (pinDialog.isPinOK()) {
                    pinOK = true;
                    authCardId = app.getVerzAppDat().getVerzStatTeil().getAppInstanzID();
                } else {
                    pinOK = false;
                    authCardId = null;
                }
            }
            if (pinOK) {
                if (!kum.getData(false, app.getKundendaten().getSepDataTeil(), (byte) app.getKundendaten().getVerzTeil().getSysSpezVerzTeil().getZeiger().getValue())) {
                    custOK = false;
                    if (kum.getLastError() == KundenMedium.NM_PARSER_ERROR) {
                        LOGGER.severe("Kundendaten konnten nicht verarbeitet werden.");
                    } else {
                        LOGGER.severe("GetData (Kundendaten) schlug fehl.");
                        LOGGER.finest(kum.getLastErrorString());
                    }
                } else {
                    custOK = true;
                    LOGGER.info("GetData (SeparateDaten_Kundendaten) erfolgreich ausgeführt.");
                }
            } else {
                custOK = false;
                LOGGER.info("Kundendaten werden ausgelassen: Keine PIN!");
            }
        }
        
        if (app.getVersion() == KAVersion.V_1108 || app.getVersion().compare(KAVersion.V_1200) >= 0) {
            if (!kum.getData(false, app.getKeystore().getSepDataTeil(), (byte) app.getKeystore().getVerzTeil().getSysSpezVerzTeil().getZeiger().getValue())) {
                keystoreOK = false;
                if (kum.getLastError() == KundenMedium.NM_PARSER_ERROR) {
                    LOGGER.severe("Keystore-Teil konnte nicht verarbeitet werden.");
                } else {
                    LOGGER.severe("GetData (Keystore) schlug fehl.");
                    LOGGER.finest(kum.getLastErrorString());
                }
            } else {
                keystoreOK = true;
                LOGGER.info("GetData (SeparateDaten_Keystore) erfolgreich ausgeführt.");
            }
        } else {
            if (!kum.getData(false, app.getWes().getSepDataTeil(), (byte) app.getWes().getVerzTeil().getSysSpezVerzTeil().getZeiger().getValue())) {
                    keystoreOK = false;
                if (kum.getLastError() == KundenMedium.NM_PARSER_ERROR) {
                    LOGGER.severe("WES-Teil konnte nicht verarbeitet werden.");
                } else {
                    LOGGER.severe("GetData (WES) schlug fehl.");
                    LOGGER.finest(kum.getLastErrorString());
                }
            } else {
                keystoreOK = true;
                LOGGER.info("GetData (SeparateDaten_WES) erfolgreich ausgeführt.");
            }
        }
        // Jetzt geht es an die Ausgabe
        //this.normalTableInhalt = ErstelleNormaleTabelle(app);
        //this.expertTableInhalt = ErstelleExpertenTabelle(app);
        //this.buildTable();
        return true;
    }

    private boolean LangesAppVerzeichnisNoetig() {
        return (app.getPrios().getValidPrioCount() > 5);
    }

    private void helpMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_helpMenuItemActionPerformed
        this.hilfeButtonActionPerformed(evt);
    }//GEN-LAST:event_helpMenuItemActionPerformed

    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
        if (this.TicketinfoAbout == null) {
            this.TicketinfoAbout = new TicketinfoAbout(this, true);
            this.TicketinfoAbout.setLocationRelativeTo(this);
        }
        this.TicketinfoAbout.setVisible(true);
    }//GEN-LAST:event_aboutMenuItemActionPerformed

    private void efmFolderMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_efmFolderMenuItemActionPerformed
        this.openEFMFolder();
    }//GEN-LAST:event_efmFolderMenuItemActionPerformed

    private void readerMenuMenuSelected(javax.swing.event.MenuEvent evt) {//GEN-FIRST:event_readerMenuMenuSelected
    }//GEN-LAST:event_readerMenuMenuSelected

    private void updateMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_updateMenuItemActionPerformed
        String url = getConfig(CONFIG.UpdateUrl, "");
        if (Desktop.isDesktopSupported()) {
            try {
                Desktop.getDesktop().browse(URI.create(url));
            } catch (Exception ex) {
                JOptionPane.showMessageDialog(null,
                    "Beim Öffnen der Url mit dem Standardbrowser ist ein Fehler aufgetreten.\n"
                  + "Bitte öffnen Sie einen Internetbrowser und besuchen Sie die Webseite um Updates herunterzuladen:\n\n"
                  + url + "\n", "Information",
                    JOptionPane.INFORMATION_MESSAGE);
                LOGGER.severe("Öffnen eines Links mit dem Standardsystembrowser fehlgeschlagen.");
            }
        } else {
            JOptionPane.showMessageDialog(null,
                "Ihr JavaRuntimeEnvironment scheint das Öffnen einer Url mit dem Standardbrowser nicht zu unterstützen.\n"
              + "Bitte öffnen Sie einen Internetbrowser und besuchen Sie die Webseite um Updates herunterzuladen:\n\n"
              + url + "\n", "Information",
                JOptionPane.INFORMATION_MESSAGE);
            LOGGER.warning("java.awt.Desktop wird von der verwendeten Java Version nicht unterstützt.");
        }
    }//GEN-LAST:event_updateMenuItemActionPerformed

    private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed
        MenuItem mi = (MenuItem) evt.getSource();
        TemplateEditor te = new TemplateEditor(this, true, TemplateProvider.getInstance().getTemplate(mi.getName()));
        te.setLocationRelativeTo(this);
        te.setVisible(true);
    }//GEN-LAST:event_jMenuItem1ActionPerformed

    private void readSamButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_readSamButtonActionPerformed
        viewerState = ViewerState.READING;
        checkGUI();

        sam = null;
        readSam = true;
        progressBar();
    }//GEN-LAST:event_readSamButtonActionPerformed

    private boolean loadKey = false;
    private void loadKeyButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadKeyButtonActionPerformed
        loadKey = true;
        readSamButtonActionPerformed(evt);
    }//GEN-LAST:event_loadKeyButtonActionPerformed

    private void upButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_upButtonActionPerformed
        int idx = ((MultiSelectTable)normalTable).getGroupIdx();
        app.moveBerechtigung(idx, true);
    }//GEN-LAST:event_upButtonActionPerformed

    private void downButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_downButtonActionPerformed
        int idx = ((MultiSelectTable)normalTable).getGroupIdx();
        app.moveBerechtigung(idx, false);
    }//GEN-LAST:event_downButtonActionPerformed

    private boolean mProfessional = false;
    private void professionalEnableMenuItemStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_professionalEnableMenuItemStateChanged
        mProfessional = professionalEnableMenuItem.getState();
        mUseSam = mProfessional;
        buildReaderDropDown(false);
    }//GEN-LAST:event_professionalEnableMenuItemStateChanged

    private boolean mShowCustomer = false;
    private void jCheckBoxMenuItem_showCustomerStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jCheckBoxMenuItem_showCustomerStateChanged
        if (jCheckBoxMenuItem_showCustomer.getState() && !mShowCustomer) {
            int opt = JOptionPane.showConfirmDialog(this,
                    "Um die Kundendaten anzeigen zu können wird die PIN der Karte benötigt.\n\n"
                    + "Bei Falscheingabe der PIN kann die PIN von der Karte gesperrt werden!\n"
                    + "Deshalb sollten Sie diese Option nur aktivieren, wenn Sie die PIN zu der Karte haben.\n"
                    + "Haben Sie die PIN zu Ihrer Karte?", "Karten-PIN", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (opt != JOptionPane.YES_OPTION) {
                jCheckBoxMenuItem_showCustomer.setState(false);
            } else {
                mShowCustomer = true;
            }
        } else {
            mShowCustomer = jCheckBoxMenuItem_showCustomer.getState();
        }
        
    }//GEN-LAST:event_jCheckBoxMenuItem_showCustomerStateChanged

    private final IConfigurator mConfigurator;
    private Properties mProperties;
    private void profPropertiesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_profPropertiesMenuItemActionPerformed
        boolean isRun = false;
        if (timer != null) {
            if(timer.isRunning()) {
                timer.stop();
                isRun = true;
            }
        }
        OptionDialog dialog = new OptionDialog(this, true);
        dialog.setProperties(new Properties(mProperties));
        centerWindow(dialog);
        dialog.setVisible(true);

        if (dialog.getOK()) {
            mProperties = dialog.getProperties();
            mConfigurator.storeConfiguration(mProperties);
        }
        if (timer != null) {
            if (isRun) {
                timer.start();
            }
        }
    }//GEN-LAST:event_profPropertiesMenuItemActionPerformed

    private void jButtonForwardActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonForwardActionPerformed
        if (nmAnzeigeIndex >= nmListe.getAnzahl()) {
            return;
        }

        nmAnzeigeIndex++;
        Nutzermedium nm = nmListe.getElement(nmAnzeigeIndex);
        this.expertTableInhalt = ErstelleExpertenTabelle(nm.getApp());
        this.buildTable();
        currentItemLabel.setText(nmAnzeigeIndex + " / " + nmListe.getAnzahl());
        textAreaLabel.setText(nm.getFreitext());
        errorTextArea.setText(nm.getErrortext());
    }//GEN-LAST:event_jButtonForwardActionPerformed

    private void jButtonBackwardActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonBackwardActionPerformed
        if (nmAnzeigeIndex <= 1) {
            return;
        }

        nmAnzeigeIndex--;
        Nutzermedium nm = nmListe.getElement(nmAnzeigeIndex);
        this.expertTableInhalt = ErstelleExpertenTabelle(nm.getApp());
        this.buildTable();
        currentItemLabel.setText(nmAnzeigeIndex + " / " + nmListe.getAnzahl());
        textAreaLabel.setText(nm.getFreitext());
        errorTextArea.setText(nm.getErrortext());
    }//GEN-LAST:event_jButtonBackwardActionPerformed

    private void buttonAktualisierenActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonAktualisierenActionPerformed
        if(nmListe.getAnzahl() == 0) {
            return;
        }

        Nutzermedium nm = nmListe.getElement(nmAnzeigeIndex);
        nm.setFreitext(textAreaLabel.getText());

        try {
            DatenSpeichern(false);
        } catch(Exception ex) {
            LOGGER.log(Level.SEVERE, "Speichern der Daten fehlgeschlagen: {0}", ex.getMessage());
        }
    }//GEN-LAST:event_buttonAktualisierenActionPerformed

    /**
     * Setzt die Ignorieren Liste der Elemente, die beim Vergleichen wegfallen
     */
    private void setIgnoreList() {
        try {
            String homePath = System.getProperty("user.dir");
            String line = "";
            File ignoreFile = new File(homePath + File.separator + "ignoreList.txt");
            BufferedReader in = new BufferedReader(new FileReader(ignoreFile));
        
            while ((line = in.readLine()) != null) {
                try {
                    musterNM.SetIgnoreElements(line);
                } catch(Exception e) {
                }
            }
            in.close();
        } catch (Exception ex) {
        }
    }

    private void loadParadigmMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadParadigmMenuItemActionPerformed
        if (mProfessional) {
            JFileChooser fc = new JFileChooser();
            fc.addChoosableFileFilter(nmFileFilter);
            fc.setFileFilter(nmFileFilter);
            fc.setCurrentDirectory(new File(getConfig(CONFIG.Pfad)));
            if( fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                app = new Applikationsverzeichnis();       
                
                File openFile = fc.getSelectedFile();
                app = SaveLoadFile.getInstance().load(openFile);
                if (app == null) {
                    app = new Applikationsverzeichnis();
                }

                musterNM = new Nutzermedium();
                musterNM.Init(app);

                app = new Applikationsverzeichnis();

                setIgnoreList();        

                readParadigmLabel.setText("Muster-Medium bereit");
                readParadigmLabel.setVisible(true);
                stopNMCompareMenuItem.setVisible(true);
            }
        }
    }//GEN-LAST:event_loadParadigmMenuItemActionPerformed

    private void readParadigmMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_readParadigmMenuItemActionPerformed
        musterNMeinlesen = 1;
        readParadigmLabel.setText("Mustermedium kann eingelesen werden");
        readParadigmLabel.setVisible(true);
    }//GEN-LAST:event_readParadigmMenuItemActionPerformed

    private void stopNMCompareMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopNMCompareMenuItemActionPerformed
        musterNM = null;
        readParadigmLabel.setVisible(false);
        stopNMCompareMenuItem.setVisible(false);
    }//GEN-LAST:event_stopNMCompareMenuItemActionPerformed

    private void ladenButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ladenButtonActionPerformed
        ladenMenuItemActionPerformed(evt);
    }//GEN-LAST:event_ladenButtonActionPerformed

    private void saveParadigmMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveParadigmMenuItemActionPerformed
        if (app == null) {
            JOptionPane.showMessageDialog(null, "Speichern fehlgeschlagen", "Fehler", JOptionPane.ERROR);
            return;
        }
        try {
            JFileChooser fc = new JFileChooser();
            fc.setCurrentDirectory(new File(getConfig(CONFIG.Pfad)));
            fc.accept(null);
            fc.addChoosableFileFilter(nmFileFilter);
            fc.setFileFilter(nmFileFilter);
            if (fc.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
                String musterFilename = fc.getSelectedFile().getAbsolutePath();
                if (!musterFilename.endsWith(nmFileFilter.toString())) {
                    musterFilename += nmFileFilter.toString();
                }
                Nutzermedium nm = nmListe.getElement(nmAnzeigeIndex);
                if (SaveLoadFile.getInstance().save(new File(musterFilename), nm.getApp())) {
                    JOptionPane.showMessageDialog(null, "Speichern fehlgeschlagen", "Fehler", JOptionPane.ERROR);
                }
            }
        } catch(Exception e) {
            JOptionPane.showMessageDialog(null, "Speichern fehlgeschlagen", "Fehler", JOptionPane.ERROR);
        }
    }//GEN-LAST:event_saveParadigmMenuItemActionPerformed

    /**
     * Nutzermedium speichern in XML
     * 3 Arten: Eine Datei / Einzelne Karte / Einzelne Berechtigung
     */
    private void DatenSpeichern(boolean neu) {
        int speicherIndex;
        int ramindex;
        if(neu) {
            speicherIndex = nmIndex;
            ramindex = nmListe.getAnzahl();
        } else {
            speicherIndex = nmIndex - 1 - nmListe.getAnzahl() + nmAnzeigeIndex;
            ramindex = nmAnzeigeIndex;
        }

        try {
            File dir = new File(getConfig(CONFIG.Pfad));
            //String p = prop.getProperty("Pfad");
            if(!dir.exists()) {
                dir.mkdirs();
            }
            if(!dir.exists()) {
                JOptionPane.showMessageDialog(null, "Daten nicht gespeichert.\r\n" + dir.getPath() + " nicht vorhanden.");
                return;
            }
            if(!dir.isDirectory()) {
                JOptionPane.showMessageDialog(null, "Daten nicht gespeichert.\r\n" + dir.getPath() + " ist kein gültiges Verzeichnis.");
                return;
            }

            // alles in eine Datei
            if (getConfigBoolean(CONFIG.einzelneDatei)) {
                String dateiname = 
                    getConfig(CONFIG.Pfad) + getConfig(CONFIG.PraefixDatei)
                    + int2FixedLenStr(datIndex, 3) + ".xml";
                File f = new File(dateiname);
                FileOutputStream output = new FileOutputStream(f);
                JAXB.marshal(nmListe, output);
            }
        
            // einzelne Nutzermedien
            if (getConfigBoolean(CONFIG.einzelneNM)) {
                String zahl = int2FixedLenStr(speicherIndex/*nmIndex*/, getConfigInt(CONFIG.StellenNM));
                String dateiname = 
                    getConfig(CONFIG.Pfad) + 
                    getConfig(CONFIG.PraefixNM) +
                    getConfig(CONFIG.Trennzeichen1) +
                    zahl +
                    ".xml";
                File f = new File(dateiname);
                try {
                    FileOutputStream output = new FileOutputStream(f);
                    JAXB.marshal(nmListe.getElement(ramindex)/*nmListe.LetztesElement()*/, output);
                } catch (Exception ex) {
                    JOptionPane.showMessageDialog(null, "Fehler beim Speichern des einzelnen Nutzermediums-" + ex.getMessage());
                }
            }

            // einzelne Berechtigungen
            if (getConfigBoolean(CONFIG.einzelneBer)) {
                Nutzermedium nm = nmListe.getElement(ramindex);/*nmListe.LetztesElement();*/
                int berIndex = 1;
                SeparateDatenBerechtigung[] sepDatList = nm.getSepDat();
                for(int i = 0; i < sepDatList.length; i++) {
                    SeparateDatenBerechtigung sdb = sepDatList[i];
                    if (sdb != null) {
                        NMBerechtigung ber = new NMBerechtigung(sdb);
                        ber.setFehlerVDVKA(nm.getFehlerVDVKA());
                        ber.setFehlerTarif(nm.getFehlerTarif());
                        ber.setFehlerVergleich(nm.getFehlerVergleich());
                        ber.setFreiText(nm.getFreitext());
                        
                        String zahl1 = int2FixedLenStr(speicherIndex/*nmIndex*/, getConfigInt(CONFIG.StellenNM));
                        String zahl2 = int2FixedLenStr(berIndex, getConfigInt(CONFIG.StellenBer));
                        String dateiname = 
                            getConfig(CONFIG.Pfad) + getConfig(CONFIG.PraefixBer) +
                            getConfig(CONFIG.Trennzeichen1) +
                            zahl1 +
                            getConfig(CONFIG.Trennzeichen2) +
                            zahl2 +
                            ".xml";
                        File f = new File(dateiname);
                        try {
                            FileOutputStream output = new FileOutputStream(f);
                            JAXB.marshal(ber, output);
                        } catch(Exception ex) {
                            JOptionPane.showMessageDialog(null, "Fehler beim Speichern der Berechtigung-" + ex.getMessage());
                        }
                        berIndex++;
                    }
                }
            }
        
            if (neu) {
                nmIndex++;
            }
        } catch(Exception ex) {
            JOptionPane.showMessageDialog(null, "Fehler beim Speichern der Daten-" + ex.getMessage());
        }
    }
    
    private String int2FixedLenStr(int zahl, int laenge) {
        String result = Integer.toString(zahl);
        while (result.length()<laenge) {
            result = "0" + result;            
        }
        return result;
    }

    private boolean mUseSam = false;

    /**
     * Startet den MainView.
     * @param args the command line arguments
     * @throws IOException Fehler beim View.
     */
    public static void main(final String[] args) throws IOException {
        if (args != null) {
            for (String arg : args) {
                LOGGER.log(Level.INFO, "Param: \""+arg+"\"");
            }
        }
        
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                IConfigurator cfg = null;
                if (args != null) {
                    if (args.length > 0) {
                        if (args.length > 1) {
                            cfg = new ConfigWebLoader(args[0], args[1]);
                            LOGGER.log(Level.INFO, "Using web-configuration (\""+args[0]+"\",\""+args[1]+"\")");
                        } else {
                            cfg = new ConfigWebLoader(args[0], null);
                            LOGGER.log(Level.INFO, "Using web-configuration (\""+args[0]+"\",null)");
                        }
                    }
                }
                if (cfg == null) {
                    cfg = new ConfigFileLoader(getDefaultConfig());
                    LOGGER.log(Level.INFO, "Using file-configuration");
                }
                new Ticketinfo(cfg).setVisible(true);
            }
        });
    }

    public boolean getHexdumpStateW() {
        return this.hexdumpStateW;
    }

    public boolean getHexdumpStateR() {
        return this.hexdumpStateR;
    }

    private String[][] ErstelleNormaleTabelle(Applikationsverzeichnis app) {
        Queue<Pair<String, List<String>>> result =
                new LinkedList<Pair<String, List<String>>>();
        
        List<String> emptyList = new ArrayList<String>();
        emptyList.add("");
        List<String> nullList = new ArrayList<String>();
        nullList.add(null);
        int lfdnr = 0;
        
        boolean cardIsValid = 
                app.getApplikation().getSepDataTeil().getAusTransKennung().getNmTransaktionID().isValid()
                && app.getVerzAppDat().getVerzDynTeil().getStatusCode().getValue32() == 7
                && app.getVerzAppDat().getVerzDynTeil().getSynchronNummer().getValue32() > 0;
        if (!cardIsValid) {
            List<String> tmpList = new ArrayList<String>();
            tmpList.add(null);
            result.add(new Pair<String,List<String>>("Die Karte ist ungültig!", tmpList));
        } else {
            DateTimeCompact now = new DateTimeCompact(new Date());
            if (app.getVerzAppDat().getVerzStatTeil().getGueltBeginn().after(now)) {
                List<String> tmpList = new ArrayList<String>();
                tmpList.add(null);
                result.add(new Pair<String,List<String>>("Die Karte ist noch nicht gültig!", tmpList));
                cardIsValid = false;
            } else {
                if (app.getVerzAppDat().getVerzStatTeil().getGueltEnde().before(now)) {
                    List<String> tmpList = new ArrayList<String>();
                    tmpList.add(null);
                    result.add(new Pair<String,List<String>>("Die Karte ist abgelaufen!", tmpList));
                    cardIsValid = false;
                }
            }
        }
        // Berechtigungen
        for (int i = 0; cardIsValid && i < app.getBerechtigungen().size(); ++i) {
            Berechtigung b = app.getBerechtigung(i);
            if ((b.getVerzTeil().getVerzDynTeil().getStatusCode().getValue32() != 7) ||
                (!b.getSepDataTeil().getAusTransKennung().getNmTransaktionID().isValid()) ) {
                continue;
            } else {
                ++lfdnr;
            }
            String tltName = b.getSepDataTeil().getProdSpezTeil().getTemplate().getTemplateName();
            result.add(new Pair<String, List<String>>(Integer.toString(lfdnr) + ". Ticket (" + tltName + ")", nullList));
            DatenPSTeilNrw nrw = b.getSepDataTeil().getProdSpezTeil().get(DatenPSTeilNrw.class);
            DatenPSTeilEfs efs = b.getSepDataTeil().getProdSpezTeil().get(DatenPSTeilEfs.class);
            if (nrw != null) {
                List<String> tmpList = new ArrayList<String>();
                tmpList.add(b.getVerzTeil().getVerzStatTeil().getProduktId().getProductname());
                result.add(new Pair<String, List<String>>("Produkt:", tmpList));
                
                String tarifgebiete = "";
                String waben = "";
                Tarifgebiet[] gebiete = nrw.GetTarifgebiete(b.getVerzTeil().getVerzStatTeil().getProduktId());
                for (int t = 0; t < gebiete.length; ++t) {
                    tarifgebiete += (gebiete[t].HasName()) ? (gebiete[t].GetName() + "(" + gebiete[t].strNummer + ")") : gebiete[t].strNummer;
                    if (t != gebiete.length - 1) {
                        tarifgebiete += "; ";
                    }

                    if (gebiete.length == 1 || gebiete[t].nummer == Tarifgebiet.DEFAULT) {
                        Wabe[] act_waben = gebiete[t].GetWaben();
                        if (gebiete[t].nummer == Tarifgebiet.DEFAULT || act_waben.length <= 2) {
                            if (act_waben.length > 0) {
                                for (int w = 0; w < act_waben.length; ++w) {
                                    waben += ((act_waben[w].HasName()) ? (act_waben[w].GetName() + "(" + act_waben[w].strNummer + ")") : act_waben[w].strNummer);
                                    if (w != act_waben.length - 1) {
                                        waben += "; ";
                                    }
                                }
                            }
                        }
                    }
                }
                tmpList = new ArrayList<String>();
                tmpList.add(tarifgebiete);
                result.add(new Pair<String,List<String>>("Gültig in (Tarifgebiete):", tmpList));

//                tmpList = new ArrayList<String>();
//                tmpList.add(waben);
//                result.add(new Pair<String,List<String>>("Waben:", tmpList));
                
                tmpList = new ArrayList<String>();
                try {
                    Pair<Date, Date> dates = nrw.getTicketteil().getGueltigkeitDates();
                    SimpleDateFormat formatter = new SimpleDateFormat("dd.MM.yyyy");
                    tmpList.add(formatter.format(dates.getValueA()) + " - " + formatter.format(dates.getValueB()));
                } catch (ParseException ex) {
                    tmpList.add(b.getVerzTeil().getVerzStatTeil().getGueltBeginn().toString() + " - " + b.getVerzTeil().getVerzStatTeil().getGueltEnde().toString());
                }
                result.add(new Pair<String,List<String>>("zeitliche Gültigkeit:", tmpList));
           
                tmpList = new ArrayList<String>();
                tmpList.add(nrw.getTicketteil().getZusaetze());
                result.add(new Pair<String,List<String>>("Zusätze:", tmpList));
                
                tmpList = new ArrayList<String>();
                tmpList.add(nrw.getPreisstufe(b.getVerzTeil().getVerzStatTeil().getProduktId()));
                result.add(new Pair<String,List<String>>("Preisstufe:", tmpList));
                
                tmpList = new ArrayList<String>();
                tmpList.add(nrw.getTicketteil().getKundeVorname());
                result.add(new Pair<String,List<String>>("Vorname:", tmpList));
                
                tmpList = new ArrayList<String>();
                tmpList.add(nrw.getTicketteil().getKundeName());
                result.add(new Pair<String,List<String>>("Name:", tmpList));
                
                tmpList = new ArrayList<String>();
                tmpList.add(nrw.getKundeGeschlecht(true).toString());
                result.add(new Pair<String,List<String>>("Geschlecht:", tmpList));
                
                tmpList = new ArrayList<String>();
                tmpList.add(nrw.getFahrgastGeburtsdatum(true).toString());
                result.add(new Pair<String,List<String>>("Geburtsdatum:", tmpList));
                
                tmpList = new ArrayList<String>();
                tmpList.add(b.getVerzTeil().getVerzStatTeil().getBerechtigungId().getOrganisationsName());
                result.add(new Pair<String,List<String>>("Herausgeber:", tmpList));
                
                tmpList = new ArrayList<String>();
                tmpList.add(b.getVerzTeil().getVerzStatTeil().getProduktId().getOrganisationsName());
                result.add(new Pair<String,List<String>>("Produktherausgeber:", tmpList));
            } else {
                List<String> tmpList = new ArrayList<String>();
                tmpList.add(b.getVerzTeil().getVerzStatTeil().getProduktId().getProductname());
                result.add(new Pair<String, List<String>>("Produkt:", tmpList));
                
                tmpList = new ArrayList<String>();
                tmpList.add(b.getVerzTeil().getVerzStatTeil().getBerechtigungId().getOrganisationsName());
                result.add(new Pair<String,List<String>>("Herausgeber:", tmpList));
                
                tmpList = new ArrayList<String>();
                tmpList.add(b.getVerzTeil().getVerzStatTeil().getProduktId().getOrganisationsName());
                result.add(new Pair<String,List<String>>("Produktherausgeber:", tmpList));
                
//                for (Pair<String, List<String>> p : b.getVerzTeil().getVerzStatTeil().getProduktId().getData(true)) {
//                    p.setValueA("Produkt:");
//                    result.add(p);
//                }
            
                if (efs != null) {
                    OrtID ort = efs.getStartOrt();
                    if (ort.isValid()) {
                        tmpList = new ArrayList<String>();
                        String tmpStr = ort.getOrtTypBezeichnung() + " " + ort.getOrt().getBezeichnung();
                        tmpStr += " (" + Integer.toString(ort.getOrtNummer()) + "/ " + ort.getOrganisationsName() + ")";
                        tmpList.add(tmpStr);
                        result.add(new Pair<String,List<String>>("Start:", tmpList));
                    }
                    ort = efs.getViaOrt1();
                    String viaStr = "";
                    if (ort.isValid()) {
                        if (!viaStr.isEmpty()) {
                            viaStr += "; ";
                        }
                        viaStr += ort.getOrtTypBezeichnung() + " " + ort.getOrt().getBezeichnung();
                        viaStr += " (" + Integer.toString(ort.getOrtNummer()) + "/ " + ort.getOrganisationsName() + ")";
                    }
                    ort = efs.getViaOrt2();
                    if (ort.isValid()) {
                        if (!viaStr.isEmpty()) {
                            viaStr += "; ";
                        }
                        viaStr += ort.getOrtTypBezeichnung() + " " + ort.getOrt().getBezeichnung();
                        viaStr += " (" + Integer.toString(ort.getOrtNummer()) + "/ " + ort.getOrganisationsName() + ")";
                    }
                    ort = efs.getViaOrt3();
                    if (ort.isValid()) {
                        if (!viaStr.isEmpty()) {
                            viaStr += "; ";
                        }
                        viaStr += ort.getOrtTypBezeichnung() + " " + ort.getOrt().getBezeichnung();
                        viaStr += " (" + Integer.toString(ort.getOrtNummer()) + "/ " + ort.getOrganisationsName() + ")";
                    }
                    if (!viaStr.isEmpty()) {
                        tmpList = new ArrayList<String>();
                        tmpList.add(viaStr);
                        result.add(new Pair<String,List<String>>("Via:", tmpList));
                    }
                    ort = efs.getZielOrt();
                    if (ort.isValid()) {
                        tmpList = new ArrayList<String>();
                        String tmpStr = ort.getOrtTypBezeichnung() + " " + ort.getOrt().getBezeichnung();
                        tmpStr += " (" + Integer.toString(ort.getOrtNummer()) + "/ " + ort.getOrganisationsName() + ")";
                        tmpList.add(tmpStr);
                        result.add(new Pair<String,List<String>>("Ziel:", tmpList));
                    }
                }
                
                for (Pair<String, List<String>> p : b.getVerzTeil().getVerzStatTeil().getGueltBeginn().getData(true)) {
                    p.setValueA("Gültig ab:");
                    result.add(p);
                }
                for (Pair<String, List<String>> p : b.getVerzTeil().getVerzStatTeil().getGueltEnde().getData(true)) {
                    p.setValueA("Gültig bis:");
                    result.add(p);
                }
            
                for (Pair<String, List<String>> p : b.getVerzTeil().getVerzStatTeil().getBerechtigungId().getData(true)) {
                    p.setValueA("Fahrschein:");
                    result.add(p);
                }
            
            //result.add(new Pair<String, List<String>>(tltName, emptyList));
                result.addAll(b.getSepDataTeil().getProdSpezTeil().getSpezifischerTeil().getData(true));
            }
//            tmpList = new ArrayList<String>();
//            tmpList.add(b.getVerzTeil().getVerzStatTeil().getBerechtigungId().getOrganisationsName());
//            result.add(new Pair<String,List<String>>("Herausgeber:",tmpList));
//            
//            tmpList = new ArrayList<String>();
//            tmpList.add(b.getVerzTeil().getVerzStatTeil().getProduktId().getOrganisationsName());
//            result.add(new Pair<String,List<String>>("Produktverantwortlicher:",tmpList));
            
            result.add(new Pair<String, List<String>>("", emptyList));
        }

        if (lfdnr == 0) {
            List<String> tmpList = new ArrayList<String>();
            tmpList.add(null);
            result.add(new Pair<String,List<String>>("Keine gültigen Tickets gefunden", tmpList));
        }

        if (custOK && mShowCustomer) {
            List<String> tmpList = new ArrayList<String>();
            tmpList.add(null);
            result.add(new Pair<String, List<String>>("", emptyList));
            result.add(new Pair<String,List<String>>("Kundendaten", tmpList));

            tmpList = new ArrayList<String>();
            tmpList.add(app.getKundendaten().getSepDataTeil().getKundeName().toString().trim());
            result.add(new Pair<String, List<String>>("Name:", tmpList));
            
            tmpList = new ArrayList<String>();
            tmpList.add(app.getKundendaten().getSepDataTeil().getKundeVorname().toString().trim());
            result.add(new Pair<String, List<String>>("Name:", tmpList));
            
            result.addAll(app.getKundendaten().getSepDataTeil().getKundeSprache().getData(true));
            tmpList = new ArrayList<String>();
            tmpList.add(app.getKundendaten().getSepDataTeil().getKundeGeburtsdatum().toString());
            result.add(new Pair<String, List<String>>("Geburtsdatum:", tmpList));
            result.addAll(app.getKundendaten().getSepDataTeil().getKundeGeschlecht().getData(true));
        }

        return TLVParser.dataToTable(result);
    }

    private String[][] ErstelleExpertenTabelle(Applikationsverzeichnis app) {
        Queue<Pair<String, List<String>>> result =
                new LinkedList<Pair<String, List<String>>>();
        
        Pair<String,List<String>> emptyPair = new Pair<String,List<String>>("", new ArrayList<String>());
        emptyPair.getValueB().add("");
        
        List<String> header = new ArrayList<String>();
        header.add(null);
        header.add(null);
        
        result.add(new Pair<String,List<String>>("Prioritäten", header));
        result.addAll(app.getPrios().getData(false));
        result.add(emptyPair);
        
        result.add(new Pair<String,List<String>>("Letzte Transaktion", header));
        result.addAll(app.getLetzteTrans().getData(false));
        result.add(emptyPair);
        
        
        if (appOK) {
            result.add(new Pair<String,List<String>>("Applikation", header));
            result.addAll(app.getApplikation().getData(false));
            result.add(emptyPair);
        }

        if (logOK) {
            result.add(new Pair<String,List<String>>("Applikationslogbuch", header));
            result.addAll(app.getLogbuch().getData(false));
            result.add(emptyPair);
        }

        if (custOK) {
            result.add(new Pair<String,List<String>>("Kundendaten", header));
            result.addAll(app.getKundendaten().getData(false));
            result.add(emptyPair);
        }

        if (keystoreOK) {
            if (app.getVersion() == KAVersion.V_1108 || app.getVersion().compare(KAVersion.V_1200) >= 0) {
                result.add(new Pair<String,List<String>>("Schlüsselspeicher", header));
                result.addAll(app.getKeystore().getData(false));
            } else {
                result.add(new Pair<String,List<String>>("WES", header));
                result.addAll(app.getWes().getData(false));
            }
            result.add(emptyPair);
        }

        result.add(new Pair<String,List<String>>("Berechtigungen", header));        
        for (int i = 0; i < app.getBerechtigungen().size(); ++i) {
            result.addAll(app.getBerechtigung(i).getData(false));
            result.add(emptyPair);
        }
        
        return TLVParser.dataToTable(result);
    }

    private String[] normalColHeaders = new String[]{"Bezeichnung", "Wert"};
    private String[] expertColHeaders = new String[]{"Bezeichnung", "Wert", "Raw-Daten"};
    /**
     * Erstellt den Tabelleninhalt.
     */
    private void buildTable() {
        if (this.normalTableInhalt != null && this.expertTableInhalt != null) {
            DefaultTableModel normaldtm =
                    new DefaultTableModel(this.normalTableInhalt, normalColHeaders) {

                        @Override
                        public Class<?> getColumnClass(final int columnIndex) {
                            return String.class;
                        }

                        @Override
                        public boolean isCellEditable(final int nRow, final int nCol) {
                            return false;
                        }
                    };
//            normalScrollPane.setVisible(true);
            normalTable.setModel(normaldtm);
            ((MultiSelectTable)normalTable).setSelectEnabled(pinOK);
            int optimalsize = sizeColumnOptimal(0, 0, this.normalTable.getRowCount(), normalTable, 50);
            this.normalTable.getColumnModel().getColumn(1).setCellRenderer(UmbruchTableCellRenderer.getInstance());
            this.normalTable.getColumnModel().getColumn(0).setCellRenderer(NormalTableCellRenderer.getInstance());
            normalTable.getColumnModel().getColumn(0).setMaxWidth(this.mainPanel.getWidth() / 2);
            normalTable.repaint();

            
            DefaultTableModel expertdtm =
                    new DefaultTableModel(this.expertTableInhalt,
                    expertColHeaders) {
                        private static final long serialVersionUID = 1L;

                        @Override
                        public Class<?> getColumnClass(final int columnIndex) {
                            return String.class;
                        }

                        @Override
                        public boolean isCellEditable(final int nRow, final int nCol) {
                            return false;
                        }
                    };
            expertTable.setModel(expertdtm);
            sizeColumnOptimal(0, 0, this.expertTable.getRowCount(), expertTable, 50);
            this.expertTable.getColumnModel().getColumn(2).setCellRenderer(UmbruchTableCellRenderer.getInstance());
            this.expertTable.getColumnModel().getColumn(1).setCellRenderer(UmbruchTableCellRenderer.getInstance());
            this.expertTable.getColumnModel().getColumn(0).setCellRenderer(NormalTableCellRenderer.getInstance());
            //expertTable.getColumnModel().getColumn(0).setMaxWidth(this.expertScrollPane.getWidth() / 2);
            expertTable.repaint();

            this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }
    }

    /**
     * Stellt für eine Spalte eine optimale Breite ein.
     * @param index der Index der Spalte
     * @param fromRow die erste zu optimierende Zeile
     * @param toRow die letzte zu optimierende Zeile
     * @param table Tabelle für welche die Optimale Spaltenbreite errechnet werden soll.
     * @param offset Offset welches noch zur Spaltenbreite addiert wird (optik).
     * @return die ermittelte Breite
     */
    private int sizeColumnOptimal(int index, int fromRow, int toRow, JTable table, int offset) {
        TableColumn col = table.getColumnModel().getColumn(index);
        int prefSize = 0;

        if (col.getHeaderRenderer() != null) {
            prefSize = col.getHeaderRenderer().getTableCellRendererComponent(
                    null,
                    col.getHeaderValue(),
                    false,
                    false,
                    0,
                    0).getPreferredSize().width;
        } else {
            prefSize = table.getDefaultRenderer(String.class).getTableCellRendererComponent(
                    table,
                    col.getHeaderValue(),
                    false,
                    false,
                    0,
                    0).getPreferredSize().width;
        }

        for (int rowIndex = fromRow; rowIndex < toRow; rowIndex++) {
            prefSize = Math.max(prefSize,
                    table.getCellRenderer(rowIndex, index).getTableCellRendererComponent(
                    table,
                    table.getValueAt(rowIndex, index),
                    false,
                    false,
                    rowIndex,
                    index).getPreferredSize().width);
        }

        // Für die Optik
        prefSize += table.getIntercellSpacing().width;
        prefSize += offset;

        col.setPreferredWidth(prefSize);
        col.setMinWidth(prefSize);
        col.setWidth(prefSize);

        return prefSize;
    }

    private void openEFMFolder() {
        this.folderChooser.setDialogTitle("EFM-Daten");
        if (this.folderChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            this.efmFilePfad = this.folderChooser.getSelectedFile().getAbsolutePath();
            EfmPvDatei.DeleteBekanntePVs();
            if (this.saxParser.FindEfmXmlFiles(efmFilePfad) > 0) {
                return;
            }
            File f = new File(efmFilePfad);
            if (!f.exists()) {
                JOptionPane.showMessageDialog(null,
                        "Das Verzeichnis: \"" + efmFilePfad + "\" existiert nicht!", "Warnung",
                        JOptionPane.WARNING_MESSAGE);
                LOGGER.log(Level.SEVERE, "Keine EFM-Daten in {0} gefunden: Datei existiert nicht!", f.getAbsolutePath());
            } else if (!f.isDirectory()) {
                JOptionPane.showMessageDialog(null,
                        "\"" + efmFilePfad + "\" ist kein Verzeichnis!", "Warnung",
                        JOptionPane.WARNING_MESSAGE);
                LOGGER.log(Level.SEVERE, "Keine EFM-Daten in {0} gefunden: Ist kein Verzeichnis!", f.getAbsolutePath());
            } else if (!f.canRead()) {
                JOptionPane.showMessageDialog(null,
                        "Das Verzeichnis: \"" + efmFilePfad + "\" ist lesegeschützt!", "Warnung",
                        JOptionPane.WARNING_MESSAGE);
                LOGGER.log(Level.SEVERE, "Keine EFM-Daten in {0} gefunden: Verzeichnis kann nicht gelesen werden!", f.getAbsolutePath());
            } else {
                JOptionPane.showMessageDialog(null,
                        "Es wurden keine EFM-Xml-Dateien im Verzeichnis: \"" + efmFilePfad + "\" gefunden!", "Warnung",
                        JOptionPane.WARNING_MESSAGE);
                LOGGER.log(Level.SEVERE, "Keine EFM-Daten in {0} gefunden: Keine passenden XML-Dateien gefunden!", f.getAbsolutePath());
            }
            openEFMFolder();
            if (this.efmFilePfad.isEmpty() || EfmPvDatei.HasNoFiles()) {
                JOptionPane.showMessageDialog(null,
                        "Es wurde kein valider Pfad zu EFM-Daten angegeben.\n"
                      + "Deshalb werden keine EFM-Daten zur Dekodierung der ausgelesenen Daten verwendet.\n"
                      + "Die Ausgabe ist daher lückenhaft und uninterpretiert!", "Warnung",
                        JOptionPane.WARNING_MESSAGE);
                LOGGER.info("Kein Verzeichnis für EFM-Daten angegeben");
            }
        }
    }

    /**
     * Laden der FileFilter für Laden.
     * Wegen unterschiedlicher Datenunterstützung.
     */
    private void ladeLoadFileFilter() {
        if (!loadFileFilter) {
            loadFileChooser.addChoosableFileFilter(utfFileFilter);
            loadFileChooser.addChoosableFileFilter(nmFileFilter);
            
            loadFileChooser.setFileFilter(nmFileFilter);
            
            loadFileFilter = true;
        }
    }

    /**
     * Laden der FileFilter für Speichern.
     * Wegen unterschiedlicher Datenunterstützung.
     */
    private void ladeSaveFileFilter() {
        if (!saveFileFilter) {
//            saveFileChooser.addChoosableFileFilter(excelFileFilter);
            saveFileChooser.addChoosableFileFilter(utfFileFilter);
            saveFileChooser.addChoosableFileFilter(nmFileFilter);
            
            saveFileChooser.setAcceptAllFileFilterUsed(false);      
            saveFileChooser.setFileFilter(nmFileFilter);
            
            saveFileFilter = true;
        }
    }

    /**
     * Blendet ProgressBar ein und aus.
     */
    private void switchProgressBar(boolean viewable) {
        sucheProgressBar.setVisible(viewable);
        sucheProgressBar.updateUI();
    }

    private boolean readSam = false;
    private ISAM sam = null;
    
    /**
     * Füllen der Progressbar und
     * abfrage ob sich eine Karte im RFID-feld befindet.
     */
    private void progressBar() {
        final int progressBarMaximum = 100;
        final int step = 10;
        sucheProgressBar.setMaximum(progressBarMaximum);
        sucheProgressBar.setValue(0);
//        switchProgressBar(true);
        sci.disconnect();
        sci = new SCConnection(null);
        if (!sci.connectToTerminal((readSam) ? this.samReaderName : this.readerName)) {
            JOptionPane.showMessageDialog(null,
                    "Es konnte keine Verbindung zum Lesegerät: \"" + readerName + "\" hergestellt werden.\n"
                  + "Der Lesevorgang wird abgebrochen!", "Warnung",
                    JOptionPane.WARNING_MESSAGE);
            LOGGER.log(Level.SEVERE, "Es konnte keine Verbindung zum Leseger\u00e4t: {0}hergestellt werden", readerName);
            readerRefreshButtonActionPerformed(null);
//            switchProgressBar(false);
            return;
        } else {
            if (readSam) {
                sam = new SAM(sci);
            } else {
                kum.setReader(sci);
            }
        }
        timer = new Timer(INTERVALL, new ActionListener() {

            private int value = 0;

            private void startReading() {
                LOGGER.info("Smartcard gefunden.");
                timer.stop();
                sucheProgressBar.setValue(0);
//                switchProgressBar(false);
                if (readSam) {
                    ReadSAM();
                } else {
                    ReadKundenMedium();
                }
            }
            
            private void stopScanning() {
                timer.stop();
                sucheProgressBar.setValue(0);
//                switchProgressBar(false);
                viewerState = ViewerState.READERMODE_K;
                checkGUI();
                app = new Applikationsverzeichnis();
                JOptionPane.showMessageDialog(null,
                        "Es konnte keine Karte gefunden werden!\n"
                      + "Zum Lesen wurde das Gerät: \"" + readerName + "\" verwendet.\n"
                      + "Bitte prüfen Sie, dass die Karte korrekt an das Lesegerät gehalten wird\n"
                      + "und sich weder andere Karte noch störende Geräte oder Gegenstände\n"
                      + "in der Nähe des Lesegerätes befinden.", "Info",
                        JOptionPane.INFORMATION_MESSAGE);
            }

            public void actionPerformed(ActionEvent e) {
                if ((sci.isConnected() || sci.waitForCard(500)) && sci.isPresent()) {
                    startReading();
                } else {
                    value += step;
                    if (value > sucheProgressBar.getMaximum()) {
                        if (!mProfessional || readSam) {
                            stopScanning();
                            return;
                        } else {
                            value = 0;
                        }
                    }
                    sucheProgressBar.setValue(value);
                }
            }
        });
        timer.setInitialDelay(0);
        timer.start();
    }

    /**
     * Erstellt dass Drop-Down der Angeschlossenen Reader mit den im SmartCardInterface angegebenen
     * Lesegeräten und markiert das Lesegerät das zu "readerName" passt als augewählt.
     *
     * @param rebuild true: das Menü wird komplett neu erstellt. false: die Elemente des Menüs werden nur entsprechend der Auswahl geändert.
     */
    private void buildReaderDropDown(boolean rebuild) {
        if (rebuild) {
            this.readerMenu.removeAll();
            this.samReaderMenu.removeAll();
            boolean selected = false;
            boolean selectedSam = false;
            for (Object reader : PCSC.getAvailableTerminals()) {
                CardTerminal ct = (CardTerminal) reader;
                JCheckBoxMenuItem item = new JCheckBoxMenuItem(ct.getName());
                JCheckBoxMenuItem samItem = new JCheckBoxMenuItem(ct.getName());
                item.setName(ct.toString());
                samItem.setName(ct.toString());
                if (!selected && readerName.equals(item.getName())) {
                    item.setSelected(true);
                    selected = true;
                    samItem.setEnabled(false);
                    LOGGER.log(Level.INFO, "Verwende Leseger\u00e4t {0}", readerName);
                } else if (mUseSam && !selectedSam && samReaderName.equals(samItem.getName())) {
                    samItem.setSelected(true);
                    selectedSam = true;
                    item.setEnabled(false);
                    LOGGER.log(Level.INFO, "Verwende SAM-Leseger\u00e4t {0}", samReaderName);
                }
                item.addItemListener(readerItemList);
                samItem.addItemListener(samReaderItemList);
                this.readerMenu.add(item);
                this.samReaderMenu.add(samItem);
            }
            if (!selected) {
                if (readerMenu.getItemCount() > 0) {
                    readerName = readerMenu.getItem(0).getName();
                    readerMenu.getItem(0).setSelected(true);
                    samReaderMenu.getItem(0).setEnabled(false);
                    LOGGER.log(Level.INFO, "Verwende Leseger\u00e4t {0}", readerName);
                } else {
                    LOGGER.info("Keine Lesegeräte angeschlossen");
                }
            }
        } else {
            for (int i = 0; i < this.readerMenu.getItemCount(); i++) {
                if (this.readerName.equals(this.readerMenu.getItem(i).getName())) {
                    if (!readerMenu.getItem(i).isSelected()) {
                        this.readerMenu.getItem(i).setSelected(true);
                        this.samReaderMenu.getItem(i).setSelected(false);
                    }
                    this.readerMenu.getItem(i).setEnabled(true);
                    this.samReaderMenu.getItem(i).setEnabled(false);
                } else {
                    if (mUseSam && this.samReaderName.equals(this.samReaderMenu.getItem(i).getName())) {
                        if (!samReaderMenu.getItem(i).isSelected()) {
                            this.samReaderMenu.getItem(i).setSelected(true);
                            this.readerMenu.getItem(i).setSelected(false);
                        }
                        this.samReaderMenu.getItem(i).setEnabled(true);
                        this.readerMenu.getItem(i).setEnabled(false);
                    } else {
                        this.readerMenu.getItem(i).setEnabled(true);
                        this.samReaderMenu.getItem(i).setEnabled(true);
                        if (readerMenu.getItem(i).isSelected()) {
                            this.readerMenu.getItem(i).setSelected(false);
                        }
                        if (samReaderMenu.getItem(i).isSelected()) {
                            this.samReaderMenu.getItem(i).setSelected(false);
                        }
                    }
                }
            }
        }
        checkGUI();
    }

    private void buildTemplateDropDown() {
        this.TemplatesMenu.removeAll();
        for (Entry<String, Template> entry : TemplateProvider.getInstance().getTemplates().entrySet()) {
            JMenuItem item = new JMenuItem(entry.getKey());
            item.setName(entry.getKey());
            item.addActionListener(templateActionListener);
            this.TemplatesMenu.add(item);
        }
    }

    /**
     * Zeigt einen Abfragedialog an, ob die Anwendung wirklich beendet werden soll.
     * Falls dies bestätigt wird, werden die Einstellungen gespeichert, die Logdatei (falls vorhanden)
     * geschlossen und das Programm beendet.
     */
    private void beenden() {
        int beenden = JOptionPane.showConfirmDialog(Ticketinfo.this,
                "Möchten Sie die Anwendung jetzt beenden?", "Beenden", JOptionPane.YES_NO_OPTION);
        if (beenden == 0) {
            savePreferences();
            LOGGER.info("Das Programm wird beendet");
            FileLogManager.getInstance().disableFileLogging();
            System.exit(0);
        }
    }

    /**
     * Diese Funktion setzt die Grundeinstellungen des Programms. Die Grundeinstellungen sind folgende:
     *
     * <ul>
     *   <li>
     */
    private void grundeinstellung() {
        this.viewerState = viewerState.VIEWONLY;
        this.expertTable.setDefaultRenderer(String.class, NormalTableCellRenderer.getInstance());
        //this.persDatenMenuItem.setEnabled(false);
        //this.logoALLabel.setVisible(false);
        //Setzen des Icons der Applikation. (Oben links)
        setIconImage(Toolkit.getDefaultToolkit().getImage(getClass().getResource("/de/alag/eticket/ticketinfo/resources/read.png")));
        checkGUI();
        
        this.centerWindow(this);
        this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        //Close-Funktion ändern
        this.addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent we) {
                beenden();
            }
        });

        this.setFocusable(true);

        addKeyListener(new KeyAdapter() {

            @Override
            public void keyPressed(KeyEvent event) {
//                if (event.isAltDown() && event.isControlDown() && event.isShiftDown() && event.getKeyCode() == KeyEvent.VK_P) {
//                    checkBoxProfi.setSelected(!checkBoxProfi.isSelected());
//                }
                LOGGER.info("KEYLISTENER called!");
                if (event.isAltDown() && event.isControlDown() && event.isShiftDown() && event.getKeyCode() == KeyEvent.VK_H) {
                    hexdumpStateW = !hexdumpStateW;
                    kum.setWriteHexDump(hexdumpStateW);
                    if (hexdumpStateW) {
                        LOGGER.info("HexDump-Ausgabe aktiviert");
                        if (hexdumpStateR) {
                            hexdumpStateR = false;
                            kum.setReadFromHexDump(false);
                            LOGGER.info("HexDump-Einlesen deaktiviert");
                        }
                    } else {
                        LOGGER.info("HexDump-Ausgabe deaktiviert");
                    }
                }

                if (event.isAltDown() && event.isControlDown() && event.isShiftDown() && event.getKeyCode() == KeyEvent.VK_C) {
                    hexdumpStateR = !hexdumpStateR;
                    kum.setReadFromHexDump(hexdumpStateR);
                    if (hexdumpStateR) {
                        LOGGER.info("HexDump-Einlesen aktiviert");
                        if (hexdumpStateW) {
                            hexdumpStateW = false;
                            kum.setWriteHexDump(false);
                            LOGGER.info("HexDump-Ausgabe deaktiviert");
                        }
                    } else {
                        LOGGER.info("HexDump-Einlesen deaktiviert");
                    }
                }
            }
        });
    }

    /**
     * Diese Funktion läd Einstellungen zum Logging (checkBoxLog), Lesegerät (readerName) und EFM-Verzeichnis (efmFilePfad) und setzt diese.
     */
    private void loadPreferences() {
        mProperties = mConfigurator.loadConfiguration();
        FileLogManager.getInstance().setLogFile(getConfig(CONFIG.LogFile));
        FileLogManager.getInstance().setFileLoggingLevel(Level.parse(getConfig(CONFIG.LogLevel)));
        checkBoxLog.setState(Boolean.parseBoolean(getConfig(CONFIG.LoggerState)));
        readerName = getConfig(CONFIG.KumReader);
        samReaderName = getConfig(CONFIG.SamReader);
        efmFilePfad = getConfig(CONFIG.EfmPath);
        if (checkBoxLog.getState()) {
            FileLogManager.getInstance().enableFileLogging(null, null, null);
        }
        String templateConfig = getConfig(CONFIG.Templates);
        if (templateConfig != null) {
            String[] lines = templateConfig.split("\n");
            if (lines == null) {
                return;
            }
            for (String line : lines) {
                if (line == null || line.trim().isEmpty()) {
                    continue;
                }
                String[] prods = line.split(";");
                if (prods != null && prods.length > 1) {
                    Template t = TemplateProvider.getInstance().getTemplate(prods[0]);
                    if (t == null) {
                        continue;
                    }
                    for (int i=1; i<prods.length; ++i) {
                        if (prods[i] == null || prods[i].trim().isEmpty()) {
                            continue;
                        }
                        byte[] raw = Converters.getBytesFromHexString(prods[i]);
                        if (raw == null) {
                            continue;
                        }
                        EfmProduktID pid = new EfmProduktID();
                        if (pid.unmarshal(raw, 0) == pid.getLength()) {
                            t.add(pid);
                        }
                    } 
                }
            }
        }
        for (Entry<String, Set<EfmProduktID>> e : mConfigurator.getWebTemplates().entrySet()) {
            Template t = TemplateProvider.getInstance().getTemplate(e.getKey());
            if (t == null) {
                continue;
            }
            for (EfmProduktID p : e.getValue()) {
                t.add(p);
            }
        }
    }

    /**
     * Diese Funktion speichert Einstellungen zum Logging (checkBoxLog), Lesegerät (readerName) und EFM-Verzeichnis (efmFilePfad) mittels eines PreferenceObjects.
     */
    private void savePreferences() {
        setConfig(CONFIG.KumReader, readerName);
        setConfig(CONFIG.SamReader, samReaderName);
        setConfig(CONFIG.LoggerState, Boolean.toString(checkBoxLog.isSelected()));
        setConfig(CONFIG.EfmPath, efmFilePfad);
        StringBuilder templateConfig = new StringBuilder();
        for (Entry<String, Template> entry : TemplateProvider.getInstance().getTemplates().entrySet()) {
            templateConfig.append(entry.getKey());
            for (EfmProduktID prodId : entry.getValue().getProducts()) {
                templateConfig.append(";").append(Converters.bytesToHexString(prodId.toRaw()));
            }
            templateConfig.append("\n");
        }
        setConfig(CONFIG.Templates, templateConfig.toString());
        mConfigurator.storeConfiguration(mProperties);
    }

    /**
     * Diese Funktion zentriert das übergebene Fenster in der Bildschirmmitte.
     * @param window Zu zentrierendes Fenster.
     */
    private void centerWindow(final Window window) {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = window.getSize();
        if (frameSize.height > screenSize.height) {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width) {
            frameSize.width = screenSize.width;
        }
        window.setLocation((screenSize.width - frameSize.width) / 2,
                (screenSize.height - frameSize.height) / 2);
    }

    /**
     * Kodierte Zustände, die die Ansicht der Karteninformationen steuern.
     */
    private enum AnsichtState {

        /**
         * Die normale Ansicht für den Standardanwender
         */
        NORMAL,
        /**
         * Die Expertenansicht mit mehr Informationenen zur Karte und weniger Informationen aus den EFM-Daten
         */
        EXPERT
    }

    /**
     * Kodierte Zustände der GUI, die die Programmfunktionalität steuern.
     */
    private enum ViewerState {

        /**
         * Volle Funktionalität.
         */
        READERMODE_K,
        
        /**
         * Eingeschränkte Funktionalität.
         */
        VIEWONLY,
        
        /*
         * Beschäftigt, Lesend
         */
        READING,
        
        SCANNING
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenu TemplatesMenu;
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JMenu ansichtMenu;
    private javax.swing.JButton buttonAktualisieren;
    private javax.swing.JCheckBoxMenuItem checkBoxLog;
    private javax.swing.JLabel currentItemLabel;
    private javax.swing.JButton downButton;
    private javax.swing.JMenuItem druckMenuItem;
    private javax.swing.JButton druckenButton;
    private javax.swing.JMenuItem efmFolderMenuItem;
    private javax.swing.JTextArea errorTextArea;
    private javax.swing.JButton exitButton;
    private javax.swing.JRadioButtonMenuItem expertRadioButtonMenuItem;
    private javax.swing.JTable expertTable;
    private javax.swing.JFileChooser folderChooser;
    private javax.swing.JMenuItem helpMenuItem;
    private javax.swing.JMenu help_aboutMenu;
    private javax.swing.JButton hilfeButton;
    private javax.swing.JButton jButtonBackward;
    private javax.swing.JButton jButtonForward;
    private javax.swing.JCheckBoxMenuItem jCheckBoxMenuItem_showCustomer;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JPopupMenu.Separator jSeparator1;
    private javax.swing.JPopupMenu.Separator jSeparator2;
    private javax.swing.JPopupMenu.Separator jSeparator3;
    private javax.swing.JPopupMenu.Separator jSeparator4;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JLabel labelLabel;
    private javax.swing.JButton ladenButton;
    private javax.swing.JMenuItem ladenMenuItem;
    private javax.swing.JFileChooser loadFileChooser;
    private javax.swing.JButton loadKeyButton;
    private javax.swing.JMenuItem loadParadigmMenuItem;
    private javax.swing.JLabel logoALLabel;
    private javax.swing.JScrollPane lowerScrollPane;
    private javax.swing.JScrollPane mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JTable normalTable;
    private javax.swing.JMenu optionMenu;
    private javax.swing.JPopupMenu.Separator optionSeparator;
    private javax.swing.JPanel profBottomPanel;
    private javax.swing.JMenuItem profPropertiesMenuItem;
    private javax.swing.JSplitPane profSplittPane;
    private javax.swing.JCheckBoxMenuItem professionalEnableMenuItem;
    private javax.swing.JMenu professionalMenu;
    private javax.swing.JPanel professionalPanel;
    private javax.swing.JButton readButton;
    private javax.swing.JMenuItem readMenuItem;
    private javax.swing.JLabel readParadigmLabel;
    private javax.swing.JMenuItem readParadigmMenuItem;
    private javax.swing.JButton readSamButton;
    private javax.swing.JMenuItem readSamMenuItem;
    private javax.swing.JMenu readerMenu;
    private javax.swing.JButton readerRefreshButton;
    private javax.swing.JMenu samReaderMenu;
    private javax.swing.JButton saveButton;
    private javax.swing.JFileChooser saveFileChooser;
    private javax.swing.JMenuItem saveParadigmMenuItem;
    private javax.swing.JMenuItem schriftGroesserMenuItem;
    private javax.swing.JMenuItem schriftKleinerMenuItem;
    private javax.swing.JMenu schriftMenu;
    private javax.swing.JMenuItem speicherMenuItem;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JMenuItem stopNMCompareMenuItem;
    private javax.swing.JProgressBar sucheProgressBar;
    private javax.swing.JTextArea textAreaLabel;
    private javax.swing.JButton upButton;
    private javax.swing.JMenuItem updateMenuItem;
    private javax.swing.JScrollPane upperScrollPane;
    // End of variables declaration//GEN-END:variables
    /**
     * Timer der Progressbar.
     */
    private Timer timer;
    /**
     * Intervall für den Timer der Progressbar.
     */
    private static final int INTERVALL = 1000;
    /**
     * Tabelleninhalt der Normalansicht.
     */
    private String[][] normalTableInhalt = null;
    /**
     * Tabelleninhalt der Expertenansicht.
     */
    private String[][] expertTableInhalt = null;
    /**
     * Momentan verwendete Ansicht.
     */
    //private AnsichtState ansichtState = AnsichtState.NORMAL;
    /**
     * Momentan verwendete Programmkonfiguration.
     */
    private ViewerState viewerState;
    /**
     * Name des gewählten Lesegerätes.
     */
    private String readerName = "PC/SC terminal SCM Microsystems Inc. SCL01x Contactless Reader 0";
    /**
     * Name des gewählten Lesegerätes.
     */
    private String samReaderName = "";
    /**
     * Filter für die Anzeige des Datei-Lade-Dialogs.
     */
    private boolean loadFileFilter = false;
    /**
     * Filter für die Anzeige des Datei-Speicher-Dialogs.
     */
    private boolean saveFileFilter = false;
    /**
     * Verwendeter Pfad zum EFM-Daten-Verzeichnis.
     */
    private String efmFilePfad = ".";
    /**
     * Variable die angibt, ob alle vom Transponder gelesenen Bytes in die Hexdump-Datei geschrieben werden sollen.
     */
    private boolean hexdumpStateW = false;
    /**
     * Variable die angibt, ob die Kommunikation mit einem Transponder durch eine Hexdump-Datei simuliert werden soll.
     */
    private boolean hexdumpStateR = false;
    /**
     * Die Instanz des Info-Fensters.
     */
    private JDialog TicketinfoAbout;
    /**
     * AktionListener für Laden.
     */
    private ActionListener ladenActionListener = new ActionListener() {

        public void actionPerformed(ActionEvent evt) {
            ladenMenuItemActionPerformed(evt);
        }
    };
    /**
     * AktionListener für Speichern.
     */
    private ActionListener speichernActionListener = new ActionListener() {

        public void actionPerformed(ActionEvent evt) {
            speicherMenuItemActionPerformed(evt);
        }
    };
    
    private ActionListener templateActionListener = new ActionListener() {

        public void actionPerformed(ActionEvent e) {
            TemplateMenuItemActionPerformed(e);
        }
    };
    
    /**
     * ItemListener der bei Auswahl eines Lesegerätes aus der Liste aktiviert wird und die Liste entsprechend
     * der Auswahl ändert (Nur ein Lesegerät kann ausgewählt sein). Das gewählte Element wird ausgewählt, alle anderen
     * Elemente werden abgewählt.
     */
    private ItemListener readerItemList = new ItemListener() {

        public void itemStateChanged(ItemEvent e) {
            JCheckBoxMenuItem itemBuffer = (JCheckBoxMenuItem) e.getItem();
            if (!itemBuffer.isSelected()) { // Das Item war angewählt
                if (readerName.equals(itemBuffer.getName())) {
                    itemBuffer.setSelected(true);
                }
            } else { // Das Item war nicht ausgewählt
                readerName = itemBuffer.getName();
                if (samReaderName != null && readerName.equals(samReaderName)) {
                    samReaderName = "";
                }
                buildReaderDropDown(false);
            }
        }
    };

    /**
     * ItemListener der bei Auswahl eines Lesegerätes aus der Liste aktiviert wird und die Liste entsprechend
     * der Auswahl ändert (Nur ein Lesegerät kann ausgewählt sein). Das gewählte Element wird ausgewählt, alle anderen
     * Elemente werden abgewählt.
     */
    private ItemListener samReaderItemList = new ItemListener() {

        public void itemStateChanged(ItemEvent e) {
            JCheckBoxMenuItem itemBuffer = (JCheckBoxMenuItem) e.getItem();
            if (!itemBuffer.isSelected()) { // Das Item war angewählt
                if (mUseSam && samReaderName.equals(itemBuffer.getName())) {
                    samReaderName = "";
                }
                itemBuffer.setSelected(false);
                buildReaderDropDown(false);
            } else { // Das Item war nicht ausgewählt
                samReaderName = itemBuffer.getName();
                buildReaderDropDown(false);
            }
        }
    };
    
    
    private void TemplateMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        try {
            JMenuItem mi = (JMenuItem) evt.getSource();
            TemplateEditor te = new TemplateEditor(this, true, TemplateProvider.getInstance().getTemplate(mi.getName()));
            te.setLocationRelativeTo(this);
            te.setVisible(true);
        } catch (Exception ex) {
            LOGGER.log(Level.WARNING, ex.getMessage());
        }
    }
    
}
