//  File:        TicketinfoAbout.java
//  Copyright:   Copyright (C) 2010 AL ENGINEERING AG. All rights reserved.
//  Website:     http://www.al-ag.de
//  Description:
//  Comments:
//  Last edit:   18.06.2013
//  Author:      AL ENGINEERING AG
//  E-mail:      info@al-ag.de
//
//   This file is part of d(((eti-API.
//
//   d(((eti-API is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   d(((eti-API is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; If not, see <http://www.gnu.org/licenses/> or
//   write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//   Boston, MA 02111-1307 USA.
package de.alag.eticket.ticketinfo;

import java.awt.Desktop;
import java.io.File;
import java.io.InputStream;
import java.net.URI;
import java.net.URL;
import java.util.jar.Manifest;

/**
 * About Box für den Viewer.
 * @author Frank Hamm, Alexander Schuetz
 */
public class TicketinfoAbout extends javax.swing.JDialog {

    private static String herausgeberUri = "http://www.vrr.de";
    private static String gpl3Uri = "http://www.gnu.org/licenses/gpl.html";
    private String herstellerUri = "http://www.al-ag.de";
    
    private boolean getManifestInfos() {
        String className = TicketinfoAbout.class.getSimpleName() + ".class";
        String classPath = TicketinfoAbout.class.getResource(className).toString();
        if (!classPath.startsWith("jar")) {
            // Class not from JAR
            String relativePath = TicketinfoAbout.class.getName().replace('.', File.separatorChar) + ".class";
            String classFolder = classPath.substring(0, classPath.length() - relativePath.length() - 1);
            String manifestPath = classFolder + "/META-INF/MANIFEST.MF";
            if (classFolder.endsWith("/classes")) {
                classFolder = classFolder.substring(0, classFolder.lastIndexOf("/classes"));
                manifestPath = classFolder + "/MANIFEST.MF";
            }
            return readManifest(manifestPath);
        } else {
            String manifestPath = classPath.substring(0, classPath.lastIndexOf("!") + 1) + "/META-INF/MANIFEST.MF";
            return readManifest(manifestPath);
        }
    }
    
    private boolean readManifest(String manifestPath) {
        boolean result = false;
        String tVers = null;
        String tTitl = null;
        String tVend = null;
        String tHome = null;
        Manifest manifest = null;
        try {
            InputStream is = getClass().getClassLoader().getResourceAsStream("/META-INF/MANIFEST.MF");
            if (is == null) {
                is = ClassLoader.getSystemClassLoader().getResourceAsStream("/META-INF/MANIFEST.MF");
            }
            if (is == null) {
                is = (new URL(manifestPath)).openStream();
            }
            if (is != null) {
                manifest = new Manifest();
                manifest.read(is);
            }
        } catch (Exception ex) {
            manifest = null;
        }
        if (manifest != null) {
            tTitl = manifest.getMainAttributes().getValue("Implementation-Title");
            tVers = manifest.getMainAttributes().getValue("Implementation-Version");
            tVend = manifest.getMainAttributes().getValue("Implementation-Vendor");
            tHome = manifest.getMainAttributes().getValue("Implementation-URL");
            result = true;
        }
        if (tVers != null) {
            this.appVersionLabel.setText(tVers);
        }
        if (tTitl != null) {
            this.titelLabel.setText(tTitl);
        }
        if (tVend != null) {
            this.herstellerLabel.setText(tVend);
        }
        if (tHome != null) {
            this.herstellerUri = tHome;
        }
        return result;
    }

    private boolean getPackageInfos() {
        boolean result = false;
        String tVers = null;
        String tTitl = null;
        String tVend = null;
        Package aPackage = null;
        try {
            aPackage = TicketinfoAbout.class.getPackage();
        } catch (Exception ex) {
            aPackage = null;
        }
        if (aPackage != null) {
            tTitl = aPackage.getImplementationTitle();
            tVers = aPackage.getImplementationVersion();
            tVend = aPackage.getImplementationVendor();
            result = true;
        }
        if (tVers != null) {
            this.appVersionLabel.setText(tVers);
        }
        if (tTitl != null) {
            this.titelLabel.setText(tTitl);
        }
        if (tVend != null) {
            this.herstellerLabel.setText(tVend);
        }
        return result;
    }

    /** Creates new form TicketinfoAbout */
    public TicketinfoAbout(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        if (!getManifestInfos()) {
            getPackageInfos();
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        closeButton = new javax.swing.JButton();
        logoLabel = new javax.swing.JLabel();
        titelLabel = new javax.swing.JLabel();
        versionLabel = new javax.swing.JLabel();
        appVersionLabel = new javax.swing.JLabel();
        herausgeberLabel = new javax.swing.JLabel();
        appHerausgeberLabel = new javax.swing.JLabel();
        beschreibungLabel = new javax.swing.JLabel();
        herstellerLabel = new javax.swing.JLabel();
        appHerstellerLabel = new javax.swing.JLabel();
        lizenzLabel = new javax.swing.JLabel();
        appLizenzLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Info");
        setName("Info"); // NOI18N
        setResizable(false);

        closeButton.setText("OK");
        closeButton.setName("closeButton"); // NOI18N
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });

        logoLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/de/alag/eticket/ticketinfo/resources/read.png"))); // NOI18N
        logoLabel.setName("logoLabel"); // NOI18N

        titelLabel.setFont(new java.awt.Font("Tahoma", 1, 15)); // NOI18N
        titelLabel.setText("eTicketinfo");
        titelLabel.setName("titelLabel"); // NOI18N

        versionLabel.setFont(new java.awt.Font("Tahoma", 1, 11)); // NOI18N
        versionLabel.setText("Version:");
        versionLabel.setName("versionLabel"); // NOI18N

        appVersionLabel.setText("unbekannt");
        appVersionLabel.setName("appVersionLabel"); // NOI18N

        herausgeberLabel.setFont(new java.awt.Font("Tahoma", 1, 11)); // NOI18N
        herausgeberLabel.setText("Herausgeber:");
        herausgeberLabel.setName("herausgeberLabel"); // NOI18N

        appHerausgeberLabel.setForeground(new java.awt.Color(0, 51, 255));
        appHerausgeberLabel.setText("<html><u>Verkehrsverbund Rhein-Ruhr AöR</u></html>");
        appHerausgeberLabel.setName("appHerausgeberLabel"); // NOI18N
        appHerausgeberLabel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                appHerausgeberLabelMouseClicked(evt);
            }
        });

        beschreibungLabel.setText("<html>Ein Programm zum Auslesen von Chipkarten<br>(genauer: Nutzermedien nach VDV-Kernapplikation)<br>und Anzeigen der elektronischen Tickets.</html>"); // NOI18N
        beschreibungLabel.setName("descriptionLabel"); // NOI18N

        herstellerLabel.setFont(new java.awt.Font("Tahoma", 1, 11)); // NOI18N
        herstellerLabel.setText("Hersteller:");
        herstellerLabel.setName("herstellerLabel"); // NOI18N

        appHerstellerLabel.setForeground(new java.awt.Color(0, 51, 255));
        appHerstellerLabel.setText("<html><u>AL-ENGINEERING AG</u></html>");
        appHerstellerLabel.setName("appHerstellerLabel"); // NOI18N
        appHerstellerLabel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                appALLabal(evt);
            }
        });

        lizenzLabel.setFont(new java.awt.Font("Tahoma", 1, 11)); // NOI18N
        lizenzLabel.setText("Lizenz:");
        lizenzLabel.setName("lizenzLabel"); // NOI18N

        appLizenzLabel.setForeground(new java.awt.Color(0, 51, 255));
        appLizenzLabel.setText("<html><u>GNU General Public License 3</u></html>");
        appLizenzLabel.setName("appLizenzLabel"); // NOI18N
        appLizenzLabel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                appLizenzLabelMouseClicked(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(logoLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 66, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(titelLabel)
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(versionLabel)
                                    .addComponent(herstellerLabel)
                                    .addComponent(herausgeberLabel)
                                    .addComponent(lizenzLabel))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(appHerausgeberLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(appVersionLabel)
                                    .addComponent(appHerstellerLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(appLizenzLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                            .addComponent(beschreibungLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 344, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(closeButton, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 112, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(logoLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 42, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(titelLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(beschreibungLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 77, Short.MAX_VALUE)
                        .addGap(18, 18, 18)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(versionLabel)
                            .addComponent(appVersionLabel))
                        .addGap(4, 4, 4)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(herstellerLabel)
                            .addComponent(appHerstellerLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(herausgeberLabel)
                            .addComponent(appHerausgeberLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(lizenzLabel)
                            .addComponent(appLizenzLabel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(closeButton)))
                .addContainerGap())
        );

        titelLabel.getAccessibleContext().setAccessibleName("eTicketinfo");

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
        this.dispose();
    }//GEN-LAST:event_closeButtonActionPerformed

    private void appALLabal(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_appALLabal
        if (Desktop.isDesktopSupported()) {
            try {
                URI uri = URI.create(herstellerUri);
                Desktop.getDesktop().browse(uri);
            } catch (Exception ex) { }
        }
    }//GEN-LAST:event_appALLabal

    private void appHerausgeberLabelMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_appHerausgeberLabelMouseClicked
        if (Desktop.isDesktopSupported()) {
            try {
                URI uri = URI.create(herausgeberUri);
                Desktop.getDesktop().browse(uri);
            } catch (Exception ex) { }
        }
    }//GEN-LAST:event_appHerausgeberLabelMouseClicked

    private void appLizenzLabelMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_appLizenzLabelMouseClicked
        if (Desktop.isDesktopSupported()) {
            try {
                URI uri = URI.create(gpl3Uri);
                Desktop.getDesktop().browse(uri);
            } catch (Exception ex) { }
        }
    }//GEN-LAST:event_appLizenzLabelMouseClicked

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel appHerausgeberLabel;
    private javax.swing.JLabel appHerstellerLabel;
    private javax.swing.JLabel appLizenzLabel;
    private javax.swing.JLabel appVersionLabel;
    private javax.swing.JLabel beschreibungLabel;
    private javax.swing.JButton closeButton;
    private javax.swing.JLabel herausgeberLabel;
    private javax.swing.JLabel herstellerLabel;
    private javax.swing.JLabel lizenzLabel;
    private javax.swing.JLabel logoLabel;
    private javax.swing.JLabel titelLabel;
    private javax.swing.JLabel versionLabel;
    // End of variables declaration//GEN-END:variables

}
