//  File:        TicketinfoApplet.java
//  Copyright:   Copyright (C) 2014 AL ENGINEERING AG. All rights reserved.
//  Website:     http://www.al-ag.de
//  Description:
//  Comments:
//  Author:      AL ENGINEERING AG
//  E-mail:      info@al-ag.de
//
//   This file is part of d(((eti-API.
//
//   d(((eti-API is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   d(((eti-API is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; If not, see <http://www.gnu.org/licenses/> or
//   write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//   Boston, MA 02111-1307 USA.
package de.alag.eticket.ticketinfo;

import de.alag.eticket.data.efm.IEfmXmlParser;
import de.alag.eticket.service.CharGeschlechtInterpreter;
import de.alag.eticket.service.LocalCodeInterpreter;
import de.alag.eticket.service.LocalOrganisationInterpreter;
import de.alag.eticket.service.WebInterpreter;
import de.alag.eticket.ticketinfo.helpers.EfmXmlParser;
import de.alag.util.Converters;
import de.alag.vdv.ka.service.ICodeInterpreter;
import de.alag.vdv.ka.service.IOrganisationInterpreter;
import de.alag.vdv.ka.service.IOrtInterpreter;
import de.alag.vdv.ka.service.IServicekennungInterpreter;
import de.alag.vdv.ka.tlv.global.EfmProduktID;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.CookieHandler;
import java.net.CookieManager;
import java.net.CookiePolicy;
import java.net.CookieStore;
import java.net.HttpCookie;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JApplet;

/**
 *
 * @author Alexander Schuetz
 */
public class TicketinfoApplet extends JApplet implements IConfigurator {

    //private String baseUrl = "http://www.al-ag.de";
    // TODO overwrite start(), stop() and destroy() methods
    @Override
    public void start() {
        new Ticketinfo(this).setVisible(true);
    }
    
    public Map<String, Set<EfmProduktID>> getWebTemplates() {
        HashMap<String, Set<EfmProduktID>> res = new HashMap<String, Set<EfmProduktID>>();
        String webTemplates = webInterpreter.requestTemplates();
        if (webTemplates != null && !webTemplates.isEmpty()) {
            String[] t = webTemplates.split(";");
            if (t != null && t.length>1) {
                Set<EfmProduktID> prods = res.get(t[0]);
                if (prods == null) {
                    prods = new HashSet<EfmProduktID>();
                    res.put(t[0], prods);
                }
                for (int i=1; i<t.length; ++i) {
                    EfmProduktID p = new EfmProduktID();
                    p.unmarshal(Converters.getBytesFromHexString(t[i]), 0);
                    prods.add(p);
                }
            }
        }
        return res;
    }
    
    private List<HttpCookie> mCookies = null;
    private boolean getCookies() {
        try {
            // Instantiate CookieManager;
            // make sure to set CookiePolicy
            CookieManager manager = new CookieManager();
            manager.setCookiePolicy(CookiePolicy.ACCEPT_ALL);
            CookieHandler.setDefault(manager);

            // get content from URLConnection;
            // cookies are set by web site
            URL url = new URL(getDocumentBase().toExternalForm());
            URLConnection connection = url.openConnection();
            connection.getContent();

            // get cookies from underlying
            // CookieStore
            CookieStore cookieJar =  manager.getCookieStore();
            mCookies = cookieJar.getCookies();
            return mCookies != null;
        } catch(Exception e) {
            System.out.println("Unable to get cookie using CookieHandler");
            e.printStackTrace();
            return false;
        }
    }

    private void setCookie(String name, String value) {
        try {
            // Instantiate CookieManager;
            // make sure to set CookiePolicy
            CookieManager manager = new CookieManager();
            manager.setCookiePolicy(CookiePolicy.ACCEPT_ALL);
            CookieHandler.setDefault(manager);

            // get content from URLConnection;
            // cookies are set by web site
            URL url = new URL(getDocumentBase().toExternalForm());
            URLConnection connection = url.openConnection();
            connection.getContent();

            // get cookies from underlying
            // CookieStore
            CookieStore cookieJar =  manager.getCookieStore();
            HttpCookie cookie = new HttpCookie(name, value);
            cookieJar.add(url.toURI(), cookie);
            mCookies = cookieJar.getCookies();
        } catch(Exception e) {
            System.out.println("Unable to get cookie using CookieHandler");
            e.printStackTrace();
        }
    }

    private String getCookie(String name) {
        if (name == null || name.isEmpty()) {
            return null;
        }
        if (mCookies != null) {
            for (HttpCookie cookie : mCookies) {
                if (name.equals(cookie.getName())) {
                    return cookie.getValue();
                }
            }
        }
        if (!getCookies()) {
            return null;
        }
        for (HttpCookie cookie : mCookies) {
            if (name.equals(cookie.getName())) {
                return cookie.getValue();
            }
        }
        return null;
    }

    public boolean isLocalConfig() {
        return false;
    }

    @Override
    public void storeConfiguration(Properties config) {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        try {
            config.storeToXML(bos, null);
            bos.flush();
            setCookie("Ticketinfo", bos.toString());
        } catch (IOException ex) {
            Logger.getLogger(TicketinfoApplet.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    @Override
    public Properties loadConfiguration() {
        String tmp = getCookie("Ticketinfo");
        if (tmp == null) {
            return Ticketinfo.getDefaultConfig();
        }
        Properties res = new Properties(Ticketinfo.getDefaultConfig());
        InputStream stream = null;
        try {
            stream = new ByteArrayInputStream(tmp.getBytes("UTF-8"));
            res.loadFromXML(stream);
        } catch (Exception ex) {
            Logger.getLogger(TicketinfoApplet.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                stream.close();
            } catch (IOException ex) {
                Logger.getLogger(TicketinfoApplet.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return res;
    }
    
    public final static boolean LOGGER = true;
    private static final int VERSION = 20;
    private static final String URL = "url";
    
    
    private static final Logger logger = Logger.getLogger(TicketinfoApplet.class.getName());
    private String connectionUrl;

    private WebInterpreter webInterpreter;
    private ICodeInterpreter cInterpreter = new LocalCodeInterpreter(new CharGeschlechtInterpreter());
    private IOrganisationInterpreter oInterpreter = new LocalOrganisationInterpreter();
    
    
    @Override
    public void init() {
        super.init(); //To change body of generated methods, choose Tools | Templates.
        configApplet();
    }
    
    
    private boolean configApplet() {
        if (LOGGER) {
            logger.log(Level.INFO, "Version: " + Integer.toString(VERSION));
        }
        boolean result = true;
        connectionUrl = getParam(URL);
        if (connectionUrl == null) {
            connectionUrl = getConnectionUrl();
        }
        if (LOGGER) {
            logger.log(Level.INFO, URL+"="+connectionUrl);
        }
        webInterpreter = new WebInterpreter(connectionUrl);
        return result;
    }
    
    private String getConnectionUrl() {
        try {
            String tmp = getDocumentBase().toExternalForm();
            int idx = tmp.lastIndexOf("/p/");
            if (idx < 0) {
                idx = tmp.lastIndexOf("/e/");
            }
            if (idx < 0) {
                idx = tmp.lastIndexOf("/resources/");
            }
            if (idx >= 0) {
                tmp = tmp.substring(0, idx);
            }
            if (tmp.endsWith("/")) {
                return tmp + "tim";
            } else {
                return tmp + "/tim";
            }
        } catch (Exception ex) {
            if (LOGGER) {
                logger.log(Level.SEVERE, "Adresse konnte nicht hergeleitet werden: {0}!",
                    ex.getMessage());
            }
            return null;
        }
    }
    
    private String getParam(final String pName) {
        try {
            String result = getParameter(pName);
            if (result == null) {
                if (LOGGER) {
                    logger.log(Level.SEVERE, "Parameter \"{0}\": nicht vorhanden!",
                        pName);
                }
                return null;
            } else {
//                logger.log(Level.INFO, "Parameter \"{0}\": \"{1}\"",
//                    new Object[] {
//                        pName,
//                        result
//                    });
                return result;
            }
        } catch (Exception ex) {
            if (LOGGER) {
                logger.log(Level.SEVERE, "Parameter \"{0}\": Fehler: {1}!",
                    new Object[] {
                        pName,
                        ex.getMessage()
                    });
            }
            return null;
        }
    }

    @Override
    public ICodeInterpreter getCodeInterpreter() {
        return cInterpreter;
    }

    @Override
    public IOrganisationInterpreter getOrganisationInterpreter() {
        return oInterpreter;
    }

    @Override
    public IOrtInterpreter getOrtInterpreter() {
        return webInterpreter;
    }

    @Override
    public IServicekennungInterpreter getServicekennungInterpreter() {
        return webInterpreter;
    }

    private IEfmXmlParser parser = new EfmXmlParser();
    @Override
    public IEfmXmlParser getParser() {
        return parser;
    }
}
