//  File:        ConfigFileLoader.java
//  Copyright:   Copyright (C) 2010 AL ENGINEERING AG. All rights reserved.
//  Website:     http://www.al-ag.de
//  Description:
//  Comments:
//  Last edit:   18.06.2013
//  Author:      AL ENGINEERING AG
//  E-mail:      info@al-ag.de
//
//   This file is part of d(((eti-API.
//
//   d(((eti-API is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   d(((eti-API is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; If not, see <http://www.gnu.org/licenses/> or
//   write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//   Boston, MA 02111-1307 USA.
package de.alag.eticket.ticketinfo.helpers;


import de.alag.eticket.data.efm.IEfmXmlParser;
import de.alag.eticket.data.efm.SaxEfmXmlParser;
import de.alag.eticket.service.CharGeschlechtInterpreter;
import de.alag.eticket.service.LocalCodeInterpreter;
import de.alag.eticket.service.LocalOrganisationInterpreter;
import de.alag.eticket.service.XmlInterpreter;
import de.alag.eticket.ticketinfo.IConfigurator;
import de.alag.util.FileLogManager;
import de.alag.vdv.ka.service.ICodeInterpreter;
import de.alag.vdv.ka.service.IOrganisationInterpreter;
import de.alag.vdv.ka.service.IOrtInterpreter;
import de.alag.vdv.ka.service.IServicekennungInterpreter;
import de.alag.vdv.ka.tlv.global.EfmProduktID;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Alexander Schuetz
 */
public class ConfigFileLoader implements IConfigurator {
    /**
     * Der globale @see java.util.logging.Logger Logger für diese Klasse.
     */
    private static final Logger LOGGER = Logger.getLogger(SaveLoadFile.class.getName());

    private static final String CFG_FILE = "." + File.separator + "Ticketinfo.cfg";
    
    private String updateUrl = "http://www.ticket-info.net/";

    private Level fileLogLevel = Level.CONFIG;

    private String logFile = System.getProperty("user.home") + File.separator + "Ticketinfo.log";

    private final Properties defaultConfig;
    
    public ConfigFileLoader(Properties defaultConfig) {
        this.defaultConfig = defaultConfig;
        FileLogManager.getInstance().registerLogger(LOGGER);
    }

    @Override
    public Properties loadConfiguration() {
        Properties config = new Properties(defaultConfig);
        try {
            config.loadFromXML(new FileInputStream(CFG_FILE));
        } catch (Exception ex) {
            LOGGER.log(Level.WARNING, "Konfigurationsdatei \""+CFG_FILE+"\" konnte nicht geladen werden:", ex);
        }
        return config;
    }

    @Override
    public Map<String, Set<EfmProduktID>> getWebTemplates() {
        return new HashMap<String, Set<EfmProduktID>>();
    }

    @Override
    public void storeConfiguration(Properties config) {
        try {
            config.storeToXML(new FileOutputStream(CFG_FILE), null);
        } catch (Exception ex) {
            LOGGER.log(Level.WARNING, "Konfigurationsdatei \""+CFG_FILE+"\" konnte nicht gespeichert werden:", ex);
        }
    }

//    public void LoadConfig() {
//        try {
//            FileReader freader = new FileReader(CFG_FILE);
//            BufferedReader br = new BufferedReader(freader);
//            String s;
//            while ((s = br.readLine()) != null) {
//                s = s.trim();
//                if (s.startsWith("#")) continue;
//                if (s.isEmpty()) continue;
//                if (s.startsWith("loglevel=")) {
//                    String level = s.substring(9).trim();
//                    try {
//                        fileLogLevel = Level.parse(level);
//                        if (fileLogLevel != Level.ALL &&
//                            fileLogLevel != Level.CONFIG &&
//                            fileLogLevel != Level.FINE &&
//                            fileLogLevel != Level.FINER &&
//                            fileLogLevel != Level.FINEST &&
//                            fileLogLevel != Level.INFO &&
//                            fileLogLevel != Level.OFF &&
//                            fileLogLevel != Level.SEVERE &&
//                            fileLogLevel != Level.WARNING) {
//                            fileLogLevel = Level.CONFIG;
//                        }
//                    } catch (Exception ex) {
//                        LOGGER.warning("Der Loglevel " + level + " wurde nicht erkannt. CONFIG wird verwendet.");
//                    }
//                } else if (s.startsWith("logdatei=")) {
//                    String file = s.substring(9);
//                    file = file.trim();
//                    if (!file.isEmpty()) {
//                        logFile = file;
//                    } else {
//                        LOGGER.warning("Der Dateiname der Logdatei ist zu kurz. " + logFile + " wird verwendet.");
//                    }
//                } else if (s.startsWith("updateurl=")) {
//                    String url = s.substring(10).trim();
//                    if (!url.isEmpty()) {
//                        updateUrl = url;
//                    } else {
//                        LOGGER.warning("Die UpdateUrl ist zu kurz. " + updateUrl + " wird verwendet.");
//                    }
//                }
//            }
//            freader.close();
//        } catch (Exception ex) {
//            LOGGER.severe("Konfigurationsdatei: " + CFG_FILE + " konnte nicht geöffnet werden!");
//            LOGGER.log(Level.FINEST, null, ex);
//        }
//    }
//
//    public void ConfigureLogging() {
//        FileLogManager.getInstance().setLogFile(logFile);
//        FileLogManager.getInstance().setFileLoggingLevel(fileLogLevel);
//    }
//
//    public URI GetUpdateUrl() {
//        URI result = null;
//        try {
//            result = URI.create(updateUrl);
//        }
//        catch (Exception ex) {
//            if (updateUrl != null && updateUrl.length() > 0) {
//                LOGGER.severe("Die Url: " + updateUrl + " ist nicht RFC2396 konform.");
//            } else {
//                LOGGER.severe("Es wurde keine URL angegeben!");
//            }
//        }
//        return result;
//    }

    @Override
    public boolean isLocalConfig() {
        return true;
    }

    private ICodeInterpreter cInterpreter = new LocalCodeInterpreter(new CharGeschlechtInterpreter());
    private IOrganisationInterpreter oInterpreter = new LocalOrganisationInterpreter();
    private XmlInterpreter xInterpreter = null;
    private IEfmXmlParser parser = new EfmXmlParser();
    
    public IEfmXmlParser getParser() {
        return parser;
    }
    
    @Override
    public ICodeInterpreter getCodeInterpreter() {
        return cInterpreter;
    }

    @Override
    public IOrganisationInterpreter getOrganisationInterpreter() {
        return oInterpreter;
    }

    @Override
    public IOrtInterpreter getOrtInterpreter() {
        if (xInterpreter == null) {
            xInterpreter = new XmlInterpreter(parser);
        }
        return xInterpreter;
    }

    @Override
    public IServicekennungInterpreter getServicekennungInterpreter() {
        return xInterpreter;
    }
}
