//  File:        ConfigWebLoader.java
//  Copyright:   Copyright (C) 2014 AL ENGINEERING AG. All rights reserved.
//  Website:     http://www.al-ag.de
//  Description:
//  Comments:
//  Author:      AL ENGINEERING AG
//  E-mail:      info@al-ag.de
//
//   This file is part of d(((eti-API.
//
//   d(((eti-API is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   d(((eti-API is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; If not, see <http://www.gnu.org/licenses/> or
//   write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//   Boston, MA 02111-1307 USA.
package de.alag.eticket.ticketinfo.helpers;

import de.alag.eticket.service.WebInterpreter;
import de.alag.eticket.data.efm.IEfmXmlParser;
import de.alag.eticket.service.CharGeschlechtInterpreter;
import de.alag.eticket.service.LocalCodeInterpreter;
import de.alag.eticket.service.LocalOrganisationInterpreter;
import de.alag.eticket.ticketinfo.IConfigurator;
import de.alag.eticket.ticketinfo.Ticketinfo;
import de.alag.util.Converters;
import de.alag.vdv.ka.service.ICodeInterpreter;
import de.alag.vdv.ka.service.IOrganisationInterpreter;
import de.alag.vdv.ka.service.IOrtInterpreter;
import de.alag.vdv.ka.service.IServicekennungInterpreter;
import de.alag.vdv.ka.tlv.global.EfmProduktID;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.net.CookieHandler;
import java.net.CookieManager;
import java.net.CookiePolicy;
import java.net.CookieStore;
import java.net.HttpCookie;
import java.net.URI;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author ASchuetz
 */
public class ConfigWebLoader implements IConfigurator {
    
    private final String COOKIE = "eTicketinfo";
    private List<HttpCookie> mCookies = null;
    private boolean connectionOk = true;
    private boolean getCookies() {
        try {
            // Instantiate CookieManager;
            // make sure to set CookiePolicy
            CookieManager manager = new CookieManager();
            manager.setCookiePolicy(CookiePolicy.ACCEPT_ALL);
            CookieHandler.setDefault(manager);

            // get content from URLConnection;
            // cookies are set by web site
            URL url = new URL(documentBase);
            URLConnection connection = url.openConnection();
            connection.getContent();

            // get cookies from underlying
            // CookieStore
            CookieStore cookieJar =  manager.getCookieStore();
            read(cookieJar);
            mCookies = cookieJar.getCookies();
            connectionOk = (mCookies != null);
            return connectionOk;
        } catch(Exception e) {
            System.out.println("Unable to get cookie using CookieHandler");
            e.printStackTrace();
            connectionOk = false;
            return false;
        }
    }

    private void setCookie(String name, String value) {
        Logger.getLogger(this.getClass().getName()).log(Level.INFO, "cookie: "+name);
        Logger.getLogger(this.getClass().getName()).log(Level.INFO, "cookie-value: "+value);
        try {
            // Instantiate CookieManager;
            // make sure to set CookiePolicy
            CookieManager manager = new CookieManager();
            manager.setCookiePolicy(CookiePolicy.ACCEPT_ALL);
            CookieHandler.setDefault(manager);

            // get content from URLConnection;
            // cookies are set by web site
            URL url = new URL(documentBase);
            URLConnection connection = url.openConnection();
            connection.getContent();

            // get cookies from underlying
            // CookieStore
            CookieStore cookieJar =  manager.getCookieStore();
            HttpCookie cookie = new HttpCookie(name, value);
            cookieJar.add(url.toURI(), cookie);
            write(cookieJar);
            mCookies = cookieJar.getCookies();
        } catch(Exception e) {
            System.out.println("Unable to get cookie using CookieHandler");
            e.printStackTrace();
        }
    }

    private void write(CookieStore store) {
        String path = Converters.getCombinePath(System.getProperty("java.io.tmpdir"), "eticketinfo");
        File dir = new File(path);
        dir.mkdirs();
        try {
            FileOutputStream fos = new FileOutputStream(Converters.getCombinePath(dir.getAbsolutePath(), "uri.txt"), false);
            for (URI u : store.getURIs()) {
                String us = u.toString();
                fos.write(us.getBytes());
                fos.write("\n".getBytes());
                fos.flush();
                FileOutputStream fos2 = new FileOutputStream(Converters.getCombinePath(dir.getAbsolutePath(), "eti_"+Integer.toString(us.hashCode())), false);
                StringBuilder b = new StringBuilder();
                for (HttpCookie c : store.get(u)) {
                    b.append(c.getName()).append("###").append(c.getValue().replaceAll("\r\n", "#n#")).append('\n');
                }
                fos2.write(b.toString().getBytes());
                fos2.close();
            }
            fos.close();
        } catch (Exception e) {
            System.out.println("Unable to persist cookies");
            e.printStackTrace();
        }
    }
    
    private void read(CookieStore store) {
        String path = Converters.getCombinePath(System.getProperty("java.io.tmpdir"), "eticketinfo");
        File dir = new File(path);
        dir.mkdirs();
        try {
            BufferedReader br = new BufferedReader(new FileReader(Converters.getCombinePath(dir.getAbsolutePath(), "uri.txt")));
            String line;
            while ((line = br.readLine()) != null) {
                URI u = new URI(line);
                FileReader fr = new FileReader(Converters.getCombinePath(dir.getAbsolutePath(), "eti_"+Integer.toString(line.hashCode())));
                BufferedReader br2 = new BufferedReader(fr);
                String line2;
                while ((line2=br2.readLine())!= null) {
                    String[] nv = line2.split("###");
                    if (nv != null && nv.length==2) {
                        store.add(u, new HttpCookie(nv[0], nv[1].replaceAll("#n#", "\r\n")));
                    }
                }
                br2.close();
            }
            br.close();
        } catch (Exception e) {
            System.out.println("Unable to restore cookies");
            e.printStackTrace();
        }
    }

    private String getCookie(String name) {
        if (name == null || name.isEmpty()) {
            return null;
        }
        if (mCookies != null) {
            for (HttpCookie cookie : mCookies) {
                if (name.equals(cookie.getName())) {
                    return cookie.getValue();
                }
            }
        }
        if (!getCookies()) {
            return null;
        }
        for (HttpCookie cookie : mCookies) {
            if (name.equals(cookie.getName())) {
                return cookie.getValue();
            }
        }
        return null;
    }

    @Override
    public void storeConfiguration(Properties config) {
        Logger.getLogger(this.getClass().getName()).log(Level.INFO, "Store config");
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        try {
            config.storeToXML(bos, null);
            bos.flush();
            setCookie(COOKIE, bos.toString());
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
        }
    }

    @Override
    public Properties loadConfiguration() {
        Logger.getLogger(getClass().getName()).log(Level.INFO, "Loading config");
        String tmp = getCookie(COOKIE);
        Properties res = new Properties(Ticketinfo.getDefaultConfig());
        if (tmp != null) {
            InputStream stream = null;
            try {
                stream = new ByteArrayInputStream(tmp.getBytes("UTF-8"));
                res.loadFromXML(stream);
            } catch (Exception ex) {
                Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
            } finally {
                try {
                    stream.close();
                } catch (IOException ex) {
                    Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
                }
            }
        } else {
            Logger.getLogger(getClass().getName()).log(Level.WARNING, "no cookie found");
        }
        return res;
    }
    
    @Override
    public Map<String, Set<EfmProduktID>> getWebTemplates() {
        HashMap<String, Set<EfmProduktID>> res = new HashMap<String, Set<EfmProduktID>>();
        String webTemplates = webInterpreter.requestTemplates();
        if (webTemplates != null && !webTemplates.trim().isEmpty()) {
            String[] lines = webTemplates.split("\n");
            if (lines == null) {
                return res;
            }
            for (String line : lines) {
                if (line == null || line.trim().isEmpty()) {
                    continue;
                }
                String[] prods = line.split(";");
                if (prods != null && prods.length > 1) {
                    Set<EfmProduktID> prodset = res.get(prods[0]);
                    if (prodset == null) {
                        prodset = new HashSet<EfmProduktID>();
                        res.put(prods[0], prodset);
                    }
                    for (int i=1; i<prods.length; ++i) {
                        if (prods[i] == null || prods[i].trim().isEmpty()) {
                            continue;
                        }
                        byte[] raw = Converters.getBytesFromHexString(prods[i]);
                        if (raw == null) {
                            continue;
                        }
                        EfmProduktID pid = new EfmProduktID();
                        if (pid.unmarshal(raw, 0) == pid.getLength()) {
                            prodset.add(pid);
                        }
                    } 
                }
            }
        }
        return res;
    }

    @Override
    public boolean isLocalConfig() {
        return !connectionOk;
    }
    public final static boolean LOGGER = true;
    private static final int VERSION = 22;
    private static final String URL = "url";
    
    
    private static final Logger logger = Logger.getLogger(ConfigWebLoader.class.getName());
    private String connectionUrl;

    private WebInterpreter webInterpreter;
    private ICodeInterpreter cInterpreter = new LocalCodeInterpreter(new CharGeschlechtInterpreter());
    private IOrganisationInterpreter oInterpreter = new LocalOrganisationInterpreter();
    
    private String documentBase;
    public ConfigWebLoader(String codebase, String href) {
        this.documentBase = codebase;
        if (href != null) {
            if (this.documentBase.endsWith("/")) {
                this.documentBase += href;
            } else {
                this.documentBase += "/"+href;
            }
        }
        configApplet();
    }
    
    private boolean configApplet() {
        if (LOGGER) {
            logger.log(Level.INFO, "Version: " + Integer.toString(VERSION));
        }
        boolean result = true;
        connectionUrl = getConnectionUrl();
        if (LOGGER) {
            logger.log(Level.INFO, URL+"="+connectionUrl);
        }
        webInterpreter = new WebInterpreter(connectionUrl);
        return result;
    }
    
    private String getConnectionUrl() {
        try {
            String tmp = documentBase;
            int idx = tmp.lastIndexOf("/p/");
            if (idx < 0) {
                idx = tmp.lastIndexOf("/e/");
            }
            if (idx < 0) {
                idx = tmp.lastIndexOf("/resources/");
            }
            if (idx >= 0) {
                tmp = tmp.substring(0, idx);
            }
            if (tmp.endsWith("/")) {
                return tmp + "eticketinfo";
            } else {
                return tmp + "/eticketinfo";
            }
        } catch (Exception ex) {
            if (LOGGER) {
                logger.log(Level.SEVERE, "Adresse konnte nicht hergeleitet werden: {0}!",
                    ex.getMessage());
            }
            return null;
        }
    }
    
    @Override
    public ICodeInterpreter getCodeInterpreter() {
        return cInterpreter;
    }

    @Override
    public IOrganisationInterpreter getOrganisationInterpreter() {
        return oInterpreter;
    }

    @Override
    public IOrtInterpreter getOrtInterpreter() {
        return webInterpreter;
    }

    @Override
    public IServicekennungInterpreter getServicekennungInterpreter() {
        return webInterpreter;
    }

    private IEfmXmlParser parser = new EfmXmlParser();

    @Override
    public IEfmXmlParser getParser() {
        return parser;
    }
}
