//  File:        EfmXmlParser.java
//  Copyright:   Copyright (C) 2010 AL ENGINEERING AG. All rights reserved.
//  Website:     http://www.al-ag.de
//  Description:
//  Comments:
//  Last edit:   18.06.2013
//  Author:      AL ENGINEERING AG
//  E-mail:      info@al-ag.de
//
//   This file is part of d(((eti-API.
//
//   d(((eti-API is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   d(((eti-API is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; If not, see <http://www.gnu.org/licenses/> or
//   write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//   Boston, MA 02111-1307 USA.
package de.alag.eticket.ticketinfo.helpers;

import de.alag.eticket.data.efm.EfmPvDatei;
import de.alag.eticket.data.efm.IEfmXmlParser;
import de.alag.eticket.data.efm.SaxHandlerEfmHeader;
import java.util.List;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.XMLConstants;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import de.alag.vdv.ka.service.IOrt;
import de.alag.vdv.ka.service.Ort;
import de.alag.vdv.ka.tlv.Template;
import de.alag.vdv.ka.tlv.TemplateProvider;
import de.alag.vdv.ka.tlv.global.EfmProduktID;
import java.util.Date;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;

/**
 * SaxEfmXmlParser ist ein kleiner SAX-Parser, der die IEfmXmlParser-Schnitt-
 * stelle implementiert.
 *
 * @author Alexander Schuetz
 */
public class EfmXmlParser extends de.alag.eticket.xml.SaxHandler implements IEfmXmlParser {

    private String efmRegex;
    private Schema schema;
    private int tiefe;
    private static final int MAX_TIEFE = 1;
    //private static final String DEFAULT_REGEX = ".*EFM.*\\d\\d\\d\\d_\\d\\d_\\d\\d[.][xX][mM][lL]";
    //private static final String DEFAULT_REGEX = "[a-zA-Z]{3}-[eE][fF][mM]-[dD]aten-[0-9]{4}-[0-9]{2}-[0-9]{2}.[xX][mM][lL]";
    private static final String DEFAULT_REGEX = "[pP][vV][0-9]+.*[xX][mM][lL]";

    private static Map<Integer,Map<Integer,String>> skMap = new HashMap<Integer,Map<Integer, String>>();
    private static Map<Long, Set<Long>> ortMap = new HashMap<Long, Set<Long>>();
    private static Map<String, Long> relationenList = new HashMap<String, Long>();
    private static Map<Long, String> ortNamen = new HashMap<Long, String>();
    private static Map<String, Set<EfmProduktID>> templates = new HashMap<String, Set<EfmProduktID>>();
    
    protected long convert(IOrt ort) {
        return convert(ort.getOrtNummer(), ort.getOrtTyp(), ort.getOrganisationsNummer());
    }
    
    protected long convert(int nr, int typ, int org) {
        long res = (nr & 0xFFFF);
        res <<= 8;
        res |= (typ & 0xFF);
        res <<= 16;
        res |= (org & 0xFFFF);
        return res;
    }
    
    public EfmXmlParser() {
        this.efmRegex = DEFAULT_REGEX;
        this.schema = null;
        tiefe = 0;
    }

    
    /**
     * Konstruktor.
     *
     * @param regex Der reguläre Ausdruck der verwendet werden soll um EFM-Dateien zu identifizieren.
     * @param schema Das Schema das verwendet werden soll um EFM-Dateien zu verifizieren.
     */
    public EfmXmlParser(String regex, String schema) {
        this.efmRegex = (regex == null) ? DEFAULT_REGEX : regex;
        this.schema = (schema == null) ? null : loadSchema(schema);
        tiefe = 0;
    }

    /**
     * Lädt ein xsd-Schema aus der Datei deren Pfad angegeben ist.
     * @param fileName Der (absolute oder relative) Pfad der XSD-Datei.
     * @return Das gelesene Schema
     */
    private Schema loadSchema(String fileName) {
        Schema s = null;
        File file = new File(fileName);
        int extIdx = file.getName().lastIndexOf('.') + 1;
        String fileExt = file.getName().substring(extIdx);
        if ( (!file.exists()) ||
             (!file.canRead()) ||
             (!file.isFile()) ||
             (!fileExt.equalsIgnoreCase("xsd"))) {
            return s;
        }
        try {
            String language = XMLConstants.W3C_XML_SCHEMA_NS_URI;
            SchemaFactory sf = SchemaFactory.newInstance(language);
            s = sf.newSchema(file);
        } catch (Exception ex) {
            Logger.getLogger(getClass().getName()).log(Level.SEVERE, null, ex);
        }
        return s;
    }

    @Override
    public int FindEfmXmlFiles(String pfad) {
        File verzeichnis = new File(pfad);
        if ( (!verzeichnis.isDirectory()) ||
             (!verzeichnis.canRead()) ) {
            return -1;
        }
        tiefe = 0;
        int foundFiles = 0;
        File efmVerz = DurchsucheVerzeichnis(verzeichnis);
        if (efmVerz != null) {
            // Dateien dieses Verzeichnisses verwenden
            File[] dateien = efmVerz.listFiles();
            SAXParser parser = null;
            try {
                SAXParserFactory factory = SAXParserFactory.newInstance();
                if (schema != null) {
                    factory.setSchema(schema);
                }
                parser = factory.newSAXParser();
            } catch (Exception ex) {
                Logger.getLogger(EfmXmlParser.class.getName()).log(Level.SEVERE, null, ex);
                return -2;
            }
            SaxHandlerEfmHeader handler = new SaxHandlerEfmHeader();

            for (int i=0; i<dateien.length; ++i) {
                if (dateien[i].isFile() && dateien[i].canRead() && dateien[i].getName().matches(efmRegex)) {
                    // ist es eine xml-datei und wie ist die nummer des PV
                    int pvNr = -1;
                    handler.Reset();
                    try {
                        parser.parse(dateien[i], handler);
                    } catch (Exception ex) {
                        if (!ex.getMessage().equalsIgnoreCase("Finished")) {
                            Logger.getLogger(EfmXmlParser.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    }
                    if (!handler.FoundPVID()) {
                        continue;
                    }
                    ++foundFiles;
                    try {
                        pvNr = Integer.decode(handler.GetPvid());
                        File f = dateien[i];
                        Calendar cal = Calendar.getInstance();
                        cal.setTime(new Date(f.lastModified()));
                        EfmPvDatei exd_pv = new EfmPvDatei(pvNr, dateien[i].getCanonicalPath(), cal.getTime());
                        if (!EfmPvDatei.UpdateBekanntePVs(exd_pv)) {
//                            ++foundFiles;
                        }
                    } catch (IOException ex) {
                        Logger.getLogger(EfmXmlParser.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
            }
        }
        parseFiles();
        return foundFiles;
    }

    /**
     * Durchsucht ein Verzeichnis rekursiv (inklusive Unterverzeichnisse) nach dem Verzeichnis "efm".
     * @param verz Das zu durchsuchende Verzeichnis
     * @return Das gefundene efm-Verzeichnis oder null.
     */
    private File DurchsucheVerzeichnis(File verz) {
        if (tiefe > MAX_TIEFE) {
            return null;
        }
        ++tiefe;
        if ( (!verz.isDirectory()) ||
             (!verz.canRead()) ) {
            --tiefe;
            return null;
        }
        if (verz.getName().equalsIgnoreCase("efm") || verz.getName().equalsIgnoreCase("efm-daten")) {
            --tiefe;
            return verz;
        }
        File[] files = verz.listFiles();
        File result = null;
        for (int i=0; i<files.length; ++i) {
            result = DurchsucheVerzeichnis(files[i]);
            if (result != null) {
                break;
            }
        }
        --tiefe;
        return result;
    }

    
    @Override
    public String decodeServiceKennung(int orgid, int kennung) {
        if (!skMap.containsKey(orgid)) {
            skMap.put(orgid, new HashMap<Integer, String>());
        }
        Map<Integer,String> tmp = skMap.get(orgid);
        if (tmp.containsKey(kennung)) {
            return tmp.get(kennung);
        } else {
            return "Nr: " + Integer.toString(kennung);
        }
    }
    
    @Override
    public List<IOrt> getSubAreas(IOrt ort) {
        ArrayList<IOrt> result = new ArrayList<IOrt>();
        if (ortMap.containsKey(ort)) {
            for (Long l : ortMap.get(ort)) {
                result.add(convert(l));
            }
        }
        return result;
    }
    
    @Override
    public String decodeOrt(IOrt ort) {
        long id = convert(ort);
        return ortNamen.get(new Long(id));
    }

    @Override
    public IOrt validateRelation(int orgid, String relation) {
        Long result = normalizeRelation(orgid, relation);
        if (result != null) {
            if (ortMap.containsKey(result)) {
                return convert(result);
            }
        }
        String tmp = normalizeBezeichnung(relation);
        result = relationenList.get(tmp);
        if (result != null) {
            if (ortMap.containsKey(result)) {
                return convert(result);
            }
        }
        result = relationenList.get(relation);
        if (result != null) {
            if (ortMap.containsKey(result)) {
                return convert(result);
            }
        }
        return null;
    }

    private Long normalizeRelation(int orgid, String relation) {
        if (relation != null) {
            return null;
        }
        relation = relation.trim();
        if (relation .isEmpty()) {
            return null;
        }
        if(relation.startsWith("R")) {
            // Hinten abschneiden
            String nrStr = relation;
            if (nrStr.contains("_")) {
                nrStr = nrStr.substring(0, nrStr.indexOf("_"));
            }
            // R entfernen
            nrStr.replace("R", "");
            if (nrStr.contains("_")) {
                nrStr = nrStr.substring(0, nrStr.indexOf("_"));
            }
            try {
                int n = Integer.parseInt(nrStr);
                return convert(n, 202, orgid);
            } catch (Exception ex) {
                return null;
            }
        } else {
            return null;
        }
    }
    
    private String normalizeBezeichnung(String bez) {
        if (bez == null) {
            return null;
        }
        bez = bez.trim();
        if (bez.isEmpty()) {
            return null;
        }
        bez = bez.replace(' ', '_');
        String[] bezTeile = bez.split("_");
        if (bezTeile == null) {
            return bez;
        }
        bez = "";
        Set<Integer> iParts = new HashSet<Integer>();
        for (String teil : bezTeile) {
            String tmp = teil.trim();
            if (!tmp.isEmpty()) {
                try {
                    int iTmp = Integer.parseInt(tmp);
                    if (iParts.add(new Integer(iTmp))) {
                        if (!bez.isEmpty()) {
                            bez += "_";
                        }
                        bez += Integer.toString(iTmp);
                    }
                } catch (Exception ex) {
                    if (!bez.isEmpty()) {
                        bez += "_";
                    }
                    bez += tmp;
                }
            }
        }
        return bez;
    }
    
    protected IOrt convert(long id) {
        IOrt res = new Ort();
        int tmp = (int)(id & 0xFFFFFFL);
        res.setOrganisationsNummer(tmp & 0xFFFF);
        res.setOrtTyp((tmp >> 16) & 0xFF);
        tmp = (int)((id >> 24) & 0xFFFF);
        res.setOrtNummer(tmp);
        res.setBezeichnung(ortNamen.get(new Long(id)));
        return res;
    }
    
    void parseFiles() {
        SAXParser parser = null;
        if (EfmPvDatei.HasNoFiles()) {
            return;
        }
        for (String name : EfmPvDatei.GetFileNames()) {
            File efm_datei = new File(name);
            try {
                SAXParserFactory factory = SAXParserFactory.newInstance();
                if (schema != null) {
                    factory.setSchema(schema);
                }
                parser = factory.newSAXParser();
            } catch (Exception ex) {
                Logger.getLogger(getClass().getName()).log(Level.SEVERE, null, ex);
                continue;
            }
        
            if (efm_datei.isFile() && efm_datei.canRead() && efm_datei.getName().matches(efmRegex)) {
                this.Reset();
                try {
                    parser.parse(efm_datei, this);
                } catch (Exception ex) {
                    Logger.getLogger(getClass().getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    private Long currentOrt = null;
    
    @Override
    protected boolean addOrt(int orgID, int type, int nr, String name) {
        ortNamen.put(convert(nr, type, orgID), name);
        return true;
    }

    @Override
    protected boolean addUnterOrt(int orgID, int type, int nr) {
        if (currentOrt == null) {
            return false;
        }
        Set<Long> uo = ortMap.get(currentOrt);
        if (uo == null) {
            uo = new HashSet<Long>();
            ortMap.put(currentOrt, uo);
        }
        uo.add(convert(nr, type, orgID));
        return true;
    }

    @Override
    protected boolean addNachbarOrt(int orgID, int type, int nr) {
        return true;
    }

    @Override
    protected boolean setOrt(int orgID, int type, int nr) {
        currentOrt = convert(nr, type, orgID);
        return true;
    }

    @Override
    protected boolean addOrganisation(int id, String name, String kuerzel) {
        return true;
    }

    @Override
    protected boolean addPreisstufe(int pvOrgid, String name, int preis) {
        return true;
    }

    @Override
    protected boolean addRaum2Preisstufe(int pvOrgid, String preisstufe, Date validUntil, int raumOrgid, int raumTyp, int raumNr) {
        long rid = convert(raumNr, raumTyp, raumOrgid);
        String rName = ortNamen.get(rid);
        if (rName == null) {
            return false;
        } else {
            relationenList.put(rName, rid);
            return true;
        }
    }

    @Override
    protected boolean addKerngebiet2Preisstufe(int pvOrgid, String preisstufe, int raumOrgID, int raumTyp, int raumNr, int orgID, int typ, int nr) {
        return true;
    }

    @Override
    protected boolean interpretProdParam(String name, String value, String price, String type) {
        return true;
    }

    @Override
    protected boolean addProdukt(int orgid, int nr, String name, int abrechnungsverfahren, String serviceKennung, String template) {
        Map<Integer, String> prods = skMap.get(orgid);
        if (prods == null) {
            prods = new HashMap<>();
            skMap.put(orgid, prods);
        }
        prods.put(nr, name);
        if (template != null && !template.isEmpty()) {
            TemplateProvider.getInstance().addProduct(template, new EfmProduktID(nr, orgid));
        }
        return true;
    }

    @Override
    protected boolean setProdGueltigkeit(Date from, Date to) {
        return true;
    }

    @Override
    protected boolean addErfNmKey(int orgid, int kid, int version) {
        return true;
    }

    @Override
    protected boolean addProdDescription(String desc) {
        return true;
    }

    @Override
    protected boolean addProdPreisstufe(int pvid, String preisstufe) {
        return true;
    }
}
