//  File:        SimpleFileFilter.java
//  Copyright:   Copyright (C) 2010 AL ENGINEERING AG. All rights reserved.
//  Website:     http://www.al-ag.de
//  Description:
//  Comments:
//  Last edit:   18.06.2013
//  Author:      AL ENGINEERING AG
//  E-mail:      info@al-ag.de
//
//   This file is part of d(((eti-API.
//
//   d(((eti-API is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   d(((eti-API is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; If not, see <http://www.gnu.org/licenses/> or
//   write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//   Boston, MA 02111-1307 USA.
package de.alag.eticket.ticketinfo.helpers;


/**
 * FileFilter zur auswahl der zu ladenden Dateien.
 * @author Frank Hamm
 */
public class SimpleFileFilter extends javax.swing.filechooser.FileFilter
{
    /**
     * Description der Dateien.
     */
    private String myDescription;

    /**
     * File Extensions.
     */
    private String myExtension;

    /**
     * Konstruktor.
     * @param desc Description.
     * @param ext Extesnion.
     */
    public SimpleFileFilter(String desc, String ext) {
        myDescription = desc;
        myExtension = ext;
    }

    @Override
    public boolean accept(java.io.File f) {
        if(f.isDirectory()) {
            return true;
        } // ist ein Folder - wird angezeigt

        if(getExt(f).equals(myExtension)) {
            return true;
        } // richtige Extension - wird angezeigt

        return false; // alles andere nicht anzeigen
    }

    @Override
    public String getDescription() {
        return myDescription;
    }

    @Override
    public final String toString() {
        return myExtension;
    }

    /**
     * Gibt die Extension der Datei an.
     * @param f Datei dessen Extension gelsen werden soll.
     * @return Extesnion der Datei.
     */
    private String getExt(java.io.File f) {
        return f.getName().substring(f.getName().toLowerCase().lastIndexOf(".") + 1);
    }
}
