package de.alag.eticket.ticketinfo.professional;

/*
 *   Copyright (C) <2010>  <AL-ENGINEERING AG>
 *   written by Hans-Joachim Haas
 *   Email <info@al-ag.de>
 *
 *   d(((eti-API is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   d(((eti-API is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this library; if not, write to the Free Software Foundation,
 *   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

import de.alag.util.Pair;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import de.alag.vdv.ka.tlv.log.SeparateDatenApplikationslogbuch;
import de.alag.vdv.ka.tlv.directory.Applikationsverzeichnis;
import de.alag.vdv.ka.tlv.entitlement.SeparateDatenBerechtigung;
import de.alag.vdv.ka.tlv.customer.SeparateDatenKundendaten;
import de.alag.vdv.ka.tlv.wes.SeparateDatenWes;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;
import de.alag.vdv.ka.tlv.ITlvElement;
import de.alag.vdv.ka.tlv.entitlement.Berechtigung;
import de.alag.vdv.ka.tlv.templates.nrw.DatenPSTeilNrw;
import de.alag.vdv.ka.xml.NmApplikationslogbuchType;
import de.alag.vdv.ka.xml.NmApplikationsverzeichnisType;
import de.alag.vdv.ka.xml.NmBerechtigungsdatenType;
import de.alag.vdv.ka.xml.NmKundendatenType;
import de.alag.vdv.ka.xml.WESDatenType;
/**
 * Repräsentiert ein Nutzermedium
 * @author Hans-Joachim Haas
 */
@XmlAccessorType(XmlAccessType.FIELD)
@XmlRootElement
public class Nutzermedium {
    
    @XmlTransient
    private Applikationsverzeichnis app;
    @XmlTransient
    private StringBuilder errorTextVDVKA;       // Parser + BOM
    @XmlTransient
    private StringBuilder errorTextTarif;       // Plausibilität
    @XmlTransient
    private StringBuilder errorTextVergleich;   // Vergleich der Berechtigungen
    @XmlTransient
    protected String errorString;
    @XmlTransient
    protected String FehlerVDVKA;
    @XmlTransient
    protected String FehlerTarif;
    @XmlTransient
    protected String FehlerVergleich;
    
    private NmApplikationsverzeichnisType applikationsVerzeichnis;
    private NmBerechtigungsdatenType[] separateDatenBerechtigung;
    private NmApplikationslogbuchType separateDatenApplikationslogbuch;
    private NmKundendatenType separateDatenKundendaten;
    private WESDatenType separateDatenWes;
    private String freiText;
    protected String LogZeitstempel;
    
    @XmlElement
    protected DifferenzItemList FehlerListeVergleich = null;
    
    @XmlElement
    protected TarifItemList FehlerListeTarif = null;
    
    @XmlElement
    protected BomItemList FehlerListeVDVKA = null;
    
    /**
     * Initialisiert die einzelnen Teile eines Nutzermediums
     * @param a 
     */
    public void Init(Applikationsverzeichnis a) {
        
        app = a;

        errorTextVDVKA = new StringBuilder();
        errorTextTarif = new StringBuilder();
        errorTextVergleich = new StringBuilder();

        if(app != null && !app.getFlagObj().isEmpty())
            errorTextVDVKA.append(app.getErrorText());
        
        if(app == null)
            errorTextVDVKA.append("Fehler beim Lesen des Applikationsverzeichnis\r\n");
        
        for(Berechtigung b: app.getBerechtigungen()) {
            if (b.getSepDataTeil() != null && !app.getFlagObj().isEmpty())
                errorTextVDVKA.append(b.getSepDataTeil().getErrorText());
            if(b.getSepDataTeil() == null)
                errorTextVDVKA.append("Fehler beim Lesen der Berechtigung\r\n");
        }
        
        if(app.getLogbuch() != null && !app.getFlagObj().isEmpty())
            errorTextVDVKA.append(app.getLogbuch().getErrorText());
        if(app.getLogbuch() == null)
            errorTextVDVKA.append("Fehler beim Lesen des Logbuchs\r\n");
        
        if(app.getKundendaten() != null && !app.getFlagObj().isEmpty())
            errorTextVDVKA.append(app.getKundendaten().getErrorText());
        
        if(app.getWes() != null && !app.getFlagObj().isEmpty())
            errorTextVDVKA.append(app.getWes().getErrorText());
        if(app.getWes() == null)
            errorTextVDVKA.append("Fehler beim Lesen der Wesdaten\r\n");
        
        errorString = errorTextVDVKA.toString();
        FehlerVDVKA = errorTextVDVKA.toString();
        
        try {
            if(app != null)
                applikationsVerzeichnis = app.getXml(true);
            
            separateDatenBerechtigung = new NmBerechtigungsdatenType[app.getBerechtigungen().size()];
            for(int i = 0; i < app.getBerechtigungen().size(); i++) {
                if( app.getBerechtigung(i) != null)
                    separateDatenBerechtigung[i] = app.getBerechtigung(i).getSepDataTeil().getXml(true); 
            }
            
            if(app.getLogbuch() != null)
                separateDatenApplikationslogbuch = app.getLogbuch().getSepDataTeil().getXml(true);
            
            if(app.getWes() != null)
                separateDatenWes = app.getWes().getSepDataTeil().getXml(true);
        }
        catch(Exception ex){}
        
        Date dt = new Date();
        // Festlegung des Formats:
        SimpleDateFormat df = new SimpleDateFormat( "dd.MM.yyyy HH:mm:ss" );
        // Formatierung zu String:
        LogZeitstempel = df.format( dt );
        
        //FehlerlisteVergleich = new LinkedList<List<String>>();
        //DifferenzItem = new LinkedList<String>();
    }
    
    /**
     * Validiert das Nutzermedium nach SpecBom
     */
    public void validateBom() {
        FehlerListeVDVKA = new BomItemList();
        if(app != null) {
            List<Pair<ITlvElement, String>> appBomErrorList = app.validateBom();
            if (appBomErrorList != null) {
                for(Pair<ITlvElement, String> pair: appBomErrorList) {
                    appendErrortextVDVKA(pair.getValueB().trim());
                    String error = pair.getValueB().toString();
                    if(!error.endsWith(" / ")) {  
                        FehlerListeVDVKA.Add(error.trim());
                    }
                }
            }
        }
        for(Berechtigung b: app.getBerechtigungen()) {
            if(b != null) {
                List<Pair<ITlvElement, String>> sepDatBomErrorList = b.getSepDataTeil().validateBom();
//                if(b.getSepDataTeil().getProdSpezTeil().get(DatenPSTeilNrw.class) != null)
//                    sepDatBomErrorList = b.getSepDataTeil().validateBom();
                if (sepDatBomErrorList != null) {
                    for(Pair<ITlvElement, String> pair: sepDatBomErrorList) {
                        appendErrortextVDVKA(pair.getValueB().trim());
                        String error = pair.getValueB().toString();
                        if(!error.endsWith(" / ")) {  
                            FehlerListeVDVKA.Add(error.trim());
                        }
                    }
                }
            }
        }
        if(app.getLogbuch().getSepDataTeil() != null) {
            List<Pair<ITlvElement, String>> sepLogBomErrorList = app.getLogbuch().getSepDataTeil().validateBom();
            if (sepLogBomErrorList != null) {
                for(Pair<ITlvElement, String> pair: sepLogBomErrorList) {
                    appendErrortextVDVKA(pair.getValueB().trim());
                    String error = pair.getValueB().toString();
                    if(!error.endsWith(" / ")) {  
                        FehlerListeVDVKA.Add(error.trim());
                    }
                }
            }
        }
        if(app.getWes().getSepDataTeil() != null) {
            List<Pair<ITlvElement, String>> sepWesBomErrorList = app.getWes().getSepDataTeil().validateBom();
            if (sepWesBomErrorList != null) {
                for(Pair<ITlvElement, String> pair: sepWesBomErrorList) {
                    appendErrortextVDVKA(pair.getValueB().trim());
                    String error = pair.getValueB().toString();
                    if(!error.endsWith(" / ")) {  
                        FehlerListeVDVKA.Add(error.trim());
                    }
                }
            }
        }
        if(FehlerListeVDVKA.Size() == 0)
            FehlerListeVDVKA = null;
    }
 
    /**
     * Gibt das Applikationsverzeichnis zurück
     * @return Applikationsverzeichnis
     */
    public Applikationsverzeichnis getApp() {
        return app;
    }
    
    /**
     * Gibt das SeparateDatenApplikationslogbuch zurück
     * @return SeparateDatenApplikationslogbuch
     */
    public SeparateDatenApplikationslogbuch getSepLog() {
        return app.getLogbuch().getSepDataTeil();
    }
    
    /**
     * Gibt die SeparateDatenWes zurück
     * @return SeparateDatenWes
     */
    public SeparateDatenWes getSepWes() {
        return app.getWes().getSepDataTeil();
    }
    
    /**
     * Gibt die SeparateDatenKundendaten zurück
     * @return SeparateDatenKundendaten
     */
    public SeparateDatenKundendaten getSepKunde() {
        return app.getKundendaten().getSepDataTeil();
    }
    
    /**
     * Gibt die SeparateDatenBerechtigungen zurück
     * @return Liste mit SeparateDatenBerechtigung
     */
    public SeparateDatenBerechtigung[] getSepDat() {
        SeparateDatenBerechtigung[] result = new SeparateDatenBerechtigung[app.getBerechtigungen().size()];
        int i = 0;
        for (Berechtigung b : app.getBerechtigungen()) {
            result[i++] = b.getSepDataTeil();
        }
        return result;
    }
    
    /**
     * setzt den Freitext
     * @param txt 
     */
    public void setFreitext(String txt) {
        freiText = txt;
    }
    
    /**
     * Gibt den Freitext zurück
     * @return 
     */
    public String getFreitext(){
        return freiText;
    }
    
    /**
     * Hängt einen Fehlertext an
     * @param txt 
     */
    public void appendErrortextVDVKA(String txt) {
        errorTextVDVKA.append(txt); 
        errorString = errorTextVDVKA.toString() + "\r\n"
                + errorTextTarif.toString() + "\r\n"
                + errorTextVergleich.toString();
        
        FehlerVDVKA = errorTextVDVKA.toString();
        FehlerTarif = errorTextTarif.toString();
        FehlerVergleich = errorTextVergleich.toString();
        
        if(FehlerVDVKA.isEmpty()) {
            FehlerVDVKA = null;
        }
    }
    
    /**
     * Hängt einen Fehlertext an
     * @param txt 
     */
    public void appendErrortextVergleich(String txt) {
        errorTextVergleich.append(txt); 
        errorString = errorTextVDVKA.toString() + "\r\n"
                + errorTextTarif.toString() + "\r\n"
                + errorTextVergleich.toString();
        
        FehlerVDVKA = errorTextVDVKA.toString();
        FehlerTarif = errorTextTarif.toString();
        FehlerVergleich = errorTextVergleich.toString();
        if(FehlerVDVKA.isEmpty()) {
            FehlerVDVKA = null;
        }
    }
    
    /**
     * Hängt einen Fehlertext an
     * @param txt 
     */
    public void appendErrortextTarif(String txt) {
        errorTextTarif.append(txt); 
        errorString = errorTextVDVKA.toString() + "\r\n"
                + errorTextTarif.toString() + "\r\n"
                + errorTextVergleich.toString();
        
        FehlerVDVKA = errorTextVDVKA.toString();
        FehlerTarif = errorTextTarif.toString();
        FehlerVergleich = errorTextVergleich.toString();
        if(FehlerVDVKA == "")
            FehlerVDVKA = null;
    }
    
    /**
     * Gibt den Fehlertext zurück
     */
    public String getErrortext(){
        return errorString;
    }
    
    public String getFehlerVDVKA(){
        if(errorTextVDVKA != null)
            return errorTextVDVKA.toString();
        else
            return "";
    }
    
    public String getFehlerTarif(){
        if(FehlerListeTarif != null)
            return FehlerTarif.toString();
        else
            return "";
    }
    
    public String getFehlerVergleich(){
        if(FehlerVergleich != null)
            return FehlerVergleich.toString();
        else
            return "";
    }
    
    public void validateEFM() {
        FehlerListeTarif = new TarifItemList();
        for(Berechtigung b: app.getBerechtigungen()) {
            if( b != null) {
                String result = "";
                DatenPSTeilNrw nrw = b.getSepDataTeil().getProdSpezTeil().get(DatenPSTeilNrw.class);
                if (nrw != null)
                    result = b.getSepDataTeil().validateEFM();
                if(!result.isEmpty() && result != null) {
                    appendErrortextTarif(result.trim());
                    String[] tl = result.split("\r\n");
                    for (String s: tl)
                        if(!s.isEmpty())
                            FehlerListeTarif.Add(s.trim());
                }
            }
        }
        if(FehlerListeTarif.Size() == 0)
            FehlerListeTarif = null;
    }
    
    /**
     * Vergleicht zwei Nutzermedien miteinander
     * @param Nutzermedium
     * @return Text mit den Unterschieden
     */
    public String diff(Object obj) {
        List<Pair<ITlvElement, String>> appDiffErrorList = null;
        String diffError = "";
        
        if(obj == null)
            return "Kein Objekt 'Nutzermedium'";
        if(obj instanceof Nutzermedium) {
            int l1 = app.getBerechtigungen().size();
            int l2 = ((Nutzermedium) obj).getSepDat().length;
            int len = java.lang.Math.min(l1, l2);
            for(int i = 0; i < len; i++) {   
                //appDiffErrorList = sepDat[i].diff(((Nutzermedium) obj).getSepDat()[i]);
                appDiffErrorList = ((Nutzermedium) obj).getSepDat()[i].diff(app.getBerechtigung(i).getSepDataTeil());
                if(appDiffErrorList.size() > 0) {
                    FehlerListeVergleich = new DifferenzItemList();
                    for(Pair<ITlvElement, String> p: appDiffErrorList) {
                        String error = p.getValueB().toString();
                        diffError = diffError + p.getValueB().toString();
                        if(!error.endsWith(" / ")) {  
                            FehlerListeVergleich.Add(p.getValueB().toString().trim());
                        }
                    }
                }
            }
        }
        
        appendErrortextVergleich(diffError);
        
        return diffError;
        
    }
    
    /**
     * Vergleicht die Applikation
     * Wird nicht verwendet
     * @param Nutzermedium
     * @return 
     */
    public String diffApplikation(Object obj) {
        String diffError = "";
        
        if(obj == null)
            return "Kein Objekt 'Nutzermedium'";
        
        if(obj instanceof Nutzermedium) {
            Nutzermedium nm = (Nutzermedium) obj;
            
            // Version testen
            byte[] v1 = app.getVerzAppDat().getVerzStatTeil().getVersionNr().toRaw();
            byte[] v2 = nm.getApp().getVerzAppDat().getVerzStatTeil().getVersionNr().toRaw();
            if(v1[0] != v2[0])
                FehlerListeVergleich.Add("Applikation Version " + v1[0] + " ungleich " + v2[0]);
            
            // GueltigBeginn
            List<Pair<ITlvElement, String>> result = app.getVerzAppDat().getVerzStatTeil().getGueltBeginn().diff(nm.getApp().getVerzAppDat().getVerzStatTeil().getGueltBeginn());//, "Applikation GültigBeginn");
            if(result != null) {
                for(Pair<ITlvElement, String> p: result) {
                        String error = p.getValueB().toString();
                        diffError = diffError + p.getValueB().toString();
                        FehlerListeVergleich.Add(error.trim());
                }
            }
            
            // GueltigEnde
            result = app.getVerzAppDat().getVerzStatTeil().getGueltEnde().diff(nm.getApp().getVerzAppDat().getVerzStatTeil().getGueltEnde());//, "Applikation GültigEnde");
            if(result != null) {
                for(Pair<ITlvElement, String> p: result) {
                        String error = p.getValueB().toString();
                        diffError = diffError + p.getValueB().toString();
                        FehlerListeVergleich.Add(error.trim());
                }
            }
            
            // Status
            result = app.getVerzAppDat().getVerzDynTeil().getStatusCode().diff(nm.getApp().getVerzAppDat().getVerzDynTeil().getStatusCode());//, "Applikation Status");
            if(result != null) {
                for(Pair<ITlvElement, String> p: result) {
                        String error = p.getValueB().toString();
                        diffError = diffError + p.getValueB().toString();
                        FehlerListeVergleich.Add(error.trim());
                }
            }
                
            
        }
        
        appendErrortextVergleich(diffError);
        return diffError;
    }
    
    /**
     * Setzt die Ignorieren Elemente für den Vergleich
     * @param ignoreElem 
     */
    public void SetIgnoreElements(String ignoreElem) {
        String[] ignoreTree = ignoreElem.split("#");
        ITlvElement e = app.getBerechtigung(0).getSepDataTeil();
        for(String s: ignoreTree) {
            if(s.equals("Verzeichniseintrag")) {
                e = app.getVerzBerechtigung(0);
            } else {
                e = e.get(s);
            }
        }
        
        if(e != null) {
            e.setIgnoreDiff(true);
        }
    }
            
}
